package it.pf.access.builder;

import it.pf.access.client.ClientBase;
import it.pf.access.client.ClientBaseNew;
import it.pf.access.client.ClientCodici;
import it.pf.access.client.ClientFascicolo;
import it.pf.access.client.ClientFascicoloNew;
import it.pf.access.client.ClientPianoColturale;
import it.pf.access.client.ClientPianoColturaleNew;
import it.pf.base.ConfigSingletonDefaultSetup;
import it.pf.base.ObjectBean;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.DataSheet;
import it.pf.utils.DataUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.nio.file.InvalidPathException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.xml.sax.helpers.ParserAdapter;

import it.eng.client.servizi.*;
import it.eng.bean.aziendeModificate.xsd.*;
import it.eng.bean.codici.xsd.*;
//import it.eng.bean.fascicolo.xsd.*;
import it.eng.bean.fascicoloInterno2.xsd.*;
/**
/**
 * Scarica su file excel i cuaa delle aziende modificate dopo data parametro
 * 
 * @author paolo
 *
 */
public class ElencoCuaa extends ClientBaseNew {

	private String xlsOutputFile;
	private String dateFrom;



	/**
	 * @param args
	 * @throws InterruptedException 
	 */
	public static void main(String[] args) throws Exception {
		ElencoCuaa inst = new ElencoCuaa();
		System.out.println("************ REGIONE EMILIA-ROMAGNA ************");
		System.out.println("-- Fascicolo anagrafico aziende agricole --");
		if (inst.parseArgs(args)) {
			System.out.println("-- Acquisizione elenco cuaa di aziende con fascicolo variato dopo il "+inst.getDateFrom()+" --");
			ListProvider provider = new ServerProviderNew(inst.getDateFrom());
			provider.getDs().writeXLS(inst.getXlsOutputFile());
			System.out.println("-- Scritti "+provider.size()+" cuaa sul file "+inst.getXlsOutputFile()+" --");
		}
		System.exit(0);
	}

	public ElencoCuaa() {
		super();
	}
	
	private boolean parseArgs(String[] args) {
		if (args.length < 2) {
			System.out.println("Parametri insufficienti!");
			syntax();
			return false;
		} else {
			dateFrom = args[0];
			DataUtil dc = new DataUtil(dateFrom);
			if (!dc.verifydate()) {
				System.out.println("Data formalmente errata.");
				syntax();
				return false;
			}
			xlsOutputFile = args[1];
			if (!isValidPath(xlsOutputFile)) {
				System.out.println("Il secondo parametro deve indicare un nome di file valido completo di percorso.");
				syntax();
				return false;
			}
			if (!xlsOutputFile.endsWith(".xls") && !xlsOutputFile.endsWith(".XLS")) {
				xlsOutputFile += ".xls";
			}
			return true;
		}
	}

	private void syntax() {
		System.out.println("java -jar ElencoCuaa <data limite> <outputfile>.");
	}

	public boolean isValidPath(String path) {

		File f = new File(path);
		String parent = f.getParent();
		File dir = new File(parent);
		if (dir.isDirectory()) {
			return true;
		} else {
			return false;
		}
	}
	
	
	private boolean isDirectory(String f) {
		File x = new File(f);
		return x.isDirectory();
	}

	public String getXlsOutputFile() {
		return xlsOutputFile;
	}

	public void setXlsOutputFile(String xlsOutputFile) {
		this.xlsOutputFile = xlsOutputFile;
	}

	public String getDateFrom() {
		return dateFrom;
	}

	public void setDateFrom(String dateFrom) {
		this.dateFrom = dateFrom;
	}
	

}
