package it.pf.access.builder;

import it.pf.access.client.ClientBase;
import it.pf.access.client.ClientCodici;
import it.pf.access.client.ClientFascicolo;
import it.pf.access.client.ClientPianoColturale;
import it.pf.base.ConfigSingletonDefaultSetup;
import it.pf.base.ObjectBean;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.DataSheet;
import it.pf.utils.DataUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import it.eng.client.servizi.*;
import it.eng.bean.aziendeModificate.xsd.*;
import it.eng.bean.codici.xsd.*;
//import it.eng.bean.fascicolo.xsd.*;
import it.eng.bean.fascicoloInterno2.xsd.*;
/**
/**
 * Client fascicolo anagrafico e piano colturale.
 * Dopo una serie di richieste di parametri il programma 'gira' in batch 
 * creando una base dati MDB (oppure in alternativa Hypersonic SQL) e popolando le tabelle
 * corrispondenti ai tipi proposti dal servizio web.
 * Sfrutta entrambe i metodi del servizio web:
 * getAziendeModificate		che fornisce l'elenco dei CUAA validati dopo la data fornita come parametro
 * getFascicolo				che ritorna i dati del fascicolo del CUAA fornito come parametro
 * 
 * Le credenziali di accesso così come la scelta se MDB o H2 per l'output sono scritte nel file configOP.xml
 * che contiene anche l'indirizzo del servizio e il nome del mandato utilizzato.
 * 
 * Le scelte che si possono fare sui dialoghi iniziali consentono di:
 * - processare tutti i CUAA validati dopo una certa data
 * - acquisire l'elenco dei CUAA validati dopo una certa data e salvarlo su file excel
 * - processare tutti i CUAA presenti su un file excel fornito come input (unica colonna intestata CUAA)
 * 
 * 
 * @author paolo
 *
 */
public class MainProcess extends ClientBase {

	private File currentDir;
	private File inputDir;


	public DatiAziendaType o_azienda;
	public PersonaType o_persona;
	public UnitaAziendaleType o_unita;
	public ParticellaType o_particella;
	public ConduzioneParticellaType o_conduzione;
	public ZonaType o_zona;
	public ContrattoType o_contratto;
	public MacrousoType o_macrouso;
	public ProprietarioType o_proprietario;
	public UnitaVitataType o_uuvv;
	public VitignoType o_vitigno;
	public IscrizioneType o_iscrizione;
	public UtilizziFlat o_pc;


	/**
	 * @param args
	 * @throws InterruptedException 
	 */
	public static void main(String[] args) throws Exception {
		MainProcess inst = new MainProcess();
		System.out.println("************ REGIONE EMILIA-ROMAGNA ************");
		System.out.println("-- Fascicolo anagrafico aziende agricole --");
		inst.run();
		System.exit(0);
	}

	public MainProcess() {
		super();
	}
	
	public void run() throws Exception {
		welcome();
		String dfrom = null;
		this.setCurrentDir(new File(System.getProperty("user.dir")));
		ListProvider provider = null;
		// opzioni iniziali
		int what = getChoiceAboutCodici();
		if (what == 0) {	//richiesto aggiornamento codici
			String mdbfile = null;
			if (this.getLocalDBtype().equals("MDB")) {
				mdbfile = getMDBFile();
			} else {
				String dir = getDataDir();
				mdbfile = dir+"/"+this.getDatabasename();
			}
			if (mdbfile == null) {
				System.exit(0);
			}
			System.out.println("-- Aggiornamento tabella C_CODICI in corso ....");
			ClientCodici clientCodici = new ClientCodici();
			clientCodici.setDBfile(mdbfile);
			clientCodici.execute();
			System.out.println("-- Aggiornamento tabella C_CODICI terminato.");
			System.exit(0);
		} else {			//altrimenti scarico fascicoli
			int mainchoice = getChoiceAboutSource();
			if (mainchoice == 0) {		//da file excel
				String xlsinput = getInputFile();
				if (xlsinput == null) {
					System.exit(0);		
				} else {
					provider = new XLSProvider(xlsinput);
				}
			} else {					//da data limite
				dfrom = getDataMod();
				if (dfrom == null) {
					System.exit(0);
				}
				provider = new ServerProvider(dfrom);
				if (provider.size() == 0) {
					nomore();
					System.exit(0);
				} else {
					int listchoice = getChoiceAboutList(provider.size(), dfrom);
					if (listchoice == 1) {			//salva su file
						String xlsoutput = getOutputFile();
						if (xlsoutput == null) { 
							System.exit(0);
						} else {
							provider.getDs().writeXLS(xlsoutput);
							System.exit(0);
						}
					} else {						//prosegui con l'elaborazione
					}
				}
			}
		}
		
		//prosegue con l'elaborazione
		// richiede anno di elaborazione
		Integer anno = getAnnoPC();
		if (anno == null) {
			System.exit(9);
		}
		
		String rundir = System.getProperty("user.dir"); 
		rundir = rundir.replace('\\', '/');
		String dest = getDestinationDir();
		if (dest != null) {
			dest = dest.replace('\\', '/');
			if (dest.equalsIgnoreCase(rundir)) {
				System.out.println("La directory di destinazione non puo' coincidere con quella di partenza!");
				System.exit(9);
			}
			this.setCurrentDir(new File(dest));
			if (!dest.endsWith("/"))
				dest += "/";
			
			String destDB = dest+this.getDatabasename();
			String destFile = dest+this.getDatabasefile();
			
			setDBfile(destDB);
			
			Long verify;
			try {
				verify = CopyFile.copyFile(new File(this.getDatabasefile()), new File(destFile));
				//System.out.println("Copia del file di base, checksum:"+verify.longValue());
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
		
			System.out.println("");
			if (verify != null) {
				process(provider,anno,destFile);
			}
		}
		System.exit(0);
	}

	private void process(ListProvider provider, Integer anno, String destFile) throws Exception {
		createTables();
		boolean append = false;
		System.out.println("\nSulla base delle scelte fatte risultano "+provider.size()+" aziende da elaborare.");
		System.out.println("\n*------------INIZIO ELABORAZIONE-----------*\n");
		for (int i=0; i<provider.size(); i++) {
			DataRow dri = provider.getRow(i);
			String cuaa = dri.getColumn("CUAA").getStringValue();
			if (cuaa.length() > 0) {
				System.out.print("\nn."+ (i+1) + " - " + cuaa);
				ClientFascicolo client = new ClientFascicolo(cuaa);
				if (client.getError() == null) {
					client.setDBfile(this.getDBfile());
					if (client.insertDB()) {
						
						ClientPianoColturale clientpc = new ClientPianoColturale(cuaa,anno.intValue());
						if (clientpc.getError() == null) {
							clientpc.setDBfile(this.getDBfile());
							if (clientpc.insertDB("pianocolturale", clientpc.getPianocolturale(),this.getO_pc())) {
								System.out.println(" - elaborato con successo");
							} else {
								System.out.println(" - errori nella scrittura dei dati pianocolturale su "+destFile);
							}
						}
						
					} else
						System.out.println(" - errori nella scrittura dei dati fascicolo su "+destFile);
				} else {
					System.out.println(" ***** "+ client.getError());
				}
			}
		}
		System.out.println("*------------FINE ELABORAZIONE-----------*");
	}
	

	private void createTables() {
		String key = "";
		o_azienda = new DatiAziendaType();
		createTable("azienda", key, o_azienda);
		key = " CUAA VARCHAR(16), ID_AZIENDA INTEGER, ";
		o_unita = new UnitaAziendaleType();
		createTable("unita", key, o_unita);
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ";
		o_persona = new PersonaType();
		createTable("persone", key, o_persona);
		o_particella = new ParticellaType();
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ";
		createTable("particelle", key, o_particella);
		
		o_conduzione = new ConduzioneParticellaType();
		key = " CUAA VARCHAR(16) ,";
		createTable("conduzioni", key, o_conduzione);
		o_zona = new ZonaType();
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ID_PARTICELLA INTEGER, ";
		createTable("zone", key, o_zona);
		o_contratto = new ContrattoType();
		createTable("contratti", key, o_contratto);
		o_macrouso = new MacrousoType();
		createTable("macrousi", key, o_macrouso);
		o_proprietario = new ProprietarioType();
		createTable("proprietari", key, o_proprietario);
		o_uuvv = new UnitaVitataType();
		createTable("unita_vitate", key, o_uuvv);
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ID_PARTICELLA INTEGER, UNAR INTEGER, ";
		o_vitigno = new VitignoType();
		createTable("vitigni", key, o_vitigno);
		o_iscrizione = new IscrizioneType();
		createTable("iscrizioni_uuvv", key, o_iscrizione);
		
		String keypc = "CUAA VARCHAR(16), ";
		o_pc = new UtilizziFlat();
		createTable("pianocolturale", keypc, o_pc);
	}

	private void welcome() throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException {
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		//Configuration config = new Configuration();
		String msg = "Il programma consente di acquisire i dati del fascicolo anagrafico regionale\n";
		msg += "denominato \"Anagrafe delle Aziende Agricole\" relativamente ai Codici Fiscali dei Vostri soci.\n";
		msg += "Il risultato sara' memorizzato nel file "+this.getDatabasefile();
		msg += " nella directory di destinazione prescelta e sara' generato dall'elenco delle aziende da elaborare.\n";
		msg += "L'elenco può essere un file excel da voi predisposto\n";
		msg += "oppure può essere ottenuto come lista delle aziende validate dopo una data da voi impostata.\n";
		msg += "E' inoltre possibile aggiornare la tabella delle codifiche utilizzate (opzione seguente).\n";
		msg += "Premete 'OK' per proseguire nell'elaborazione o 'CANCEL' per terminare.";
		Object[] options = {"OK","CANCEL"};
		int rc = JOptionPane.showOptionDialog(new JFrame(), msg, "Acquisizione dati da fascicolo anagrafico RER.", JOptionPane.DEFAULT_OPTION,
				JOptionPane.INFORMATION_MESSAGE,null,options,options[0]);
		if (rc != 0) {
			System.out.println("Termine programma scelto dall'utente.");
			System.exit(0);
		}
		
	}
 
	private void nomore() throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException {
		UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		//Configuration config = new Configuration();
		String msg = "La lista dei cuaa risultante dai parametri impostati e' vuota\n";
		msg += "Riprovare con opzioni diverse.\n";
		Object[] options = {"OK","CANCEL"};
		JOptionPane.showMessageDialog(new JFrame(), msg);
		System.exit(0);
	}

	private int getChoiceAboutCodici() {
		Object[] options = {
				"Aggiornare codifiche","Scaricare fascicoli CUAA"
		};
		String msg = "Desideri:\n";
		int n = JOptionPane.showOptionDialog(new JFrame(),msg,
				"Selezione della fonte",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE,null,
				options,options[1]);
		return n;
	}

	private int getChoiceAboutSource() {
		Object[] options = {
				"Da file excel","Da data limite"
		};
		String msg = "Indica qual'è la fonte dei CUAA da elaborare:\n";
		msg += "se da un file excel con unica colonna intestata 'CUAA'\n";
		msg += "ovvero dalla data limite di validazione.";
		int n = JOptionPane.showOptionDialog(new JFrame(),msg,
				"Selezione della fonte",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE,null,
				options,options[1]);
		return n;
	}
	
	 
	private int getChoiceAboutList(int ncuaa, String dfrom) {
		Object[] options = {
				"Prosegui l'elaborazione","Salva l'elenco su file"
		};
		String msg = "Sono stati recuperati "+ncuaa+" CUAA validati dal "+dfrom+":\n";
		msg += "vuoi procedere con l'elaborazione o salvare la lista in un file?\n";
		int n = JOptionPane.showOptionDialog(new JFrame(),msg,
				"Prosegui/Salva su file",
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE,null,
				options,options[0]);
		return n;
	}

	private String getDestinationDir() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        if (this.inputDir != null) {
        	fc.setCurrentDirectory(this.inputDir);
        } else {
        	fc.setCurrentDirectory(this.getCurrentDir());
        }
        int returnVal = fc.showDialog(new JFrame(),"Scegli la directory di destinazione");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            return file.getAbsolutePath();
        } else {
            return null;
        }
	}

	private String getInputFile() {
		JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new XLSFilter());
        fc.setAcceptAllFileFilterUsed(false);
        fc.setCurrentDirectory(this.getCurrentDir());
        int returnVal = fc.showDialog(new JFrame(),"Scegli il file excel con i CUAA da elaborare");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            inputDir = this.getDirFromFile(file);
             return file.getAbsolutePath();
        } else {
            return null;
        }
	}

	private File getDirFromFile(File in) {
		if (in != null) {
			if (in.isDirectory()) {
				return in;		//already a directory
			}
			String abs = in.getAbsolutePath();
			String sep = "/";
			if (System.getProperty("os.name").startsWith("W")) {
				sep = "\\";
			}
			if (abs.endsWith(sep)) {	
				return null;
			} else {
				int p = abs.lastIndexOf(sep);
				String fd = abs.substring(0, p);
				File dir = new File(fd);
				if (dir.isDirectory()) {
					return dir;
				} else {
					return null;
				}
			}
		} else {
			return null;
		}
	}

	private String getOutputFile() {
		JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new XLSFilter());
        fc.setAcceptAllFileFilterUsed(false);
        if (this.inputDir != null) {
        	fc.setCurrentDirectory(this.inputDir);
        } else {
        	fc.setCurrentDirectory(this.getCurrentDir());
        }
        int returnVal = fc.showDialog(new JFrame(),"Scegli il file excel su cui salvare i CUAA estratti");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            String fout = file.getAbsolutePath();
            if (!fout.endsWith(".xls")) {
            	fout += ".xls";
            }
            return fout;
        } else {
            return null;
        }
	}

	private String getMDBFile() {
		JFileChooser fc = new JFileChooser();
		fc.addChoosableFileFilter(new MDBFilter());
        fc.setAcceptAllFileFilterUsed(false);
        if (this.inputDir != null) {
        	fc.setCurrentDirectory(this.inputDir);
        } else {
        	fc.setCurrentDirectory(this.getCurrentDir());
        }
        int returnVal = fc.showDialog(new JFrame(),"Scegli il file MDB su cui aggiornare la tabella C_CODICI");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            String fout = file.getAbsolutePath();
            if (!fout.endsWith(".mdb")) {
            	fout += ".mdb";
            }
            return fout;
        } else {
            return null;
        }
	}

	// per H2 chiedo la directory
	private String getDataDir() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        if (this.inputDir != null) {
        	fc.setCurrentDirectory(this.inputDir);
        } else {
        	fc.setCurrentDirectory(this.getCurrentDir());
        }
        int returnVal = fc.showDialog(new JFrame(),"Scegli la directory del DB da aggiornare");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            return file.getAbsolutePath();
        } else {
            return null;
        }
	}

	private Integer getAnnoPC() {
		Calendar now = Calendar.getInstance();
		int ac = now.get(Calendar.YEAR);
		String annocorrente = (new Integer(ac)).toString();
		String a = (String)JOptionPane.showInputDialog(new JFrame(),"Anno di riferimento Piano Colturale:","Acquisizione dati da fascicolo anagrafico RER.",
                JOptionPane.PLAIN_MESSAGE,null,null,annocorrente);
		if (a == null) {
			return null;
		}
		Integer anno = null;
		try {
			anno = new Integer(a);
		} catch (NumberFormatException nfe) {
			nfe.printStackTrace();
			return null;
		}
		return anno;
	}

	private String getDataMod() {
		DataUtil du = new DataUtil();
		du.addDaysToDate(-7);
		String datamod = du.getDataForMap("/");
		String a = (String)JOptionPane.showInputDialog(new JFrame(),"Aziende modificate dal (gg/mm/aaaa):","Acquisizione dati da fascicolo anagrafico RER.",
                JOptionPane.PLAIN_MESSAGE,null,null,datamod);
		if (a == null) {
			return null;
		}
		DataUtil dc = new DataUtil(a);
		if (!dc.verifydate()) {
			System.out.println("****** Data non valida");
			return null;
		} else {
			return a;
		}
	}

	public File getCurrentDir() {
		return currentDir;
	}

	public void setCurrentDir(File currentDir) {
		this.currentDir = currentDir;
	}

	public DatiAziendaType getO_azienda() {
		return o_azienda;
	}

	public void setO_azienda(DatiAziendaType oAzienda) {
		o_azienda = oAzienda;
	}

	public ParticellaType getO_particella() {
		return o_particella;
	}

	public void setO_particella(ParticellaType oParticella) {
		o_particella = oParticella;
	}

	public ConduzioneParticellaType getO_conduzione() {
		return o_conduzione;
	}

	public void setO_conduzione(ConduzioneParticellaType oConduzione) {
		o_conduzione = oConduzione;
	}

	public ZonaType getO_zona() {
		return o_zona;
	}

	public void setO_zona(ZonaType oZona) {
		o_zona = oZona;
	}

	public ProprietarioType getO_proprietario() {
		return o_proprietario;
	}

	public void setO_proprietario(ProprietarioType oProprietario) {
		o_proprietario = oProprietario;
	}

	public MacrousoType getO_macrouso() {
		return o_macrouso;
	}

	public void setO_macrouso(MacrousoType oMacrouso) {
		o_macrouso = oMacrouso;
	}

	public PersonaType getO_persona() {
		return o_persona;
	}

	public void setO_persona(PersonaType oPersona) {
		o_persona = oPersona;
	}

	public UnitaAziendaleType getO_unita() {
		return o_unita;
	}

	public void setO_unita(UnitaAziendaleType oUnita) {
		o_unita = oUnita;
	}

	public UtilizziFlat getO_pc() {
		return o_pc;
	}

	public void setO_pc(UtilizziFlat oPc) {
		o_pc = oPc;
	}


}
