package it.pf.access.builder;

import it.pf.access.client.ClientBase;
import it.pf.access.client.ClientBaseNew;
import it.pf.access.client.ClientCodici;
import it.pf.access.client.ClientFascicolo;
import it.pf.access.client.ClientFascicoloNew;
import it.pf.access.client.ClientPianoColturale;
import it.pf.access.client.ClientPianoColturaleNew;
import it.pf.base.ConfigSingletonDefaultSetup;
import it.pf.base.ObjectBean;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.DataSheet;
import it.pf.utils.DataUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Properties;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.xml.sax.helpers.ParserAdapter;

import it.eng.client.servizi.*;
import it.eng.bean.aziendeModificate.xsd.*;
import it.eng.bean.codici.xsd.*;
//import it.eng.bean.fascicolo.xsd.*;
import it.eng.bean.fascicoloInterno2.xsd.*;
/**
/**
 * Client fascicolo anagrafico e piano colturale.
 * Dopo una serie di richieste di parametri il programma 'gira' in batch 
 * creando una base dati MDB (oppure in alternativa Hypersonic SQL) e popolando le tabelle
 * corrispondenti ai tipi proposti dal servizio web.
 * Sfrutta entrambe i metodi del servizio web:
 * getAziendeModificate		che fornisce l'elenco dei CUAA validati dopo la data fornita come parametro
 * getFascicolo				che ritorna i dati del fascicolo del CUAA fornito come parametro
 * 
 * Le credenziali di accesso così come la scelta se MDB o H2 per l'output sono scritte nel file configOP.xml
 * che contiene anche l'indirizzo del servizio e il nome del mandato utilizzato.
 * 
 * Le scelte che si possono fare sui dialoghi iniziali consentono di:
 * - processare tutti i CUAA validati dopo una certa data
 * - acquisire l'elenco dei CUAA validati dopo una certa data e salvarlo su file excel
 * - processare tutti i CUAA presenti su un file excel fornito come input (unica colonna intestata CUAA)
 * 
 * 
 * @author paolo
 *
 */
public class MainProcessNew extends ClientBaseNew {

	private File currentDir;
	private File inputDir;

	private String xlsInputFile;
	private String annoPC;

	public DatiAziendaType o_azienda;
	public PersonaType o_persona;
	public UnitaAziendaleType o_unita;
	public ParticellaType o_particella;
	public ConduzioneParticellaType o_conduzione;
	public ZonaType o_zona;
	public ContrattoType o_contratto;
	public MacrousoType o_macrouso;
	public ProprietarioType o_proprietario;
	public UnitaVitataType o_uuvv;
	public VitignoType o_vitigno;
	public IscrizioneType o_iscrizione;
	public UtilizziFlat o_pc;


	/**
	 * @param args
	 * @throws InterruptedException 
	 */
	public static void main(String[] args) throws Exception {
		MainProcessNew inst = new MainProcessNew();
		System.out.println("************ REGIONE EMILIA-ROMAGNA ************");
		System.out.println("-- Fascicolo anagrafico aziende agricole --");
		if (inst.parseArgs(args)) {
			ListProvider provider = new XLSProvider(inst.getXlsInputFile());
			inst.process(provider, Integer.parseInt(inst.getAnnoPC()), inst.getDestinationDir());
		}
		System.exit(0);
	}

	public MainProcessNew() {
		super();
	}
	
	private boolean parseArgs(String[] args) {
		if (args.length < 3) {
			System.out.println("Parametri insufficienti!");
			syntax();
			return false;
		} else {
			this.setXlsInputFile(args[0]);
			String ddir = args[1];
			if (!ddir.endsWith("/")) {
				ddir += "/";
			}
			this.setDestinationDir(ddir);
			this.setAnnoPC(args[2]);
			if (!isFile(this.getXlsInputFile())) {
				System.out.println("Il primo parametro deve essere un file.");
				syntax();
				return false;
			}
			if (!isDirectory(this.getDestinationDir())) {
				System.out.println("Il secondo parametro deve essere una directory.");
				syntax();
				return false;
			}
			try {
				int anno = Integer.parseInt(this.getAnnoPC());
			} catch (NumberFormatException nfe) {
				System.out.println("Il terzo parametro deve essere un'anno di 4 cifre.");
				syntax();
				return false;				
			}
			return true;
		}
	}

	private void syntax() {
		System.out.println("java -jar ClientFascicolo <xlsinputfile> <destination directory> <anno PC (4 cifre)>.");
	}
	
	private boolean isFile(String f) {
		File x = new File(f);
		return x.isFile();
	}
	
	private boolean isDirectory(String f) {
		File x = new File(f);
		return x.isDirectory();
	}
	

	private void process(ListProvider provider, Integer anno, String destDir) throws Exception {
		createTables();
		boolean append = false;
		System.out.println("\nSulla base delle scelte fatte risultano "+provider.size()+" aziende da elaborare.");
		System.out.println("\n*------------INIZIO ELABORAZIONE-----------*\n");
		for (int i=0; i<provider.size(); i++) {
			DataRow dri = provider.getRow(i);
			String cuaa = dri.getColumn("CUAA").getStringValue();
			if (cuaa.length() > 0) {
				System.out.print("\nn."+ (i+1) + " - " + cuaa);
				ClientFascicoloNew client = new ClientFascicoloNew(cuaa, destDir);
				if (client.getError() == null) {
					if (client.insertDB()) {
						
						ClientPianoColturaleNew clientpc = new ClientPianoColturaleNew(cuaa,anno.intValue(),destDir);
						if (clientpc.getError() == null) {
							if (clientpc.insertDB("pianocolturale", clientpc.getPianocolturale(),this.getO_pc())) {
								System.out.println(" - elaborato con successo");
							} else {
								System.out.println(" - errori nella scrittura dei dati pianocolturale su "+destDir);
							}
						}
						
					} else
						System.out.println(" - errori nella scrittura dei dati fascicolo su "+destDir);
				} else {
					System.out.println(" ***** "+ client.getError());
				}
			}
		}
		System.out.println("*------------FINE ELABORAZIONE-----------*");
	}
	

	private void createTables() {
		String key = "";
		o_azienda = new DatiAziendaType();
		createTable("azienda", key, o_azienda);
		key = " CUAA VARCHAR(16), ID_AZIENDA INTEGER, ";
		o_unita = new UnitaAziendaleType();
		createTable("unita", key, o_unita);
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ";
		o_persona = new PersonaType();
		createTable("persone", key, o_persona);
		o_particella = new ParticellaType();
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ";
		createTable("particelle", key, o_particella);
		
		o_conduzione = new ConduzioneParticellaType();
		key = " CUAA VARCHAR(16) ,";
		createTable("conduzioni", key, o_conduzione);
		o_zona = new ZonaType();
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ID_PARTICELLA INTEGER, ";
		createTable("zone", key, o_zona);
		o_contratto = new ContrattoType();
		createTable("contratti", key, o_contratto);
		o_macrouso = new MacrousoType();
		createTable("macrousi", key, o_macrouso);
		o_proprietario = new ProprietarioType();
		createTable("proprietari", key, o_proprietario);
		o_uuvv = new UnitaVitataType();
		createTable("unita_vitate", key, o_uuvv);
		key = " CUAA VARCHAR(16) , ID_AZIENDA INTEGER, ID_PARTICELLA INTEGER, UNAR INTEGER, ";
		o_vitigno = new VitignoType();
		createTable("vitigni", key, o_vitigno);
		o_iscrizione = new IscrizioneType();
		createTable("iscrizioni_uuvv", key, o_iscrizione);
		
		String keypc = "CUAA VARCHAR(16), ";
		o_pc = new UtilizziFlat();
		createTable("pianocolturale", keypc, o_pc);
	}


	public File getCurrentDir() {
		return currentDir;
	}

	public void setCurrentDir(File currentDir) {
		this.currentDir = currentDir;
	}

	public DatiAziendaType getO_azienda() {
		return o_azienda;
	}

	public void setO_azienda(DatiAziendaType oAzienda) {
		o_azienda = oAzienda;
	}

	public ParticellaType getO_particella() {
		return o_particella;
	}

	public void setO_particella(ParticellaType oParticella) {
		o_particella = oParticella;
	}

	public ConduzioneParticellaType getO_conduzione() {
		return o_conduzione;
	}

	public void setO_conduzione(ConduzioneParticellaType oConduzione) {
		o_conduzione = oConduzione;
	}

	public ZonaType getO_zona() {
		return o_zona;
	}

	public void setO_zona(ZonaType oZona) {
		o_zona = oZona;
	}

	public ProprietarioType getO_proprietario() {
		return o_proprietario;
	}

	public void setO_proprietario(ProprietarioType oProprietario) {
		o_proprietario = oProprietario;
	}

	public MacrousoType getO_macrouso() {
		return o_macrouso;
	}

	public void setO_macrouso(MacrousoType oMacrouso) {
		o_macrouso = oMacrouso;
	}

	public PersonaType getO_persona() {
		return o_persona;
	}

	public void setO_persona(PersonaType oPersona) {
		o_persona = oPersona;
	}

	public UnitaAziendaleType getO_unita() {
		return o_unita;
	}

	public void setO_unita(UnitaAziendaleType oUnita) {
		o_unita = oUnita;
	}

	public UtilizziFlat getO_pc() {
		return o_pc;
	}

	public void setO_pc(UtilizziFlat oPc) {
		o_pc = oPc;
	}

	public String getXlsInputFile() {
		return xlsInputFile;
	}

	public void setXlsInputFile(String xlsInputFile) {
		this.xlsInputFile = xlsInputFile;
	}

	public String getAnnoPC() {
		return annoPC;
	}

	public void setAnnoPC(String annoPC) {
		this.annoPC = annoPC;
	}



}
