package it.pf.access.builder;

import it.pf.base.ConfigSingleton;
import it.pf.base.ObjectBean;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.MessageDigest;
import java.util.Properties;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;

import org.apache.ws.security.WSPasswordCallback;


/**
 * Chiamata dalla classe org.apache.ws.axis.security.WSDoAllSender. Invocata come richiesto nel file client-deploy.wsdd
 * La classe WSDoAllSender costruisce uno UserToken con un dato utente da inserire nel messaggio SOAP. Dopodich
 * invoca la suddetta classe che setta nella bust la password opportuna corrispondente all'utente passato.
 * Tale password viene letta da un file di configurazione. la pasword viene cifrata se nel file client-deploy.wsdd
 * il passwordType=passordDigest
 */
public class PWCallbackClient implements CallbackHandler {
    public void handle(Callback[] callbacks) throws IOException, UnsupportedCallbackException {
    	
    	String user;
    	String password;
        for (int i = 0; i < callbacks.length; i++) {
            if (callbacks[i] instanceof WSPasswordCallback) {
                WSPasswordCallback pc = (WSPasswordCallback)callbacks[i];
                ObjectBean config = ConfigSingleton.getResources();
        		ObjectBean ob = config.getFilteredAttribute("CONFIG:PARAMETERS:PARAMETER", "name", "userid");
        		user = ob.getAttributeValueAsString("value");
        		ob = config.getFilteredAttribute("CONFIG:PARAMETERS:PARAMETER", "name", "password");
        		password = ob.getAttributeValueAsString("value");
                if (user.equals(pc.getIdentifer())) {
                   	pc.setPassword(encriptSHA1Hex(password));   
                }
            } else {
                throw new UnsupportedCallbackException(callbacks[i], "Unrecognized Callback!!!!!!!!!");
            }
        }
        
    }
    
	/**
     * esegue il crypting della password 
     * L'algoritmo utilizzato la funzione hash SHA-1 e trasformando ogni carattere dell'hashing in esadecimale
     * @param password
     */
 	public static synchronized String encriptSHA1Hex(String toEncript)  {

 		try {
 			    byte[] passwordB = utf8decode(toEncript);
 				MessageDigest messageDigester = MessageDigest.getInstance("SHA-1");

 				messageDigester.reset();
 				messageDigester.update(passwordB);

 				byte[] pwdEncB=messageDigester.digest();
 				String pwdEnc1=new String(pwdEncB);
 				String pwdEnc="";
 				
 				//trasforma la stringa ottenuta trasformando ogni carattere in esadecimale
 				for(int i=0; i<pwdEncB.length;i++){
 				    pwdEnc+=byteToHexWithZero(pwdEncB[i]);
 				}
 				return pwdEnc;
 			} catch (java.security.NoSuchAlgorithmException e) {
 				e.printStackTrace();
 			}
 			return null;
 	}
 	
     private static byte[] utf8decode(String input) {
 		// UTF-8 enc
 		byte[] ret = null;
 		try {
 			ret = input.getBytes("UTF-8");
 		} catch (java.io.UnsupportedEncodingException e) {
 			e.printStackTrace();
 		}
 		return ret;
 	}
     
     /**
 	 * Ritorna una stringa con la rappresentazione esadecimale (con 2 caratteri) del byte 
 	 * @param b
 	 * @return
 	 */
 	private static String byteToHexWithZero(byte b){
 	    int i = b & 0xFF;
 	    String s = Integer.toHexString(i);
 	    if(s.length()==1) s="0"+s;
 	    return s.toUpperCase();
 	}

}
