package it.eng.commons.db;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import it.eng.fs.constant.DBConstant;
import it.eng.fs.constant.LXConstants;

public class ConnectionManager{
	
	private Connection connection=null;
	
	public ConnectionManager(){
		try {
			loadDriver();
			this.setConnection(this.newConnection());
			
		} catch (ConnectionManagerException e) {
			e.printStackTrace();
		}
		
	}
	
	private void setConnection(Connection _con){
		this.connection=_con;
	}
	
	private Connection getConnection(){
		return this.connection;
	}
	
	private Connection newConnection() throws ConnectionManagerException {

		try {
			System.out.println("dburl: "+LXConstants.dbUrl+ " user: "+LXConstants.dbLogin+ "password: "+LXConstants.dbPassword);
			connection = DriverManager.getConnection(LXConstants.dbUrl, LXConstants.dbLogin, LXConstants.dbPassword);
			connection.setAutoCommit(false);
			
		} catch (SQLException e) {
			throw new ConnectionManagerException("Impossibile stabilire una connessione col database: " + e.getMessage());
		}
		return connection;
	}
	
	
	public void rollback() throws SQLException {
		if (getConnection() != null) 
			getConnection().rollback();
	}
	

	public void commit(Connection connection) throws SQLException {
		if (getConnection() != null) {
			getConnection().commit();
		}

	}
	
	public void close() throws SQLException{
		if (getConnection() != null) {
			getConnection().close();
			setConnection(null);
		}

	}
	
	public PreparedStatement prepareStatement(String sql) throws SQLException {
		if (getConnection() != null) {
			return getConnection().prepareStatement(sql);
		}

		return null;
	}
	
	public CallableStatement prepareCall(String sql) throws SQLException {
		if (getConnection() != null) {
			return getConnection().prepareCall(sql);
		}

		return null;
	}
	
	private void loadDriver() throws ConnectionManagerException {
		try {			
			Class.forName(LXConstants.dbDriver);
			System.out.println("Driver caricato correttamente");
		} catch (Exception e) {
			throw new ConnectionManagerException("Errore nel caricamento dei driver: " + e.getMessage());
		}
	}
	
	
	private boolean isValidConnection() {
		CallableStatement cs = null;
		boolean isValid = false;
		try {
			cs = getConnection().prepareCall("SELECT SYSDATE FROM DUAL");
			cs.execute();
			isValid = true;
		} catch (SQLException ignored) {
		} finally {
			try {
				cs.close();
			} catch (SQLException e) {
			}
			cs = null;
		}
		return isValid;
	}
	
	public String getCodAzienda(String cuua)throws ConnectionManagerException{
		String cod_azienda=null;
		if(getConnection()!=null){			
			try {
				PreparedStatement ps = getConnection().prepareStatement( DBConstant.CODICE_AZIENDA);
				ps.setString(1, cuua);
				if(ps!=null){
					ResultSet res = ps.executeQuery();
					if(res !=null){
						  while (res.next()){
							//   int numeroColonne = res.getMetaData().getColumnCount();
						      // for ( int i = 1 ; i <= numeroColonne ; i++ ) 
						        //  System.out.println( "COLONNA " + i + " = " + res.getObject(i) );
							  cod_azienda=res.getString(1);
							  System.out.println("cod_azienda: "+cod_azienda);
						  }
						  res.close();
						  ps.close();						  
						  return cod_azienda;
					}
				}

			} catch (SQLException e) {
				throw new ConnectionManagerException("Errore nel recupero del codice Azienda: " + e.getMessage());
			}

		}else
			throw new ConnectionManagerException("Impossibile effettuare la ricerca del codice Azienda, la connection e null");
		
		return cod_azienda;
		
	}
	
	public void deleteRecord(String cod_azienda)throws ConnectionManagerException{		
		if(getConnection()!=null){
			try {
				PreparedStatement ps = getConnection().prepareStatement( DBConstant.DELETE_MACROUSO);
				ps.setString(1, cod_azienda);
		
				if(ps!=null){
					int record = ps.executeUpdate();
					getConnection().commit();
					ps.close();						  
				}
			}
			catch (SQLException e){
				throw new ConnectionManagerException("Errore nel recupero del codice Azienda: " + e.getMessage());
			}
		}
	}
	
	public void updateDate(String cod_azienda)throws ConnectionManagerException{
		if(getConnection()!=null){
			try {
				PreparedStatement ps = getConnection().prepareStatement( DBConstant.UPDATE_DATE);
				ps.setString(1, cod_azienda);
				if(ps!=null){
					int record = ps.executeUpdate();
					getConnection().commit();
					ps.close();		
					System.out.println("Record modificati: "+record);
				}
			}
			catch (SQLException e){
				throw new ConnectionManagerException("Errore nel recupero del codice Azienda: " + e.getMessage());
			}
		}
		
	}
	
	public void insert()throws ConnectionManagerException{
		if(getConnection()!=null){
			try {
				/*PreparedStatement ps = getConnection().prepareStatement( DBConstant.INSERT_MACROUSO);

					ps.setInt(1,9999);
					ps.setInt(2,9999);
					ps.setInt(3,9999);
					ps.setString(4,"cod_macrouso");
					ps.setInt(5,9999);
					ps.setDate(6, "to_date()");
					ps.setString(7,"fonte");
					ps.setString(8,"id_utente");
					ps.setString(9,null);

	
				int t=ps.executeUpdate();
				getConnection().commit();
				ps.close();		
				System.out.println("Record inseriti: ");*/
				
				double d = 3.3; 
				BigDecimal bg = new BigDecimal(d); 
				bg = bg.setScale(2, BigDecimal.ROUND_DOWN); 
				
				System.out.println (bg.doubleValue()); 
				
				Statement stmt=getConnection().createStatement();
				String sql= "INSERT INTO T_MACROUSO ( ID_AZIENDA, ID_PARTICELLA,PROGR, COD_MACROUSO, SUP_MACROUSO, DT_INIZIO_MACROUSO, FONTE, ID_UTENTE, DT_VARIAZIONE) values(30611,122728,2,'cod_mac',"+bg.doubleValue()+",TO_DATE ('31-Dic-1964', 'DD-Mon-YYYY'),'fonte','id_utente',TO_DATE ('31-Dic-1964 12:21:15', 'DD-Mon-YYYY HH24:MI:SS'))";
				System.out.println("pippo");
				int n=stmt.executeUpdate(sql);
				getConnection().commit();
				System.out.println("n: "+n);
				

			}
			catch (SQLException e){
				throw new ConnectionManagerException("Errore nell' inserimento dei macrouso: " + e.getMessage());
			}
		}
		
	}
	
	
}
