package it.eng.fs.services;

import java.rmi.Remote;

import javax.xml.rpc.ServiceException;

import org.apache.axis.AxisFault;
import org.apache.axis.ConfigurationException;
import org.apache.axis.EngineConfiguration;
import org.apache.axis.client.Call;
import org.apache.axis.client.Stub;
import org.apache.axis.configuration.FileProvider;
import org.apache.ws.security.handler.WSHandlerConstants;

import it.eng.commons.db.ConnectionManager;
import it.eng.commons.db.ConnectionManagerException;
import it.eng.fs.constant.LXConstants;
import it.eng.fs.exeption.CallException;
import it.eng.fs.exeption.ConfigException;
import it.eng.fs.stub.GetPianoColturale;
import it.eng.fs.stub.GetPianoColturaleResponse;
import it.eng.fs.stub.GetPianoColturaleServiceLocator;
import it.eng.fs.stub.GetPianoColturaleSoapBindingStub;
import it.eng.fs.stub.GetPianoColturale_PortType;
import it.eng.fs.stub.ISWSAzienda;
import it.eng.fs.stub.ISWSResponse;
import it.eng.fs.util.handlers.PWCallback;

public class PianoColturale {
	
	private String portName;
	private String addressPoint;
	private int timeout;
	private EngineConfiguration config=null;
	private GetPianoColturaleServiceLocator locator=null;
	private Remote remote=null;
	private Stub axisPort=null;
	private GetPianoColturaleSoapBindingStub stub=null;
	
	
	public PianoColturale(){}
	
	
	public PianoColturale(String _portName, String _addressPoint, int _timeout){
		this.portName=_portName;
		this.addressPoint=_addressPoint;
		this.timeout=_timeout;
	}

	public PianoColturale(String _addressPoint){
		this(LXConstants.PORT_NAME_DEFAULT,_addressPoint,LXConstants.TIMEOUT);
	}
	
	public void configure()throws ConfigException{
		try {
			//config = new FileProvider(LXConstants.BASE_PATH,LXConstants.FILE_NAME_WSDD_RELATIVE);
			config = new FileProvider(LXConstants.FILE_NAME_WSDD_ABSOLUTE);
			
			locator = new GetPianoColturaleServiceLocator(config);
			locator.setEndpointAddress(this.getPortName(), this.getAddressPoint());	

			
			//remote = locator.getPort(GetPianoColturale_PortType.class);
			remote=locator.getGetPianoColturale();
			
			axisPort = (Stub)remote;
			axisPort._setProperty(WSHandlerConstants.USER, LXConstants.USER_1);
			axisPort._setProperty(WSHandlerConstants.PW_CALLBACK_REF, new PWCallback());			
			axisPort._setProperty(WSHandlerConstants.ACTION, WSHandlerConstants.USERNAME_TOKEN);
			
			stub = (GetPianoColturaleSoapBindingStub)axisPort;			
			stub.setTimeout(this.getTimeOut());	
			
			stub.setMaintainSession(false);
			
		
			
		} //catch (ConfigurationException e) {
			//System.out.println("PianoColturale.configure(): ConfigurationException: "+e.toString());
			//throw new ConfigException("PianoColturale.configure(): ConfigurationException: "+e.toString());
	
		//}
		catch (ServiceException e) {
			//System.out.println("PianoColturale.configure(): ServiceException: "+e.toString());
			throw new ConfigException("PianoColturale.configure(): ConfigurationException: "+e.toString());
			
		}catch (Exception e){
			System.out.println("PianoColturale.configure(): Exception: "+e.toString());
			throw new ConfigException("PianoColturale.configure(): Exception: "+e.toString());
		}
		
	}
	
	
	public GetPianoColturaleResponse execute(GetPianoColturale request) throws CallException{
		GetPianoColturaleResponse response=null;
		int catch_flag=1;
		String catch_value="";
		try {
			stub.setTimeout(this.getTimeOut());
			response = stub.getPianoColturale(request);			

		} catch (AxisFault a) {
			//System.out.println("PianoColturale.execute: AxisFault: "+a.toString());
			catch_flag=-1;
			catch_value="GetPianoColturaleResponse.execute: "+a.toString();
			try {
				stampaSoapMsg();
			} catch (AxisFault e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		catch (NullPointerException n){
			//System.out.println("PianoColturale.execute(): NullPointerException: "+n.toString());
			catch_flag=-2;
			catch_value="PianoColturale.execute(): "+n.toString();
		}catch(Exception e){
			//System.out.println("PianoColturale.execute: Exception: "+e.toString());
			catch_flag=-3;
			catch_value="PianoColturale.execute(): "+e.toString();
		}	

		if(catch_flag <0)
			throw new CallException(catch_value);
		else
			return response;
	}
	
	public int deleteMacroUsi( String cuua,ISWSAzienda azienda){		
		int res=it.eng.fs.util.FormalCheck.checkCuua(cuua, azienda);
		if( res>=0) {
			ConnectionManager connMan= new ConnectionManager();
			try {
				String cod_azienda=connMan.getCodAzienda(cuua);
				if(cod_azienda !=null){
					
					
				}
			} catch (ConnectionManagerException e) {
				e.printStackTrace();
				return -1;
			}
			return 1;
				//cancellare tutti i macrousi sulla tabella t_macrouso a partire dal codice azienda
				
		}
		
		else
			return res;
			
	}
	
	
	public void stampaSoapMsg() throws AxisFault{
		Call call=stub._getCall();
	System.out.println("<REQUEST>"+call.getMessageContext().getRequestMessage().getSOAPPartAsString()+"</REQUEST>");
	System.out.println("<RESPONSE>"+call.getMessageContext().getResponseMessage().getSOAPPartAsString()+"/<RESPONSE>");
		
	}
	
	public String getRequestSOAP(){
		Call call=stub._getCall();
		try {
			return call.getMessageContext().getRequestMessage().getSOAPPartAsString();
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "Errore recupero request: "+e.toString();
		}
	}
	
	public String getResponseSOAP(){
		Call call=stub._getCall();
		try {
			return call.getMessageContext().getResponseMessage().getSOAPPartAsString();
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "Errore recupero response: "+e.toString();
		}
	}
	
	public void setAddressPoint(String _addressPoint){
		this.addressPoint=_addressPoint;
	}
	public String getAddressPoint(){
		return this.addressPoint;
	}
	
	public void setPortName(String _portName){
		this.portName=_portName;
	}
	public String getPortName(){
		return this.portName;
	}
	
	public void setTimeOut(int _timeout){
		this.timeout=_timeout;
	}
	public int getTimeOut(){
		return this.timeout;
	}

}
