package it.eng.handlers;

import java.util.Iterator;

import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;

import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;

import org.apache.axis.AxisFault;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.message.SOAPHeaderElement;

/**
 * 
 * @author Marcelli_M
 * Handler che viene chiamato dal client quando arriva la risposta SOAP dal server.
 * Viene eseguito prima di deserializzare.
 * In particolare tale handler estrapola dalla busta SOAP di risposta gli eventuali header e li processa.
 * Questo per evitare che vi siano header non processati aventi la propriet MustUnderstand settata ad 1
 * (tale propriet indica che l'header deve essere obbligatoriamente processato).
 * Nel caso dei WS del mandato sviluppati con axis2, nell'header della  risposta c' una busta WS Security 
 * avente la propriet MustUnderstand=1; se tale header non viene processata, 
 * le classi di axis1 lanciano eccezione. 
 *
 */

public class ResponseHandler extends BasicHandler{

	public void invoke(MessageContext message) throws AxisFault {
		//System.out.println("Response");
		try {
			SOAPEnvelope envelope = this.getEnvelope(message);
			SOAPHeader header = envelope.getHeader();
			Iterator headers= header.getChildElements();
			while (headers.hasNext()){
				((SOAPHeaderElement)headers.next()).setProcessed(true);
			}
		} catch (SOAPException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private SOAPEnvelope getEnvelope(org.apache.axis.MessageContext msgContext) throws SOAPException{
		SOAPMessage soapMessage = msgContext.getMessage();
		SOAPPart  soapPart = (SOAPPart) soapMessage.getSOAPPart();
		SOAPEnvelope soapEnvelope = (SOAPEnvelope) soapPart.getEnvelope();
		return soapEnvelope;
	}

}
