package it.pf.base;

import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
/**
 * La classe <code>ObjectBeanContentHandler</code> implementa l'interfaccia
 * standard di un parser SAX. Questo parser ricostruisce un oggetto di tipo
 * <code>ObjectBean</code> da uno stream XML.
 * 
 * @author Fabbri_P
 *
 */
public class ObjectBeanContentHandler extends DefaultHandler {
	/**
	 * Costruisce ed inizializza il parser.
	 */
	public ObjectBeanContentHandler() {
		_objectBeans = new Stack();
		_objectBean = null;
		_parentBean = null;
		_characters = new String();
	}

    public synchronized ObjectBean getObjectBean() {
        return _objectBean;
    }
	
	/**
	 * Gestisce l'evento di inizio stream XML.
	 */
	public void startDocument() throws SAXException {
	} 

	/**
	 * Gestisce l'evento di fine stream XML.
	 */
	public void endDocument() throws SAXException {
	}

	/**
	 * Gestisce l'evento di inizio elemento XML.
	 */
	public void startElement(String namespaceURI, String localName,
			String rawName, Attributes attributes) throws SAXException {
		ObjectBean currentBean = new ObjectBean(localName);
		for (int i = 0; i < attributes.getLength(); i++) {
			currentBean.setAttribute(attributes.getLocalName(i), attributes
					.getValue(i));
		}
		if (_parentBean != null) {
			_parentBean.setAttribute(currentBean);
		}
		_objectBeans.push(currentBean);
		_parentBean = currentBean;
		/*
		System.out.print("START ELEMENT:"+localName);
		System.out.print("  CURRENTBEAN:"+currentBean.get_name());
		System.out.print("  PARENTBEAN:"+_parentBean.get_name());
		System.out.print("  _characters:"+_characters.trim());
		System.out.println("");
		*/
	}

	/**
	 * Gestisce l'evento di fine elemento XML.
	 */
	public void endElement(String namespaceURI, String localName, String rawName)
			throws SAXException {
		String charsave = _characters;
		if (_characters.length() > 0) {			
			_parentBean.setCharacters(_characters);
			_characters = new String();
		}
		_objectBean = (ObjectBean) _objectBeans.pop();
		if (_objectBeans.empty())
			_parentBean = null;
		else
			_parentBean = (ObjectBean) _objectBeans.peek();
		/*
		System.out.print("END ELEMENT:"+localName);
		System.out.print("  _characters:"+charsave.trim());
		if (_parentBean != null)
			System.out.print("   PARENTBEAN:"+_parentBean.get_name());
		System.out.println("");
		*/
	}

	/**
	 * Gestisce il testo di un elemento XML.
	 */
	public void characters(char[] ch, int start, int length)
			throws SAXException {
			_characters += new String(ch, start, length);
	}

	/**
	 * Ritorna l'OB costruito a partire dallo stream
	 * XML.
	 */
	public synchronized ObjectBean getSourceBean() {
		return _objectBean;
	} 	

	/**
	 * Stack dei <code>ObjectBean</code> intermedi.
	 */
	private Stack _objectBeans = null;

	/**
	 * <code>ObjectBean</code> costruito a partire dallo stream XML.
	 */
	private ObjectBean _objectBean = null;

	/**
	 * <code>ObjectBean</code> padre di quello corrente.
	 */
	private ObjectBean _parentBean = null;

	private static String _characters = null;
} 
