package it.pf.base;

import java.io.Serializable;
import java.util.Vector;
/**
 * Implementa una paginazione di elementi contenuti in un <code>Vector</code>
 * ritorna le pagine richieste in forma di <code>ObjectBean</code>,
 * con numero prefissato (_size) di elementi.
 * dal nome ROWS (per default), ogni singolo <code>ObjectBeanAttribute</code>
 * ha chiave= ROW (per default)
 * 
 * @author Fabbri_P
 *
 */
public class Paginator implements Serializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private Vector _rows = null;

	private int _size = 0;
	
	private boolean rotate;
	
	private int _pages = 0;
	private int _currpage = 0;

	private String ROWS = "ROWS";
	private String ROW = "ROW";
	
	/**
	 * costruttore di default
	 */
	public Paginator() {
		_rows = new Vector();
		_size = Constants.DEFAULT_SIZE;
	}

	/**
	 * costruttore specifico: indica il nome dell'OB e la chiave degli OBA
	 * @param beanname
	 * @param attrname
	 * @param size
	 */
	public Paginator(String beanname, String attrname, int size) {
		ROWS = beanname;
		ROW = attrname;
		_rows = new Vector();
		_size = size;
	}
	
	/**
	 * costruttore specifico: indica il nome dell'OB e la chiave degli OBA
	 * @param beanname
	 * @param attrname
	 * @param size
	 */
	public Paginator(String beanname, String attrname) {
		ROWS = beanname;
		ROW = attrname;
		_rows = new Vector();
		_size = Constants.DEFAULT_SIZE;
	}

	/**
	 * costruttore specifico: acquisisce gli elementi costruttivi dall'OB parametro
	 * @param : o (ObjectBean)
	public Paginator(ObjectBean o, int size) {
		ROWS = o.get_name();
		_rows = (Vector)o.get_table();
		ROW = ((ObjectBeanAttribute)_rows.elementAt(1)).get_key();
		_size = size;
	}
	 */

	/**
	 * aggiunge un elemento al vettore da paginare
	 * @param row
	 */
	public void addRow(Object row) {
		if (row == null) {
			return;
		}
		_rows.addElement(row);
		_pages = pages();
		_currpage = 1;
	}

	/**
	 * get ampiezza della pagina
	 * @return int
	 */
	public int getPageSize() {
		return _size;
	}

	/**
	 * imposta ampiezza della singola pagina
	 * @param size
	 */
	public void setPageSize(int size) {
		if (size < 1) {
			return;
		}
		_size = size;
	}

	/**
	 * get numero elementi del vettore
	 * @return int
	 */
	public int rows() {
		return _rows.size();
	}

	/**
	 * get numero di pagine di cui si compone
	 * @return int
	 */
	public int pages() {
		if (_rows.size() == 0)
			return 0;
		return (_rows.size() - 1) / _size + 1;
	}

	/**
	 * ritorna la pagina deducibile dalla direzione richiesta e 
	 * dall'ultima pagina fornita
	 * @param direction
	 * @return ObjectBean
	 */
	public ObjectBean getPage(String direction) {
		if (direction.equalsIgnoreCase(Constants.NEXT)) {
			if (_currpage < _pages) {
				_currpage++;
			} else {
				if (rotate) {
					_currpage = 1;
				} 
			}
		} else if (direction.equalsIgnoreCase(Constants.PREVIOUS)) {
			if (_currpage == 1) {
				if (rotate) {
					_currpage = _pages;
				} 
			} else {
				_currpage--;
			}
		} else if (direction.equalsIgnoreCase(Constants.FIRST)) {
			_currpage = 1;
		} else if (direction.equalsIgnoreCase(Constants.LAST)) {
			_currpage = _pages;
		}
		return getPage(_currpage);
	}
	
	/**
	 * ritorna la pagina richiesta nel paramentro 
	 * @param page
	 * @return ObjectBean
	 */
	public ObjectBean getPage(int page) {
		ObjectBean rows = null;
		rows = new ObjectBean(ROWS);
		if ((page < 1) || (page > pages())) {
			return rows;
		}
		int startRow = (page - 1) * _size;
		int endRow = page * _size;
		if (endRow > _rows.size())
			endRow = _rows.size();
		for (int i = startRow; i < endRow; i++) {
			Object objectRow = _rows.elementAt(i);
			rows.setAttribute(ROW,objectRow);
		}
		return rows;
	}

	/**
	 * ritorna tutto il paginator in forma di OB
	 * @return ObjectBean
	 */
	public ObjectBean getAll() {
		ObjectBean rows = null;
		rows = new ObjectBean(ROWS);
		for (int i = 0; i < _rows.size(); i++) {
			Object objectRow = _rows.elementAt(i);
			rows.setAttribute(ROW,objectRow);
		}
		return rows;
	}

	/**
	 * @return true if page rotate is allowed false otherwise
	 */
	public boolean isRotate() {
		return rotate;
	}
	
	/**
	 * set rotate value
	 * @param rotate
	 */
	public void setRotate(boolean rotate) {
		this.rotate = rotate;
	}
	

} 