package it.pf.dbaccess;

import it.pf.base.ConfigSingleton;
import it.pf.base.ObjectBean;
import it.pf.utils.Tools;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.ParseException;


/**
 * mappatura di un attributo risultato di una query su db
 * laddove possibile l'oggetto ritornato viene generato anche come
 * stringa.
 * 
 * @author Fabbri_P
 *
 */
public class DataField implements Serializable {
	
    private int _sqlType;
    private int _scale;
    private String _name = null;
    private Object _objectValue = null;
    private String _stringValue = null;
    private String _description = null;

	private String _dateFormat = null;
	private String _timestampFormat = null;
	
	/**
	 * default constructor
	 *
	 */
	public DataField() {
		_sqlType = Types.NULL;
		_scale = 0;
		//_stringValue = "";
		_stringValue = null;
		_objectValue = null;
	}
	
	/**
	 * costruttore. Se objectValue non &egrave; di tipo stringa
	 * tenta comunque di rimapparlo anche come stringa dal tipo 
	 * originario.
	 * @param name
	 * @param sqlType
	 * @param objectValue
	 */
	public DataField(String name, int sqlType, Object objectValue) {
		if (ConfigSingleton.getResources() != null) {
			ObjectBean t = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:DATA_BASE:PARAMETER","NAME","dateformat");
			if (t != null)
				_dateFormat = (String)t.getAttributeValue("value");
			else
				_dateFormat = "YYYY-MM-DD";
			t = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:DATA_BASE:PARAMETER","NAME","timestampformat");
			if (t != null)
				_timestampFormat = (String)t.getAttributeValue("value");
			else
				_timestampFormat = "YYYY-MM-DD hh:mm:ss.xxx";
		} else {
			_dateFormat = "YYYY-MM-DD";
			_timestampFormat = "YYYY-MM-DD hh:mm:ss.xxx";
		}
        this.setName(name);
        this.setSqlType(sqlType);
        this.setScale(0);
        if (objectValue instanceof String)
            setStringValue((String)objectValue);
        else {
            setObjectValue(objectValue);
			mapTypesAsString();
        }
    }

	/**
	 * costruttore. Se objectValue non &egrave; di tipo stringa
	 * tenta comunque di rimapparlo anche come stringa dal tipo 
	 * originario.
	 * @param name
	 * @param sqlType
	 * @param objectValue
	 */
	public DataField(String name, int sqlType, Object objectValue, int scale) {
		if (ConfigSingleton.getResources() != null) {
			ObjectBean t = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:DATA_BASE:PARAMETER","NAME","dateformat");
			if (t != null)
				_dateFormat = (String)t.getAttributeValue("value");
			else
				_dateFormat = "YYYY-MM-DD";
			t = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:DATA_BASE:PARAMETER","NAME","timestampformat");
			if (t != null)
				_timestampFormat = (String)t.getAttributeValue("value");
			else
				_timestampFormat = "YYYY-MM-DD hh:mm:ss.xxx";
		} else {
			_dateFormat = "YYYY-MM-DD";
			_timestampFormat = "YYYY-MM-DD hh:mm:ss.xxx";
		}
        this.setName(name);
        this.setSqlType(sqlType);
        this.setScale(scale);
        if (objectValue instanceof String)
            setStringValue((String)objectValue);
        else {
            setObjectValue(objectValue);
			mapTypesAsString();
        }
    }
	
	/**
	 * get sql type
	 * @return int
	 */
    public int getSqlType() {
        return _sqlType;
    } 
	
	/**
	 * set sql type
	 * @param sqlType
	 */
    public void setSqlType(int sqlType) {
		this._sqlType = sqlType;
    } 

	/**
	 * get attribute name
	 * @return String
	 */
    public String getName() {
        return _name;
    }

	/**
	 * set attribute name
	 * @param name
	 */
    public void setName(String name) {
        this._name = name;
    }

	/**
	 * get attribute value
	 * @return Object
	 */
    public Object getObjectValue() {
        return _objectValue;
    }

	/**
	 * set attribute value
	 * @param objectValue
	 */
    public void setObjectValue(Object objectValue) {
        if (objectValue == null)  {
            _objectValue = null;
            return;
        }
        _objectValue = mapTypesAsObject(objectValue);
    }

	/**
	 * get attribute value as String
	 * @return String
	 */
    public String getStringValue() {
        return _stringValue;
    }

	/**
	 * get attribute value as String, if null return ""
	 * @return String
	 */
    public String getDisplayValue() {
		if (_stringValue == null)
			return "";
		else
			return mapTypesForDisplay();
    }
	
	/**
	 * get attribute value as String, if null return ""
	 * @return String
	 */
    public String getEncodedValue() {
		if (_stringValue == null)
			return "";
		else {
			String r = mapTypesForDisplay();
			return Tools.encodeHttp(r);
		}
    }

    /**
	 * set attribute value as String 
	 * @param stringValue
	 */
    public void setStringValue(String stringValue) {
        if (stringValue == null)  {
            _objectValue = null;
            //_stringValue = "";
            _stringValue = null;
            return;
        }
        _stringValue = stringValue;
		_objectValue = stringValue;
    }

    /**
     * il confronto avviene sull'immagine stringa dell'oggetto se ha lo stesso nome
     * @param to
     * @return
     */
	public boolean equals(DataField to) {
		if (to.getName().equalsIgnoreCase(this.getName())) {
			if (to.getStringValue().equals(this.getStringValue()))
				return true;
			else
				return false;
		} else
			return false;
	}
	
	/**
	 * rimappa come oggetto specifico l'oggetto value sulla base del
	 * tipo sql ricevuto per normalizzare il contenuto
	 * @param value
	 */
	private Object mapTypesAsObject(Object value) {
		DataMapper dm = ConfigSingleton.get_mapper();
		if (dm == null)
			try {
				dm = (DataMapper)Class.forName("it.pf.dbaccess.DataMapper4ORA").newInstance();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		dm.set_dateFormat(_dateFormat);
		dm.set_timestampFormat(_timestampFormat);
		return dm.mapTypesAsObject(_sqlType,_scale,value);
	}


	/**
	 * rimappa come stringa l'oggetto value sulla base del
	 * tipo sql ricevuto
	 * @param value
	 */
	private void mapTypesAsString() {
		DataMapper dm = ConfigSingleton.get_mapper();
		if (dm == null)
			try {
				dm = (DataMapper)Class.forName("it.pf.dbaccess.DataMapper4ORA").newInstance();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		dm.set_dateFormat(_dateFormat);
		dm.set_timestampFormat(_timestampFormat);
		_stringValue = dm.mapTypesAsString(_sqlType,this.getObjectValue());
	}

	
	/**
	 * rimappa come stringa l'oggetto value sulla base del
	 * tipo sql ricevuto
	 * @param value
	 */
	private String mapTypesForDisplay() {
		DataMapper dm = ConfigSingleton.get_mapper();
		if (dm == null)
			try {
				dm = (DataMapper)Class.forName("it.pf.dbaccess.DataMapper4ORA").newInstance();
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
		dm.set_dateFormat(_dateFormat);
		dm.set_timestampFormat(_timestampFormat);
		return dm.mapTypesForDisplay(_sqlType,this.getObjectValue());
	}

	
	/**
	 * get attribute description
	 * @return String
	 */
	public String get_description() {
		return _description;
	}
	
	/**
	 * set attribute description
	 * @param _description
	 */
	public void set_description(String _description) {
		this._description = _description;
	}

	public int getScale() {
		return _scale;
	}

	public void setScale(int scale) {
		_scale = scale;
	}
	

} 
