package it.pf.dbaccess;

import it.pf.utils.Tools;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;


public class DataMapper4DB2 extends DataMapper {

	public Object mapTypesAsObject(int _sqlType, int _scale, Object value) {
		if (value == null) {
			return null;
		}
		switch(_sqlType) {
		case Types.BIT:
		case Types.TINYINT:
		case Types.SMALLINT:
			if (value instanceof Short) {
				Short x = (Short)value;
				Integer y = new Integer(x.intValue());
				return y;
			} else 
				return value;
		case Types.INTEGER:
		case Types.BIGINT:
		case Types.FLOAT:
		case Types.REAL:
		case Types.DOUBLE:
		case Types.NUMERIC:
		case Types.CHAR:
		case Types.VARCHAR:
		case Types.LONGVARCHAR:
		case Types.TIME:
			return value;
		case Types.DATE:
			if (value instanceof Date)
				return value;
			else if (value instanceof Timestamp) {
				String tmp = ((Timestamp)value).toString();
				return Tools.getObjectValueForTimeStamp4DB2(tmp,_timestampFormat);
			}
		case Types.TIMESTAMP:
			return value;
		case Types.NULL:
			return value;
		default:
			return value;
		}
	}

	public String mapTypesAsString(int _sqlType, Object value) {
		String _stringValue = null;
		if (value == null) {
			_stringValue = "";
			return _stringValue;
		}
		switch(_sqlType) {
		case Types.BIT:
            _stringValue = "";
			break;
		case Types.TINYINT:
		case Types.SMALLINT:
		case Types.INTEGER:
		case Types.BIGINT:
		case Types.FLOAT:
		case Types.REAL:
		case Types.NUMERIC:
		case Types.DOUBLE:
		case Types.DECIMAL:
			try {
				Method m = value.getClass().getMethod("toString",null);
				_stringValue = (String) m.invoke(value,null);
			} catch (Exception e) {
				e.printStackTrace();
				_stringValue = "";
			}
			break;
		case Types.CHAR:
		case Types.VARCHAR:
		case Types.LONGVARCHAR:
			_stringValue = (String)value;
			break;
		case Types.DATE:
			if (value instanceof Date)
				_stringValue = ((Date)value).toString();
			else if (value instanceof Timestamp) {
				String tmp = ((Timestamp)value).toString();
				_stringValue = tmp.substring(0,10);
			}
			break;
		case Types.TIME:
			_stringValue = ((Time)value).toString();
			break;
		case Types.TIMESTAMP:
			_stringValue = ((Timestamp)value).toString();
			break;
		case Types.NULL:
			_stringValue = "";
			break;
		default:
			_stringValue = "";
			break;
		}
		return _stringValue;
	}

	public String mapTypesForDisplay(int _sqlType, Object value) {
		String display = null;
		if (value == null) {
			return new String("");
		}
		switch(_sqlType) {
		case Types.BIT:
            display = "";
			break;
		case Types.TINYINT:
		case Types.SMALLINT:
		case Types.INTEGER:
		case Types.BIGINT:
		case Types.FLOAT:
		case Types.REAL:
		case Types.NUMERIC:
		case Types.DOUBLE:
		case Types.DECIMAL:
			try {
				Method m = value.getClass().getMethod("toString",null);
				display = (String) m.invoke(value,null);
			} catch (Exception e) {
				e.printStackTrace();
				display = "";
			}
			break;
		case Types.CHAR:
		case Types.VARCHAR:
		case Types.LONGVARCHAR:
			display = (String)value;
			break;
		case Types.DATE:
			if (value instanceof Date) {
				String d = ((Date)value).toString();
				display = d.substring(8);
				display += "."+d.substring(5,7);
				display += "."+d.substring(0,4);
			} else
				display = "";
			break;
		case Types.TIME:
			display = ((Time)value).toString();
			break;
		case Types.TIMESTAMP:
			display = ((Timestamp)value).toString();
			break;
		case Types.NULL:
			display = "";
			break;
		default:
			display = "";
			break;
		}
		return display;
	}

}
