package it.pf.dbaccess;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

/**
 * Questa Classe rappresenta una riga dei dati di ritorno da una query database
 * Si compone di un array di <code>DataField</code>
 * 
 * @author Fabbri_P
 *
 */
public class DataRow implements Serializable, Cloneable {

	//private int _columnSize;	//ultimo inserito

	private ArrayList _columns;

	private HashMap _columnsMap = null;

	/**
	 * costruttore di default
	 * @param columnsSize
	 */
	public DataRow() {
		_columns = new ArrayList();
		_columnsMap = new HashMap();
		//_columnSize = -1;
	}

	/**
	 * costruttore: richiede ampiezza dell'array DataField[] da costruire
	 * @param columnsSize
	 */
	public DataRow(int columnsSize) {
		_columns = new ArrayList(columnsSize);
		_columnsMap = new HashMap();
		//_columnSize = -1;
	}

	/**
	 * costruttore: a partire da altro dr
	 * @param columnsSize
	 */
	public DataRow(DataRow dr) {
		_columns = new ArrayList(dr.size());
		_columnsMap = new HashMap();
		for (int i = 0; i<dr.size(); i++) {
			DataField df = dr.getColumn(i);
			this.addColumn(i, df);
		}
	}

	/**
	 * aggiunge un attributo nella posizione desiderata
	 * @param position
	 * @param dataColumn
	 */
	public void addColumn(int position, DataField dataColumn) {
		_columns.add(position,dataColumn);
		_columnsMap.put(dataColumn.getName().toUpperCase(), new Integer(position));
		//_columnSize = position;
	}

	/**
	 * rimpiazza un attributo nella posizione desiderata
	 * @param position
	 * @param dataColumn
	 */
	public void setColumn(int position, DataField dataColumn) {
		if (position <= _columns.size())
			_columns.set(position,dataColumn);
	}
	
	/**
	 * ritorna l'attributo (DataField) alla posizione richiesta
	 * @param position
	 * @return DataField
	 */
	public DataField getColumn(int position) {
		if (position <= _columns.size())
			return (DataField)_columns.get(position);
		else
			return null;
	}	

	/**
	 * ritorna l'attributo (DataField) il cui nome corrisponde a columnName
	 * @param columnName
	 * @return DataField
	 */
	public DataField getColumn(String columnName) {
		Object o = _columnsMap.get(columnName.toUpperCase());
		if (o == null)
			return null;
		else {
			int position = ((Integer) _columnsMap.get(columnName.toUpperCase())).intValue();
			return getColumn(position);
		}
	}

	/**
	 * ritorna la posizione dell'attributo (DataField) il cui nome corrisponde a columnName
	 * @param columnName
	 * @return DataField
	 */
	public int getColumnPosition(String columnName) {
		Integer xp = (Integer) _columnsMap.get(columnName.toUpperCase());
		if (xp == null)
			return -1;
		int position = xp.intValue();
		return position;
	}
	
	/**
	 * aggiorna il valore di un attributo (DataField)
	 * @param columnName
	 * @param new_value
	 */
	public void updateColumn(String columnName,Object new_value) {
		DataField t = this.getColumn(columnName);
		int pt = this.getColumnPosition(columnName);
		if (!(pt < 0)) {
			t.setObjectValue(new_value);
			if (new_value instanceof String)
				t.setStringValue((String)new_value);
			this.setColumn(pt,t);
		}
	}
	
	/**
	 * ritorna vero se il DataRow contiene il DataField in argomento
	 */
	public boolean contains(DataField df) {
		if (df == null)
			return false;
		for (int i = 0; i < _columns.size(); i++) {
			DataField dfthis = (DataField)_columns.get(i);
			if (df.equals(dfthis))
				return true;
		}
		return false;
	}
	
	/**
	 * ritorna il numero di attributi per compatibilit� con vecchia gestione
	 * @return int
	 */
	public int getColumnSize() {
		return _columns.size();
	}
	//nuovo metodo standard
	public int size() {
		return _columns.size();
	}

	/**
	 * ritorna l'OB costruito a partire dagli attributi del DataRow
	 * @return ObjectBean
	 */
	public ObjectBean getObjectBean() {
		ObjectBean sb = new ObjectBean(Constants.ROW);
		Object columnObjectValue = null;
		String columnName = null;
		for (int i = 0; i < _columns.size(); i++) {
			columnObjectValue = ((DataField)_columns.get(i)).getObjectValue();
			columnName = ((DataField)_columns.get(i)).getName();
			if (columnObjectValue != null)
				sb.setAttribute(columnName, columnObjectValue);
			else
				continue;
		}
		return sb;
	} 
	
	/**
	 * mappa il DataRow su Properties
	 * @param dr
	 * @param prop
	 * @throws Exception
	 */
	public Properties getAsProperties()  {
		Properties prop = new Properties();

		for (int i = 0; i < _columns.size(); i++) {
			DataField df = this.getColumn(i);
			prop.put(df.getName(), df.getStringValue());
		}
		return prop;
	}

	/**
	 * appende un data row (primo arg.) ad un'altro (secondo arg)
	 * il data row risultante sara' la concatenazione del secondo argomento al primo
	 * @param dr
	 * @param to
	 */
	public static DataRow append(DataRow dr, DataRow to) {
		int newsize = dr.size()+to.size();
		DataRow result = new DataRow(newsize);
		int pos = 0;
		for (int i = 0; i<to.size(); i++) {
			DataField df = to.getColumn(i);
			result.addColumn(pos, df);
			pos++;
		}
		for (int i = 0; i<dr.size(); i++) {
			DataField df = dr.getColumn(i);
			result.addColumn(pos, df);
			pos++;
		}
		return result;
	}
	
	/**
	 * clona un data row (primo arg.) su un'altro (secondo arg)
	 * eventualmente di maggiore dimensione 
	 * (secondo argomento la nuova dimensione, se = 0 allora uguale alla prima)
	 * @param dr
	 * @param newdim
	 */
	public DataRow clone() {
		try {
			return (DataRow) super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
			return null;
		}
	}
	/**
	 * elimina l'iesimo elemento
	 * @param dr
	 * @param pos
	 */
	public static DataRow delete(DataRow dr, int pos) {
		int newsize = dr.size()-1;
		DataRow result = new DataRow(newsize);
		int p=0;
		for (int i = 0; i<dr.size(); i++) {
			if (i != pos) {
				DataField df = dr.getColumn(i);
				result.addColumn(p, df);
				p++;
			}
		}
		return result;
	}


	/**
	 * elimina l'elemento di nome=name
	 * @param dr
	 * @param name
	 */
	public static DataRow delete(DataRow dr, String name) {
		int newsize = dr.size()-1;
		DataRow result = new DataRow(newsize);
		int p=0;
		for (int i = 0; i<dr.size(); i++) {
			DataField df = dr.getColumn(i);
			if (!df.getName().equals(name)) {
				result.addColumn(p, df);
				p++;
			}
		}
		return result;
	}

	/**
	 * aggiunge in coda l'elemento df
	 * @param dr
	 * @param df
	 */
	public static DataRow add(DataRow dr, DataField df) {
		int newsize = dr.size()+1;
		DataRow result = new DataRow(newsize);
		int p=0;
		for (int i = 0; i<dr.size(); i++) {
			DataField dfo = dr.getColumn(i);
			result.addColumn(i, dfo);
			p++;
		}
		result.addColumn(p, df);
		return result;
	}
	
	/**
	 * ritorna una stringa con i data fields name=valore
	 */
	public String toString() {
		String result = "";
		for (int i = 0; i<this.size(); i++) {
			DataField df = this.getColumn(i);
			result += df.getName()+"="+df.getStringValue()+";";
		}
		return result;
	}
	
	/**
	 * ritorna una stringa in formato CSV con i nomi dei campi
	 */
	public String toCSVSHeaderString() {
		String result = "";
		for (int i = 0; i<this.size(); i++) {
			DataField df = this.getColumn(i);
			result += df.getName()+",";
		}
		result += "\n";
		return result;
	}

	/**
	 * ritorna una stringa in formato CSV con i valori dei data fields
	 */
	public String toCSVString() {
		String result = "";
		for (int i = 0; i<this.size(); i++) {
			DataField df = this.getColumn(i);
			result += df.getStringValue()+",";
		}
		result += "\n";
		return result;
	}

}