package it.pf.dbaccess;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
import it.pf.utils.DataUtil;
import it.pf.utils.Tools;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Vector;
import java.io.File; 
import java.math.BigDecimal;
import java.util.Date; 

import jxl.*;
import jxl.read.biff.*;
import jxl.write.*;
import jxl.write.Number;
import jxl.write.biff.RowsExceededException;


/**
 * Questa Classe rappresenta un insieme di righe (<code>DataRow</code>)
 * costruibili a partire da un ObjectBean. Attenzione! Non un OB qualunque
 * ma solo quello costruito da <code>SQLCommand</code> come risultato di una
 * select.
 * 
 * @author Fabbri_P
 *
 */
public class DataSheet implements Serializable {

	private int _rowSize;

	private DataRow[] _rows;

	private int currentPos = 0;
	/**
	 * costruttore di default
	 * @param columnsSize
	 */
	public DataSheet() {
		_rowSize = 10;
		_rows = new DataRow[10];
		currentPos = 0;
	}

	/**
	 * costruttore: richiede ampiezza dell'array DataRow[] da costruire
	 * @param columnsSize
	 */
	public DataSheet(int size) {
		_rows = new DataRow[size];
		_rowSize = size;
		currentPos = 0;
	}


	/**
	 * costruttore a partire da un ObjectBean prodotto da SQLCommand
	 * @param ob
	 */
	public DataSheet(ObjectBean ob) {
		_rows = new DataRow[ob.get_size()];
		_rowSize = ob.get_size();
		currentPos = 0;
		Vector vector = (Vector)ob.get_table();
		for (int i=0; i<_rowSize; i++) {
			ObjectBeanAttribute oba = (ObjectBeanAttribute)vector.elementAt(i);
			Object o = oba.get_value();
			if (o instanceof DataRow)
				addRow((DataRow)o);
		}
	}

	/**
	 * costruttore a partire da un Vector
	 * @param ob
	 */
	public DataSheet(Vector vector) {
		_rows = new DataRow[vector.size()];
		_rowSize = vector.size();
		currentPos = 0;
		for (int i=0; i<_rowSize; i++) {
			if (vector.elementAt(i) instanceof ObjectBeanAttribute) {
				ObjectBeanAttribute oba = (ObjectBeanAttribute)vector.elementAt(i);
				Object o = oba.get_value();
				if (o instanceof DataRow)
					addRow((DataRow)o);
			} else if (vector.elementAt(i) instanceof DataRow) {
				addRow((DataRow)vector.elementAt(i));
			}
		}
	}

	/**
	 * costruttore a partire da un file xls
	 * la prima riga � usata come intestazione colonne
	 * la prima colonna � utilizzata come indicatore di fine processo
	 * quando la cella i-esima della colonna A � vuota � fine lettura
	 */
	public DataSheet(String xlsfile, int sheetnum) {
		Workbook workbook;
		Vector rows = new Vector();
		try {
			workbook = Workbook.getWorkbook(new File(xlsfile));
			Sheet sheet = workbook.getSheet(sheetnum);
			int shcols = sheet.getColumns();
			int shrows = sheet.getRows();
			if ( (shcols == 0) || (shrows <= 1)) {
				_rows = null;
				_rowSize = 0;
				currentPos = 0;
			} else {
				ArrayList names = buildColNames(sheet);
				int cols = names.size();
				int numrow = 1; //first row contains titles i.e. start with 2-nd
				boolean cicle = true;
				while(cicle) {
					try {
						Cell[] cell = sheet.getRow(numrow);
						if (cell == null)
							cicle = false;
						else {
							DataRow dr = buildSingleRow(cell,names,cols,numrow);
							rows.add(dr);
							numrow++;
						}
					} catch (java.lang.ArrayIndexOutOfBoundsException ixe) {
						cicle = false;
					}
				}
				workbook.close();
				_rows = new DataRow[rows.size()];
				_rowSize = rows.size();
				currentPos = 0;
				for (int i=0; i<_rowSize; i++) {
					addRow((DataRow)rows.elementAt(i));
				}
			}
		} catch (BiffException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private ArrayList buildColNames(Sheet sheet) {
		ArrayList names = new ArrayList();
		Cell[] cell = sheet.getRow(0);
		for (int i=0; i<cell.length; i++) {
			String name = cell[i].getContents().trim();
			names.add(i,name);
		}
		return names;
	}
	
	private DataRow buildSingleRow(Cell[] cell, ArrayList names, int cols, int row) {
		DataRow dr = new DataRow(cols+1);
		String cn = "ROW_NUMBER";
		int ct = Types.INTEGER;
		Integer cv = new Integer(row);
		DataField rnum = new DataField(cn,ct,cv);
		dr.addColumn(0,rnum);
		for (int i=0; i<cols; i++) {
			DataField df = null;
			if (i >= cell.length) 
				df = buildSingleDataField(null,(String)names.get(i));
			else
				df = buildSingleDataField(cell[i],(String)names.get(i));
			dr.addColumn(i+1,df);
		}
		return dr;
	}
	
	private DataField buildSingleDataField(Cell cell, String name) {
		if (cell == null) {
			DataField df = new DataField(name,Types.NULL,null);
			return df;
		}
		Object o = buildSingleValue(cell);
		int sqlType = 0;
		if (o instanceof java.lang.Boolean)
			sqlType = Types.BOOLEAN;
		else if (o instanceof java.sql.Date)
			sqlType = Types.DATE;
		else if (o instanceof Integer)
			sqlType = Types.INTEGER;
		else if (o instanceof BigDecimal)
			sqlType = Types.DECIMAL;
		else 
			sqlType = Types.VARCHAR;
		DataField df = new DataField(name,sqlType,o);
		return df;
	}

	private Object buildSingleValue(Cell cell) {
		CellType type = cell.getType();
		if (type.equals(CellType.BOOLEAN)) {
			return new java.lang.Boolean(cell.getContents().trim());
		} else if(type.equals(CellType.DATE)) {
			DataUtil data = new DataUtil(cell.getContents().trim());
			return data.getSQLDate();
		} else if(type.equals(CellType.NUMBER)) {
			String x = cell.getContents().trim();
			x = x.replace(",",".");
			if ( (x.length() > 9) || (x.contains(".")) )
				return new BigDecimal(x);
			else
				return new Integer(x);
		} else {
			return cell.getContents().trim();
		}
	}
	
	/**
	 * aggiunge una riga
	 */
	public void addRow(DataRow dataRow) {
		_rows[currentPos++] = dataRow;
	}

	/**
	 * rimpiazza una riga nella posizione desiderata
	 * @param position
	 * @param dataRow
	 */
	public void setRow(int position, DataRow dataRow) {
		if (position <= currentPos) 
			_rows[position] = dataRow;
	}
	
	/**
	 * ritorna la riga (DataRow) alla posizione richiesta
	 * @param position
	 * @return DataRow
	 */
	public DataRow getRow(int position) {
		if (position <= currentPos)
			return _rows[position];
		else
			return null;
	}	

	
	/**
	 * ritorna la riga (DataRow) che contiene il valore desiderato  
	 * dell'attributo specificato confrontato con il valore stringa
	 * oppure null se nessuna riga risponde alla richiesta.
	 * @param position
	 * @return DataRow
	 */
	public DataRow getRow(String attribute, String value ) {
		for (int i=0; i<getRowSize(); i++) {
			DataRow dr = this.getRow(i);
			DataField df = dr.getColumn(attribute);
			if (df != null) {
				if (df.getStringValue().trim().equals(value.trim())) {
					return dr;
				}
			}
		}
		return null;
	}	

	
	/**
	 * ritorna il numero di righe valorizzate 
	 * @return int
	 */
	public int getRowSize() {
		return currentPos;
	}

	/**
	 * genera il file excel a partire dal contenuto del DataSheet
	 * @throws IOException 
	 */
	public synchronized void writeXLS(String file) throws IOException {
		File f = new File(file);
		//FileOutputStream fos = new FileOutputStream(f);
		//fos.close();
		WritableWorkbook workbook = Workbook.createWorkbook(f);
		WritableSheet sheet = workbook.createSheet("Foglio1", 0);
		setXLSHeaders(sheet);
		if (_rowSize > 0) {
			for (int i = 0; i < _rowSize; i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
	}
	
	/**
	 * aggiunge al file excel il contenuto del DataSheet
	 * @throws IOException 
	 */
	public synchronized void appendXLS(String file) throws IOException {
		FileOutputStream fos = new FileOutputStream(new File(file),true);
		WritableWorkbook workbook = Workbook.createWorkbook(fos);
		WritableSheet sheet = workbook.getSheet(0);
		if (_rowSize > 0) {
			for (int i = 0; i < _rowSize; i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
	}

	/**
	 * genera il file excel a partire dal contenuto del DataSheet e 
	 * restituisce un ByteArrayOutputStream (utile per essere scritto su
	 * flusso di output di una response ad esempio)
	 * @throws IOException 
	 */
	public synchronized ByteArrayOutputStream toByteStream() throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		WritableWorkbook workbook = Workbook.createWorkbook(bos);
		WritableSheet sheet = workbook.createSheet("Foglio1", 0);
		setXLSHeaders(sheet);
		if (_rowSize > 0) {
			for (int i = 0; i < _rowSize; i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
		return bos;
	}
	
	/**
	 * imposta le label di colonna
	 */
	private void setXLSHeaders(WritableSheet sheet) {
		if (_rows[0] != null) {
			DataRow first = _rows[0];
			for (int i = 0; i<first.getColumnSize(); i++) {
				try {
					Label label = new Label(i, 0, first.getColumn(i).getName()); 
					sheet.addCell(label);
				} catch (RowsExceededException e) {
					e.printStackTrace();
				} catch (WriteException e) {
					e.printStackTrace();
				} 
			}
		}
	}

	/**
	 * imposta una riga del file xls con i dati della i-esima 
	 * riga (DataRow) del DataSheet
	 */
	private void setXLSRow(WritableSheet sheet, int r) {
		DataRow dr = _rows[r];
		if (dr == null)
			return;
		WritableCellFormat integerFormat = new WritableCellFormat (NumberFormats.INTEGER);
		WritableCellFormat floatFormat = new WritableCellFormat (NumberFormats.FLOAT);
		for (int i = 0; i<dr.getColumnSize(); i++) {
			try {
				DataField df = dr.getColumn(i);
				int sqlType = df.getSqlType();
				switch(sqlType) {
					case Types.BIT:
					case Types.TINYINT:
					case Types.SMALLINT:
					case Types.DOUBLE:
					case Types.BIGINT:
					case Types.INTEGER:
						if(df.getStringValue().equalsIgnoreCase("")){
							Label label = new Label(i, r+1, null);
							sheet.addCell(label);
						} else {
							Float x = new Float(df.getStringValue());
							Number cell = new Number(i, r+1, x.floatValue(), integerFormat);
							sheet.addCell(cell);
						}
						break;
					case Types.NUMERIC:
					case Types.DECIMAL:
					case Types.FLOAT:
					case Types.REAL:
						if(df.getStringValue().equalsIgnoreCase("")){
							Label label = new Label(i, r+1, null);
							sheet.addCell(label);
						} else {
							Float x = new Float(df.getStringValue());
							Number cell = new Number(i, r+1, x.floatValue(), floatFormat);
							sheet.addCell(cell);
						}
						break;
					case Types.CHAR:
					case Types.VARCHAR:
					case Types.LONGVARCHAR:
					case Types.DATE:
					case Types.TIME:
					case Types.TIMESTAMP:
						Label label = new Label(i, r+1, df.getStringValue()); 
						sheet.addCell(label);
						break;
					case Types.NULL:
					default:
						label = new Label(i, r+1, null); 
						sheet.addCell(label);
						break;
				}
			} catch (RowsExceededException e) {
				e.printStackTrace();
			} catch (WriteException e) {
				e.printStackTrace();
			} 
		}
	}


} 
