package it.pf.dbaccess;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
import it.pf.utils.DataUtil;
import it.pf.utils.Tools;

import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.io.File; 
import java.math.BigDecimal;
import java.util.Date; 

import jxl.*;
import jxl.read.biff.*;
import jxl.write.*;
import jxl.write.Number;
import jxl.write.biff.RowsExceededException;


/**
 * Questa Classe rappresenta un insieme di righe (<code>DataRow</code>)
 * costruibili a partire da un ObjectBean. Attenzione! Non un OB qualunque
 * ma solo quello costruito da <code>SQLCommand</code> come risultato di una
 * select.
 * 
 * @author Fabbri_P
 *
 */
public class RowSet implements Serializable {

	private Vector _rows;

	/**
	 * costruttore di default
	 * @param columnsSize
	 */
	public RowSet() {
		_rows = new Vector();
	}

	/**
	 * costruttore: richiede ampiezza dell'array DataRow[] da costruire
	 * @param columnsSize
	 */
	public RowSet(int size) {
		_rows = new Vector(size);
	}


	/**
	 * costruttore a partire da un Vector
	 */
	public RowSet(Vector v) {
		_rows = new Vector(v.size());
		_rows.addAll(v);
	}
	
	/**
	 * costruttore a partire da un ObjectBean prodotto da SQLCommand
	 * @param ob
	 */
	public RowSet(ObjectBean ob) {
		_rows = new Vector(ob.get_size());
		Vector vector = (Vector)ob.get_table();
		for (int i=0; i<ob.get_size(); i++) {
			ObjectBeanAttribute oba = (ObjectBeanAttribute)vector.elementAt(i);
			Object o = oba.get_value();
			if (o instanceof DataRow)
				_rows.add(o);
		}
	}


	/**
	 * costruttore a partire da un file xls
	 * la prima riga � usata come intestazione colonne
	 * la prima colonna � utilizzata come indicatore di fine processo
	 * quando la cella i-esima della colonna A � vuota � fine lettura
	 */
	public RowSet(String xlsfile, int sheetnum) {
		Workbook workbook;
		Vector rows = new Vector();
		try {
			workbook = Workbook.getWorkbook(new File(xlsfile));
			Sheet sheet = workbook.getSheet(sheetnum);
			int shcols = sheet.getColumns();
			int shrows = sheet.getRows();
			if ( (shcols == 0) || (shrows <= 1)) {
				_rows = null;
			} else {
				_rows = new Vector();
				ArrayList names = buildColNames(sheet);
				int cols = names.size();
				int numrow = 1; //first row contains titles i.e. start with 2-nd
				boolean cicle = true;
				while(cicle) {
					try {
						Cell[] cell = sheet.getRow(numrow);
						if (cell == null)
							cicle = false;
						else {
							DataRow dr = buildSingleRow(cell,names,cols,numrow);
							_rows.add(dr);
							numrow++;
						}
					} catch (java.lang.ArrayIndexOutOfBoundsException ixe) {
						cicle = false;
					}
				}
				workbook.close();
			}
		} catch (BiffException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * aggiunge una riga al vettore
	 * @param dr
	 * @return
	 */
	public boolean addRow(Object dr) {
		if (dr instanceof DataRow) {
			_rows.add(dr);
			return true;
		} else
			return false;
	}
	
	
	/**
	 * ritorna la riga (DataRow) alla posizione richiesta
	 * @param position
	 * @return DataRow
	 */
	public DataRow getRow(int position) {
		if (position <= _rows.size())
			return (DataRow)_rows.get(position);
		else
			return null;
	}	

	
	/**
	 * ritorna la riga (DataRow) che contiene il valore desiderato  
	 * dell'attributo specificato confrontato con il valore stringa
	 * oppure null se nessuna riga risponde alla richiesta.
	 * @param position
	 * @return DataRow
	 */
	public DataRow getRow(String attribute, String value ) {
		for (int i=0; i<_rows.size(); i++) {
			DataRow dr = this.getRow(i);
			DataField df = dr.getColumn(attribute);
			if (df != null) {
				if (df.getStringValue().trim().equals(value.trim())) {
					return dr;
				}
			}
		}
		return null;
	}	

	/**
	 * ritorna la riga (DataRow) che contiene i valori del subset 
	 * rappresentato dal datarow in argomento  
	 * confrontati per valore stringa
	 * oppure null se nessuna riga risponde alla richiesta.
	 * @param position
	 * @return DataRow
	 */
	public DataRow getRow(DataRow subset) {
		for (int i=0; i<_rows.size(); i++) {
			DataRow dr = this.getRow(i);
			int matches = 0;
			for (int j=0; j<subset.size(); j++) {
				DataField dfs = subset.getColumn(j);
				String name = dfs.getName();
				String value = dfs.getStringValue().trim();
				DataField df = dr.getColumn(name);
				if (df != null) {
					String dato = df.getStringValue().trim();
					if (match(dato,value)) {
						matches++;
					}
				} else {
					break;
				}
			}
			if (matches == subset.size()) {
				return dr;
			}
		}
		return null;
	}	

	/**
	 * ritorna le riga (vettore di DataRow) che contengono i valori del subset 
	 * rappresentato dal datarow in argomento  
	 * confrontati per valore stringa
	 * oppure null se nessuna riga risponde alla richiesta.
	 * @param position
	 * @return DataRow
	 */
	public RowSet getRows(DataRow subset) {
		RowSet rs = new RowSet();
		for (int i=0; i<_rows.size(); i++) {
			DataRow dr = this.getRow(i);
			int matches = 0;
			for (int j=0; j<subset.size(); j++) {
				DataField dfs = subset.getColumn(j);
				String name = dfs.getName();
				String value = dfs.getStringValue();
				DataField df = dr.getColumn(name);
				if (df != null) {
					if (df.getStringValue().trim().equals(value.trim())) {
						matches++;
					}
				} else {
					break;
				}
			}
			if (matches == subset.size()) {
				rs.addRow(dr);
			}
		}
		return rs;
	}	

	private boolean match(String dato, String value) {
		if ((dato == null) &&
			(value == null))
			return true;
		if ( (dato == null) && (value != null) )
			return false;
		if ( (dato != null) && (value == null) )
			return false;
		if (dato.equals(value.trim())) {
			return true;
		} else {
			if (java.util.regex.Pattern.matches("\\d*",dato)) {
				try {
					Integer x = new Integer(dato);
					Integer y = new Integer(value.trim());
					if (x.intValue() == y.intValue()) 
						return true;
				} catch (NumberFormatException nfe) {
				}
			}
		}
		return false;
	}
	
	/**
	 * ritorna il numero di elementi del vettore
	 * @return
	 */
	public int size() {
		return _rows.size();
	}
	
	/**
	 * wrapper di isEmpty
	 * @return
	 */
	public boolean isEmpty() {
		return _rows.isEmpty();
	}
	
	/**
	 * wrapper di iterator
	 * @return
	 */
	public Iterator iterator() {
		return _rows.iterator();
	}
	/**
	 *  costruisce i nomi delle colonne dalla prima riga del vettore
	 * 
	 */
	private ArrayList buildColNames(Sheet sheet) {
		ArrayList names = new ArrayList();
		Cell[] cell = sheet.getRow(0);
		for (int i=0; i<cell.length; i++) {
			String name = cell[i].getContents().trim();
			names.add(i,name);
		}
		return names;
	}
	
	/**
	 * costruisce una riga del file excel
	 * @param cell
	 * @param names
	 * @param cols
	 * @param row
	 * @return
	 */
	private DataRow buildSingleRow(Cell[] cell, ArrayList names, int cols, int row) {
		DataRow dr = new DataRow(cols+1);
		String cn = "ROW_NUMBER";
		int ct = Types.INTEGER;
		Integer cv = new Integer(row);
		DataField rnum = new DataField(cn,ct,cv);
		dr.addColumn(0,rnum);
		for (int i=0; i<cols; i++) {
			DataField df = null;
			if (i >= cell.length) 
				df = buildSingleDataField(null,(String)names.get(i));
			else
				df = buildSingleDataField(cell[i],(String)names.get(i));
			dr.addColumn(i+1,df);
		}
		return dr;
	}
	
	/**
	 * costruisce una singola cella del file excel
	 * @param cell
	 * @param name
	 * @return
	 */
	private DataField buildSingleDataField(Cell cell, String name) {
		if (cell == null) {
			DataField df = new DataField(name,Types.NULL,null);
			return df;
		}
		Object o = buildSingleValue(cell);
		int sqlType = 0;
		if (o instanceof java.lang.Boolean)
			sqlType = Types.BOOLEAN;
		else if (o instanceof java.sql.Date)
			sqlType = Types.DATE;
		else if (o instanceof Integer)
			sqlType = Types.INTEGER;
		else if (o instanceof BigDecimal)
			sqlType = Types.DECIMAL;
		else 
			sqlType = Types.VARCHAR;
		DataField df = new DataField(name,sqlType,o);
		return df;
	}

	/**
	 * imposta un singolo valore di cella
	 * @param cell
	 * @return
	 */
	private Object buildSingleValue(Cell cell) {
		CellType type = cell.getType();
		if (type.equals(CellType.BOOLEAN)) {
			return new java.lang.Boolean(cell.getContents().trim());
		} else if(type.equals(CellType.DATE)) {
			DataUtil data = new DataUtil(cell.getContents().trim());
			return data.getSQLDate();
		} else if(type.equals(CellType.NUMBER)) {
			String x = cell.getContents().trim();
			x = x.replace(",",".");
			if ( (x.length() > 9) || (x.contains(".")) )
				return new BigDecimal(x);
			else
				return new Integer(x);
		} else {
			return cell.getContents().trim();
		}
	}
	
	
	/**
	 * genera il file csv a partire dal contenuto del DataSheet
	 * @throws IOException 
	 */
	public CharSequence getCSV() {
		StringBuilder sb = new StringBuilder();
		if (_rows.size() > 0) {
			DataRow dr = (DataRow)_rows.get(0);
			sb.append(dr.toCSVSHeaderString());
			for (int i = 0; i < _rows.size(); i++) {
				dr = (DataRow)_rows.get(i);
				sb.append(dr.toCSVString());
			}
		}
		return sb;
	}

	/**
	 * genera il file excel a partire dal contenuto del DataSheet
	 * @throws IOException 
	 */
	public synchronized void writeXLS(String file) throws IOException {
		WritableWorkbook workbook = Workbook.createWorkbook(new File(file));
		WritableSheet sheet = workbook.createSheet("Foglio1", 0);
		setXLSHeaders(sheet);
		if (_rows.size() > 0) {
			for (int i = 0; i < _rows.size(); i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
	}
	
	/**
	 * aggiunge al file excel il contenuto del DataSheet
	 * @throws IOException 
	 */
	public synchronized void appendXLS(String file) throws IOException {
		FileOutputStream fos = new FileOutputStream(new File(file),true);
		WritableWorkbook workbook = Workbook.createWorkbook(fos);
		WritableSheet sheet = workbook.getSheet(0);
		if (_rows.size() > 0) {
			for (int i = 0; i < _rows.size(); i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
	}

	/**
	 * genera il file excel a partire dal contenuto del DataSheet e 
	 * restituisce un ByteArrayOutputStream (utile per essere scritto su
	 * flusso di output di una response ad esempio)
	 * @throws IOException 
	 */
	public synchronized ByteArrayOutputStream toByteStream() throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		WritableWorkbook workbook = Workbook.createWorkbook(bos);
		WritableSheet sheet = workbook.createSheet("Foglio1", 0);
		setXLSHeaders(sheet);
		if (_rows.size() > 0) {
			for (int i = 0; i < _rows.size(); i++) {
				setXLSRow(sheet, i);
				if (i > 65000)
					break;
			}
		}
		workbook.write();
		workbook.close();
		return bos;
	}
	
	/**
	 * imposta le label di colonna
	 */
	private void setXLSHeaders(WritableSheet sheet) {
		if (!_rows.isEmpty()) {
			DataRow first = (DataRow)_rows.get(0);
			for (int i = 0; i<first.getColumnSize(); i++) {
				try {
					Label label = new Label(i, 0, first.getColumn(i).getName()); 
					sheet.addCell(label);
				} catch (RowsExceededException e) {
					e.printStackTrace();
				} catch (WriteException e) {
					e.printStackTrace();
				} 
			}
		}
	}

	/**
	 * imposta una riga del file xls con i dati della i-esima 
	 * riga (DataRow) del DataSheet
	 */
	private void setXLSRow(WritableSheet sheet, int r) {
		DataRow dr = (DataRow)_rows.get(r);
		if (dr == null)
			return;
		WritableCellFormat integerFormat = new WritableCellFormat (NumberFormats.INTEGER);
		WritableCellFormat floatFormat = new WritableCellFormat (NumberFormats.FLOAT);
		for (int i = 0; i<dr.getColumnSize(); i++) {
			try {
				DataField df = dr.getColumn(i);
				int sqlType = df.getSqlType();
				switch(sqlType) {
					case Types.BIT:
					case Types.TINYINT:
					case Types.SMALLINT:
					case Types.DOUBLE:
					case Types.BIGINT:
					case Types.INTEGER:
						if(df.getStringValue().equalsIgnoreCase("")){
							Label label = new Label(i, r+1, null);
							sheet.addCell(label);
						} else {
							Float x = new Float(df.getStringValue());
							Number cell = new Number(i, r+1, x.floatValue(), integerFormat);
							sheet.addCell(cell);
						}
						break;
					case Types.NUMERIC:
					case Types.DECIMAL:
					case Types.FLOAT:
					case Types.REAL:
						if(df.getStringValue().equalsIgnoreCase("")){
							Label label = new Label(i, r+1, null);
							sheet.addCell(label);
						} else {
							Float x = new Float(df.getStringValue());
							Number cell = new Number(i, r+1, x.floatValue(), floatFormat);
							sheet.addCell(cell);
						}
						break;
					case Types.CHAR:
					case Types.VARCHAR:
					case Types.LONGVARCHAR:
					case Types.DATE:
					case Types.TIME:
					case Types.TIMESTAMP:
						Label label = new Label(i, r+1, df.getStringValue()); 
						sheet.addCell(label);
						break;
					case Types.NULL:
					default:
						label = new Label(i, r+1, null); 
						sheet.addCell(label);
						break;
				}
			} catch (RowsExceededException e) {
				e.printStackTrace();
			} catch (WriteException e) {
				e.printStackTrace();
			} 
		}
	}

	/**
	 * ritorna l'intero vettore come List
	 * @return
	 */
	public List get_rows() {
		return (List)_rows;
	}


} 
