package it.pf.dbaccess;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import com.edb.util.PGobject;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
/**
 * Classe esecutore di statements SQL
 * nel caso di SELECT imposta il risultato come OB dal nome ROWS
 * con OBA con chiave=ROW e valore=DataRow
 * 
 * @author Fabbri_P
 *
 */
public class SQLBase {

	protected boolean _valid;

	protected boolean _hasInputParms;

	protected String _name;

	protected String _query;

	protected String _queryType;

	protected Connection _con;

	protected List _inputParms;
	
	protected List _outputParms;

	protected ObjectBean _result;
	
	protected int _rowsAffected = 0;

	protected int _rowsFetched = 0;

	

	/** 
	 * ritorna le righe influenzate dallo statement SQL
	 * nel caso di UPDATE, DELETE, INSERT
	 * @return int
	 */
	public int get_rowsAffected() {
		return _rowsAffected;
	}
	
	/**
	 * imposta il numero di righe influenzate dallo statement SQL
	 * nel caso di UPDATE, DELETE, INSERT
	 * @param affected
	 */
	public void set_rowsAffected(int affected) {
		_rowsAffected = affected;
	}
	
	/**
	 * @return true se lo statement prevede parametri di input (?)
	 * false altrimenti
	 */
	public boolean hasInputParms() {
		return _hasInputParms;
	}

	/**
	 * @return String - il nome dello statement
	 */
	public String get_name() {
		return _name;
	}

	/**
	 * @return String - lo statement da eseguire
	 */
	public String get_query() {
		return _query;
	}

	/**
	 * @return true se lo statement &egrave; valido
	 * false altrimenti
	 */
	public boolean is_valid() {
		return _valid;
	}


	
	/**
	 * Genera un DataRow a partire dai metadati SQL e dal ResultSet
	 * @param rsmd
	 * @param rs
	 * @param colCount
	 * @return DataRow
	 * @throws Exception
	 */
	protected DataRow storeResult(ResultSetMetaData rsmd, ResultSet rs,
			int colCount, int rowCount) throws Exception {
		DataRow dr = new DataRow(colCount + 1);
		String cn = "ROW_NUMBER";
		int ct = Types.INTEGER;
		Integer cv = new Integer(rowCount);
		DataField rnum = new DataField(cn,ct,cv);
		dr.addColumn(0,rnum);
		for (int i = 1; i <= colCount; i++) {
			String colname = rsmd.getColumnName(i);
			int coltype = rsmd.getColumnType(i);
			int scale = rsmd.getScale(i);
			Object value = null;
			if (coltype == Types.DATE)
				value = rs.getDate(i);
			else if (coltype == Types.TIMESTAMP)
				value = rs.getTimestamp(i);
			else
				value = rs.getObject(i);
			//Object value = rs.getObject(i);
			DataField df = new DataField(colname, coltype, value, scale);
			String label = rsmd.getColumnLabel(i);
			if (label != null)
				df.set_description(label);
			dr.addColumn(i,df);
		}
		return dr;
	}

	/**
	 * ritorna il risultato di una SELECT in forma di OB
	 * @return ObjectBean
	 */
	public ObjectBean get_result() {
		return _result;
	}

	
	/**
	 * ritorna il risultato della query sotto forma di DataSheet
	 */
	public DataSheet getDatasheet() {
		if (get_result() == null)
			return null;
		DataSheet ds = new DataSheet(_result);
		return ds;
	}
	
	
	/**
	 * imposta la lista dei parametri
	 * @param parms
	 */
	public void set_inputParms(List parms) {
		_inputParms = parms;
	}

	/**
	 * ritorna il numero di righe generate da una SELECT
	 * @return int
	 */
	public int get_rowsFetched() {
		return _rowsFetched;
	}

	
}
