package it.pf.dbaccess;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
/**
 * Classe esecutore di statements SQL
 * solo per SELECT, mantiene aperta la connessione e consente
 * di accedere al result set (solo per compatibilit con precedenti 
 * implementazioni). Si consiglia di usare il metodo next()
 * per ottenere la riga successiva (DataRow) dal cursore.
 * Classe costruita soprattutto per processi batch.
 * @author Fabbri_P
 *
 */
public class SQLBigSelect extends SQLBase {

	private ResultSet _rs = null;
	private ResultSetMetaData _rsmd = null;
	
	private int _colCount = 0;
	private int _rowcnt = 1;
	/**
	 * costruttore: riceve in input un OB con lo statement SQL da eseguire
	 * e inizializza la classe
	 * @param beanDescr
	 */
	public SQLBigSelect(ObjectBean beanDescr) {
		ObjectBeanAttribute ba = null;
		try {
			ba = beanDescr.getAttribute("name");
			_name = (String) ba.get_value();
			ba = beanDescr.getAttribute("query");
			_query = (String) ba.get_value();
			_valid = true;
		} catch (ClassCastException ce) {
			_valid = false;
		} catch (NullPointerException np) {
			_valid = false;
		}

		if (_query.indexOf('?') == -1)
			_hasInputParms = false;
		else
			_hasInputParms = true;

		if (_query.length() > 0) {
			String[] words = _query.split(" ");
			_queryType = words[0].trim();
		} else {
			_queryType = null;
			_valid = false;
		}

	}

	/**
	 * costruttore: riceve in input nome e statement da eseguire 
	 * e inizializza la classe
	 * @param name
	 * @param query
	 */
	public SQLBigSelect(String name, String query) {
		_name = name;
		_query = query;
		_valid = true;

		if (_query.indexOf('?') == -1)
			_hasInputParms = false;
		else
			_hasInputParms = true;

		if (_query.length() > 0) {
			String[] words = _query.split(" ");
			_queryType = words[0].trim();
		} else {
			_queryType = null;
			_valid = false;
		}

	}


	/**
	 * esegue lo statement ricevendo la connessione gi&agrave; inizializzata
	 * @param con
	 * @throws Exception
	 */
	public void execute(Connection con) throws Exception {

		if (!is_valid())
			throw new Exception("SQLCommand not valid: not correctly initialized!");

		if (con == null)
			throw new Exception("Connection null");

		try {
			executeInternal(con);
		} catch (SQLException sqle) {
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		}
		return;
	}
	
	/**
	 * metodo interno richiamato dai metodi execute:
	 * analizza lo statement e se si tratta di SELECT
	 * genera il risultato sotto forma di OB,
	 * altrimenti imposta il numero di righe influenzate
	 * dallo statement eseguito
	 * @param con
	 * @throws Exception
	 */
	private void executeInternal(Connection con) throws Exception {
		PreparedStatement statement = null;

		statement = con.prepareStatement(_query);
		if (_hasInputParms) {
			if (_inputParms == null)
				throw new Exception("Input parameters null");
			else {
	            DataField inputParameterDataField = null;
	            for (int i = 0; i < _inputParms.size(); i++) {
	                inputParameterDataField = (DataField) _inputParms.get(i);
					if ( (inputParameterDataField.getSqlType() == Types.CHAR) 
						|| (inputParameterDataField.getSqlType() == Types.VARCHAR))
		                statement.setObject(
			                    i + 1,
			                    inputParameterDataField.getStringValue(),
			                    inputParameterDataField.getSqlType());
					else
		                statement.setObject(
		                    i + 1,
		                    inputParameterDataField.getObjectValue(),
		                    inputParameterDataField.getSqlType());
	            } 
			}
		}
		if (_queryType.equalsIgnoreCase("SELECT")) {
			_rs = statement.executeQuery();
			_rsmd = _rs.getMetaData();

			_colCount = _rsmd.getColumnCount();

			_rowsFetched = _rs.getFetchSize();
			
		} 
		
		return;
	}
	
	/**
	 * Ritorna la riga successiva del result set in forma di DataRow
	 * @return
	 */
	public DataRow next() {
		try {
			if (_rs.next()) {
				return storeResult(_rsmd, _rs, _colCount, _rowcnt++);
			} else
				return null;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * Chiude il result set
	 *
	 */
	public void rsclose() {
		try {
			_rs.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public ResultSet get_rs() {
		return _rs;
	}

	public ResultSetMetaData get_rsmd() {
		return _rsmd;
	}

	/**
	 * Ritorna una nuova connessione a partire dai parametri letti
	 * nell'OB datasource, l'OB si ottiene di solito dal configSingleton, es.:
	 * 	ObjectBean DS = configSingleton.getFilteredAttribute("CONFIG:DATA_SOURCE");
	 * @param DS
	 * @return
	 */
	public Connection getConnection(ObjectBean DS) {
		//ObjectBean DS = config.getFilteredAttribute("CONFIG:DATA_SOURCE");
		String DriverName = DS.getAttributeValueAsString("drivername");
		String url = DS.getAttributeValueAsString("jdbcurl");
		String uid = DS.getAttributeValueAsString("username");
		String pwd = DS.getAttributeValueAsString("password");
		Connection con = null;
		try {
			Class.forName(DriverName).newInstance();
			con = DriverManager.getConnection(url,uid,pwd);
		} catch (java.lang.ClassNotFoundException e) {
			System.out.println("ClassNotFoundException in getConnection");
			System.out.println(e.getMessage());
			return null;
		} catch (java.lang.IllegalAccessException e) {
			System.out.println("IllegalAccessException in getConnection");
			System.out.println(e.getMessage());
			return null;
		} catch (java.lang.InstantiationException e) {
			System.out.println("InstantiationException in getConnection");
			System.out.println(e.getMessage());
			return null;
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
		return con;		
	}
	
}
