package it.pf.dbaccess;

import java.math.BigDecimal;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.Date;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import com.edb.util.PGobject;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
/**
 * Classe esecutore di statements SQL
 * nel caso di SELECT imposta il risultato come OB dal nome ROWS
 * con OBA con chiave=ROW e valore=DataRow
 * 
 * @author Fabbri_P
 *
 */
public class SQLCommand extends SQLBase {

	
	/**
	 * costruttore: riceve in input un OB con lo statement SQL da eseguire
	 * e inizializza la classe
	 * @param beanDescr
	 */
	public SQLCommand(ObjectBean beanDescr) {
		ObjectBeanAttribute ba = null;
		try {
			ba = beanDescr.getAttribute("name");
			_name = (String) ba.get_value();
			ba = beanDescr.getAttribute("query");
			_query = ((String) ba.get_value()).trim();
			_valid = true;
		} catch (ClassCastException ce) {
			_valid = false;
		} catch (NullPointerException np) {
			_valid = false;
		}

		if (_query.indexOf('?') == -1)
			_hasInputParms = false;
		else
			_hasInputParms = true;

		if (_query.length() > 0) {
			String[] words = _query.split(" ");
			_queryType = words[0].trim();
		} else {
			_queryType = null;
			_valid = false;
		}

	}

	/**
	 * costruttore: riceve in input nome e statement da eseguire 
	 * e inizializza la classe
	 * @param name
	 * @param query
	 */
	public SQLCommand(String name, String query) {
		_name = name;
		_query = query.trim();
		_valid = true;

		if (_query.indexOf('?') == -1)
			_hasInputParms = false;
		else
			_hasInputParms = true;

		if (_query.length() > 0) {
			String[] words = _query.split(" ");
			_queryType = words[0].trim();
		} else {
			_queryType = null;
			_valid = false;
		}

	}


	/**
	 * esegue lo statement acquisendo la connessione dal DataSource
	 * ricevuto nel parametro
	 * @param ds
	 * @throws Exception
	 */
	public void execute(DataSource ds) throws Exception {
		if (ds == null) {
			System.out.println("***DataSource not initialized!");
			return;
		}

		if (!is_valid())
			throw new Exception("SQLCommand not valid: not correctly initialized!");

		_result = new ObjectBean("ROWS");
		Connection con = ds.getConnection();
		con.setAutoCommit(true);
		if (con == null)
			throw new Exception("Connection null: DataSource not initialized?");

		try {
			executeInternal(con);
			if (!con.getAutoCommit())
				con.commit();
			con.close();
		} catch (SQLException sqle) {
			try {con.close();} catch (Exception e) {}
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		} finally {
			if (con != null) {
				try {con.close();} catch (Exception e) {}
			}
		}
		return;
	}

	/**
	 * esegue lo statement ricevendo la connessione gi&agrave; inizializzata
	 * @param con
	 * @throws Exception
	 */
	public void execute(Connection con) throws Exception {
		if (con == null) {
			System.out.println("***Connection not initialized!");
			return;
		}

		if (!is_valid())
			throw new Exception("SQLCommand not valid: not correctly initialized!");

		_result = new ObjectBean(Constants.ROWS);

		if (con == null)
			throw new Exception("Connection null");

		try {
			executeInternal(con);
		} catch (SQLException sqle) {
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		}
		return;
	}
	
	/**
	 * metodo interno richiamato dai metodi execute:
	 * analizza lo statement e se si tratta di SELECT
	 * genera il risultato sotto forma di OB,
	 * altrimenti imposta il numero di righe influenzate
	 * dallo statement eseguito
	 * @param con
	 * @throws Exception
	 */
	private void executeInternal(Connection con) throws Exception {
		PreparedStatement statement = null;
		ResultSet rs = null;
		ResultSetMetaData rsmd = null;

		statement = con.prepareStatement(_query);
		//rsmd = statement.getMetaData();
		if (_hasInputParms) {
			if (_inputParms == null)
				throw new Exception("Input parameters null");
			else {
	            DataField inputParameterDataField = null;
	            for (int i = 0; i < _inputParms.size(); i++) {
	                inputParameterDataField = (DataField) _inputParms.get(i);
	                statement.setObject(
		                    i + 1,
		                    inputParameterDataField.getObjectValue(),
		                    inputParameterDataField.getSqlType());
					/*
					if ( (inputParameterDataField.getSqlType() == Types.CHAR) 
						|| (inputParameterDataField.getSqlType() == Types.VARCHAR))
		                statement.setObject(
			                    i + 1,
			                    inputParameterDataField.getStringValue(),
			                    inputParameterDataField.getSqlType());
					else
		                statement.setObject(
		                    i + 1,
		                    inputParameterDataField.getObjectValue(),
		                    inputParameterDataField.getSqlType());
					mapObjectToStatement(statement,i+1,
							inputParameterDataField.getObjectValue(),
							rsmd.getColumnDisplaySize(i+1),
							inputParameterDataField.getSqlType());
		            */
	            } 
			}
		}
		if (_queryType.equalsIgnoreCase("SELECT")) {
			rs = statement.executeQuery();
			rsmd = rs.getMetaData();

			int colCount = rsmd.getColumnCount();

			_rowsFetched = rs.getFetchSize();
			
			int rowcnt = 1;
			while (rs.next()) {
				DataRow dr = storeResult(rsmd, rs, colCount, rowcnt++);
				_result.setAttribute(Constants.ROW,dr);
			}
			rs.close();	
			_rowsFetched = _result.get_table().size();
		} else {
            int rows = statement.executeUpdate();
			set_rowsAffected(rows);
		}
		
		return;
	}

	/**
	 * esegue la call ricevendo la connessione gi&agrave; inizializzata
	 * @param con
	 * @throws Exception
	 */
	public void executeCall(Connection con) throws Exception {
		if (con == null) {
			System.out.println("***Connection not initialized!");
			return;
		}

		if (!is_valid())
			throw new Exception("SQLCall not valid: not correctly initialized!");


		if (con == null)
			throw new Exception("Connection null");

		try {
			executeInternalCall(con);
		} catch (SQLException sqle) {
			System.out.println("ERROR SQL on call:" + _query);
			throw new Exception(sqle.getMessage());
		}
		return;
	}
	
	/**
	 * metodo interno richiamato dai metodi executeCall:
	 * prepara la call (stored procedure) e imposta i 
	 * parametri di input e output
	 * @param con
	 * @throws Exception
	 */
	private void executeInternalCall(Connection con) throws Exception {
		CallableStatement statement = null;
		ResultSet rs = null;
		ResultSetMetaData rsmd = null;

		statement = con.prepareCall(_query);
		ParameterMetaData parms = statement.getParameterMetaData();
		int incnt = 0;  //input parameters count
		int outcnt = 0; //output parameters count
		for (int i=1; i<=parms.getParameterCount(); i++ ) {
			int pmode = parms.getParameterMode(i);
			int ptype = parms.getParameterType(i);
			if (pmode == ParameterMetaData.parameterModeIn) {
				DataField inputParameterDataField = (DataField) _inputParms.get(incnt);
				if (inputParameterDataField == null)
					throw new Exception("Failed to call procedure: required in parameter missing.");
                statement.setObject( i , inputParameterDataField.getObjectValue(), ptype);
				incnt++;
			} else if (pmode == ParameterMetaData.parameterModeOut) {
				Object op = _outputParms.get(outcnt);
				if (op == null)
					throw new Exception("Failed to call procedure: required out parameter type missing.");
				statement.registerOutParameter(outcnt, ptype);
				outcnt++;
			}
		}
		boolean state = statement.execute();
		
		return;
	}
	
}
