package it.pf.dbaccess;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.List;

import javax.sql.DataSource;

import com.edb.util.PGobject;

import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
/**
 * Acquisisce i metadati di una tabella
 * 
 * @author Fabbri_P
 *
 */
public class SQLMetaData extends SQLBase {

	private String qualifiedtablename;
	
	private String selectstatement;
	
	private ResultSetMetaData metadata;
	
	/**
	 * costruttore: riceve in input il nome qualificato
	 * della tabella di cui interrogare i metadati
	 * costruisce lo statement fittizio di select
	 * e inizializza la classe
	 * @param beanDescr
	 */
	public SQLMetaData(String tablename) {
		qualifiedtablename = tablename;
		selectstatement = "SELECT * FROM "+qualifiedtablename;
	}



	/**
	 * esegue lo statement acquisendo la connessione dal DataSource
	 * ricevuto nel parametro
	 * @param ds
	 * @throws Exception
	 */
	public void execute(DataSource ds) throws Exception {

		Connection con = ds.getConnection();
		if (con == null)
			throw new Exception("Connection null: DataSource not initialized?");

		try {
			executeInternal(con);
			con.close();
		} catch (SQLException sqle) {
			try {con.close();} catch (Exception e) {}
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		} finally {
			if (con != null) {
				try {con.close();} catch (Exception e) {}
			}
		}
		return;
	}

	/**
	 * esegue lo statement ricevendo la connessione gi&agrave; inizializzata
	 * @param con
	 * @throws Exception
	 */
	public void execute(Connection con) throws Exception {


		if (con == null)
			throw new Exception("Connection null");

		try {
			executeInternal(con);
		} catch (SQLException sqle) {
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		}
		return;
	}
	
	/**
	 * metodo interno richiamato dai metodi execute:
	 * acquisisce i metadati dal preparedStatement fittizio
	 * @param con
	 * @throws Exception
	 */
	private void executeInternal(Connection con) throws Exception {
		PreparedStatement statement = null;

		statement = con.prepareStatement(selectstatement);
		setMetadata(statement.getMetaData());
		/*
		for (int i = 1; i <= metadata.getColumnCount(); i++) {
			String colname = metadata.getColumnName(i);
			String collabel = metadata.getColumnLabel(i);
			int coltype = metadata.getColumnType(i);
			int dispsize = metadata.getColumnDisplaySize(i);
			String typename = metadata.getColumnTypeName(i);
			String schemaname = metadata.getSchemaName(i);
			String catname = metadata.getCatalogName(i);
			int nullable = metadata.isNullable(i);
			System.out.println(colname+" - "+collabel+" - "+typename+" ("+coltype+") - null="+nullable+" - "+dispsize);
		}
		*/
		return;
	}



	/**
	 * @return Returns the metadata.
	 */
	public ResultSetMetaData getMetadata() {
		return metadata;
	}



	/**
	 * @param metadata The metadata to set.
	 */
	public void setMetadata(ResultSetMetaData metadata) {
		this.metadata = metadata;
	}

	
	
}
