package it.pf.dbaccess;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.*;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.sql.DataSource;

public class SQLoader {
	
	private Properties _loadProp;
	
	private String _query;
	
	private ArrayList _inparms;
	
	private String[] _colnames;
	private int[] _coltypes;

	private ResultSetMetaData _rsmd = null; 
	
	private long _rows = 0;
	
	private int _rowsToCommit = 0;
	
	private int _pos = 0;

	private static String _dateFormat = "DD/MM/YYYY";
	private static String _timestampFormat = "DD/MM/YYYY hh.mm.ss";
	private static int _COMMIT_COUNT = 1000;
	private static int _COMMIT_COUNT_DEFAULT = 1000;
	
	public static void main(String[] args) throws Exception {
		SQLoader inst = new SQLoader(args[0]);
		inst.run();
	}

	public SQLoader(String propFileName) {
		if (propFileName != null) {
			FileInputStream ftmp;
			_loadProp = new Properties();
			try {
				ftmp = new FileInputStream(new File(propFileName));
				_loadProp.load(ftmp);
				ftmp.close();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			String tmp = _loadProp.getProperty("db.commit.count");
			if (tmp == null)
				_COMMIT_COUNT = _COMMIT_COUNT_DEFAULT;
			else {
				try {
					_COMMIT_COUNT = (new Integer(tmp)).intValue();
				} catch(NumberFormatException e) {
					_COMMIT_COUNT = _COMMIT_COUNT_DEFAULT;
				}
			}
			tmp = _loadProp.getProperty("timestamp.format");
			if (tmp != null)
				_timestampFormat = tmp;
			tmp = _loadProp.getProperty("date.format");
			if (tmp != null)
				_dateFormat = tmp;
		}
	}

	public void run() {
		
		if (_loadProp == null) {
			System.out.println("properties not initialized!");
			return;
		}

		Calendar t_start = Calendar.getInstance();
		
		String tableName = _loadProp.getProperty("db.table.name");
		String inFileName = _loadProp.getProperty("input.file.name");

		System.out.println("LOAD START for "+tableName);
		System.out.println("***System date: "+t_start.getTime().toString());
		
		Connection con = null;
		String line = null;
		try {
			con = getConnection();
			con.setAutoCommit(true);
			prepareRsmd(tableName,con);
			prepareStatement(tableName,con);
			con.close();

			BufferedReader rdr = new BufferedReader(new FileReader(inFileName));
			con = getConnection();
			_rowsToCommit = 0;
			while ((line = rdr.readLine()) != null) {
				loadRow(line,con);
				_rowsToCommit++;
				if (_rowsToCommit >= _COMMIT_COUNT) {
					con.close();
					con = getConnection();
					Calendar now = Calendar.getInstance();
					System.out.println("*** "+_rowsToCommit+" rows committed at "+now.getTime().toString()+" -- "+_rows+" rows loaded so far.");
					_rowsToCommit = 0;
				}
			}
			rdr.close();
			con.close();
		} catch (Exception e) {
			System.out.println("Exception when processing line n."+(_rows+1)+": ");
			System.out.println(line);
			try {
				con.close();
			} catch (SQLException e1) {
				e1.printStackTrace();
			}
			e.printStackTrace();
		}
		Calendar t_end = Calendar.getInstance();
		System.out.println("LOAD END:"+_rows+" tuples loaded");
		long elapsed = t_end.getTimeInMillis() - t_start.getTimeInMillis();
		System.out.println("***Elapsed time:" + elapsed + " millis.");

	}


	private void prepareRsmd(String tableName, Connection con) throws Exception {
		String tmpselect = "SELECT * FROM "+tableName+" LIMIT 1";
		PreparedStatement statement = null;

		statement = con.prepareStatement(tmpselect);
		statement.executeQuery();
		ResultSetMetaData rsmd = statement.getMetaData();
        int ColCount = rsmd.getColumnCount();
		set_inparms(new ArrayList(ColCount));
		_colnames = new String[ColCount];
		_coltypes = new int[ColCount];
		for (int i = 0; i < ColCount; i++) {
			_colnames[i] = rsmd.getColumnName(i + 1);
			_coltypes[i] = rsmd.getColumnType(i + 1);
		}		
        set_rsmd(rsmd);
		return;
	}

	private void prepareStatement(String tableName, Connection con) throws Exception {
		ResultSetMetaData rsmd = get_rsmd();

        int ColCount = rsmd.getColumnCount();

		StringBuffer insertStatement = new StringBuffer("INSERT INTO "+tableName+" (");
		StringBuffer values = new StringBuffer("");
		for (int i = 0; i < ColCount; i++) {
            insertStatement.append(rsmd.getColumnName(i + 1));
			if (i == (ColCount-1)) { 
	            insertStatement.append(") ");
				values.append("?)");
			} else {
	            insertStatement.append(", ");
				values.append("?,");
			}
		}
		insertStatement.append(" VALUES (");
		insertStatement.append(values);
		set_query(insertStatement.toString());
		return ;
	}

	
	private void loadRow(String line, Connection con) throws Exception {
		
		SQLCommand cmd = null;
		List inparms = null;
		int result = 0;

		
		cmd = new SQLCommand("load",get_query());
		set_inparms(new ArrayList(_rsmd.getColumnCount()));
		mapLineToParms(line);
		cmd.set_inputParms(get_inparms());
		cmd.execute(con);
		result = cmd.get_rowsAffected();
		if (result > 0) {
			_rows++;
			//System.out.println("Row n."+_rows+" loaded");
		}
	}
	
	private void mapLineToParms(String line) throws Exception {
		String maptype =_loadProp.getProperty("mapping.type");
		if (maptype.equalsIgnoreCase("mapper")) {
			String mapfile =_loadProp.getProperty("mapping.file.name");
			mapWithMapperFile(line,mapfile);
		} else {
			String charsep = _loadProp.getProperty("mapping.char");
			mapWithCharSep(line,charsep);
		}
	}

	private void mapWithCharSep(String line,String charsep) throws Exception {
		Properties map = new Properties();
		
		_pos = 0;		//reset to start of line
		
		ResultSetMetaData rsmd = get_rsmd();

		StringTokenizer token = new StringTokenizer(line, charsep);

        int ColCount = rsmd.getColumnCount();
		for (int i = 0; i < ColCount; i++) {
			String colname = _colnames[i];
			int coltype = _coltypes[i];
			if (coltype == 1111)
				coltype = Types.DATE;
			try {
				String sdato = token.nextToken().trim();
				DataField df = null;
				if (sdato.length() == 0) {
					df = new DataField(colname,coltype,null);
				} else {
					Object dato = getObjectValue(coltype,sdato);
					df = new DataField(colname,coltype,dato);
				}
				_inparms.add(df);
			} catch(NoSuchElementException ne) {
				DataField df = new DataField(colname,coltype,null);
				_inparms.add(df);
				break;
			}
		}
	}

	private void mapWithMapperFile(String line,String mapFileName) throws Exception {
		Properties map = new Properties();
		FileInputStream ftmp = new FileInputStream(new File(mapFileName));
		map.load(ftmp);
		
		_pos = 0;		//reset to start of line
		
		ResultSetMetaData rsmd = get_rsmd();

        int ColCount = rsmd.getColumnCount();
		for (int i = 0; i < ColCount; i++) {
			String colname = _colnames[i];
			int coltype = _coltypes[i];
			if (coltype == 1111)
				coltype = Types.DATE;
			String sdato = getDataFromFile(colname,map,line);
			DataField df = null;
			if (sdato.startsWith("?")) {
				df = new DataField(colname,coltype,null);
			} else {
				Object dato = getObjectValue(coltype,sdato);
				df = new DataField(colname,coltype,dato);
			}
			_inparms.add(df);
		}
	}

	private String getDataFromFile(String colname, Properties map, String line) throws Exception {
		
		String tmp = map.getProperty(colname.toUpperCase());
		StringTokenizer stk = new StringTokenizer(tmp, ",");
		String sp = stk.nextToken();
		String sl = stk.nextToken();
		int p = 0;
		int l = (new Integer(sl)).intValue();
		if (sp.equalsIgnoreCase("*")) {
			p = _pos;
			_pos += l;
		} else {
			p = (new Integer(sp)).intValue();
			_pos += l;
		}
		String dato = line.substring(p,p+l);
		return dato.trim();
	}
	
    public Object getObjectValue(int sqlType, String stringValue) {
        if (stringValue == null) {
            return null;
        }
        Object objectValue = null;
        switch (sqlType) {
            case Types.CHAR:
            case Types.VARCHAR:
            case Types.LONGVARCHAR:
                objectValue = stringValue;
                break;
            case Types.BIGINT:
                objectValue = new BigInteger(stringValue.trim());
                break;
            //case Types.BIT:
            case Types.DECIMAL:
            case Types.NUMERIC:
                objectValue = new BigDecimal(stringValue.trim());
                break;
			case Types.DOUBLE:
                objectValue = new Double(stringValue.trim());
                break;
            case Types.REAL:
            case Types.FLOAT:
                objectValue = new Float(stringValue.trim());
                break;
            case Types.INTEGER:
            case Types.SMALLINT:
            case Types.TINYINT:
                objectValue = new Integer(stringValue.trim());
                break;
            case Types.BINARY:
            case Types.LONGVARBINARY:
            case Types.VARBINARY:
                objectValue = stringValue.getBytes();
                break;
            //case 1111:
			case Types.DATE:
                objectValue = getObjectValueForDate(stringValue);
                break;
            case Types.TIMESTAMP:
                objectValue = getObjectValueForTimeStamp(stringValue);
                break;
            default:
                objectValue = stringValue;
        }
        return objectValue;
	}

	    public Object getObjectValueForDate(String stringDateValue) {
	        if (stringDateValue == null) {
	            return null;
	        }
	        Date result = null;
	        try {
	            DateFormatter pfDateFormatter = new DateFormatter(_dateFormat);
				String dateToFormat = null;
				if (stringDateValue.length() > _dateFormat.length())
					dateToFormat = stringDateValue.substring(0,_dateFormat.length());
				else
					dateToFormat = stringDateValue;
	            pfDateFormatter.format(dateToFormat);
	            int day = (pfDateFormatter.getDay() != -1 ? pfDateFormatter.getDay() : 0);
	            int month = (pfDateFormatter.getMonth() != -1 ? pfDateFormatter.getMonth() : 0);
	            int year = (pfDateFormatter.getYear() != -1 ? pfDateFormatter.getYear() : 0);
	            result = new Date(year - 1900, month - 1, day);
	        }
	        catch (ParseException pe) {
	        }
	        return result;
	    } 

	    public Object getObjectValueForTimeStamp(String stringTimeStampValue) {
	        if (stringTimeStampValue == null) {
	            return null;
	        }
	        Timestamp result = null;
	        try {
	            DateFormatter pfDateFormatter = new DateFormatter(_timestampFormat);
	            pfDateFormatter.format(stringTimeStampValue);
	            int day = (pfDateFormatter.getDay() != -1 ? pfDateFormatter.getDay() : 0);
	            int month = (pfDateFormatter.getMonth() != -1 ? pfDateFormatter.getMonth() : 0);
	            int year = (pfDateFormatter.getYear() != -1 ? pfDateFormatter.getYear() : 0);
	            int hour = (pfDateFormatter.getHours() != -1 ? pfDateFormatter.getHours() : 0);
	            int minute = (pfDateFormatter.getMinutes() != -1 ? pfDateFormatter.getMinutes() : 0);
	            int second = (pfDateFormatter.getSeconds() != -1 ? pfDateFormatter.getSeconds() : 0);
	            int nano = 0;
	            result = new Timestamp(year - 1900, month - 1, day, hour, minute, second, nano);
	        }
	        catch (ParseException pe) {
	        }
	        return result;
	    } 
	
	
	public ArrayList get_inparms() {
		return _inparms;
	}
	

	public void set_inparms(ArrayList _inparms) {
		this._inparms = _inparms;
	}
	

	public Properties get_loadProp() {
		return _loadProp;
	}
	

	public void set_loadProp(Properties prop) {
		_loadProp = prop;
	}
	

	public String get_query() {
		return _query;
	}
	

	public void set_query(String _query) {
		this._query = _query;
	}

	public ResultSetMetaData get_rsmd() {
		return _rsmd;
	}
	

	public void set_rsmd(ResultSetMetaData _rsmd) {
		this._rsmd = _rsmd;
	}
	
	private Connection getConnection() {
			String DriverName = _loadProp.getProperty("db.driver");
			String url = _loadProp.getProperty("db.url");
			String uid = _loadProp.getProperty("db.user");
			String pwd = _loadProp.getProperty("db.password");
			Connection con = null;
			try {
				Class.forName(DriverName).newInstance();
				con = DriverManager.getConnection(url,uid,pwd);
			} catch (java.lang.ClassNotFoundException e) {
				System.out.println("ClassNotFoundException in getConnection");
				System.out.println(e.getMessage());
				return null;
			} catch (java.lang.IllegalAccessException e) {
				System.out.println("IllegalAccessException in getConnection");
				System.out.println(e.getMessage());
				return null;
			} catch (java.lang.InstantiationException e) {
				System.out.println("InstantiationException in getConnection");
				System.out.println(e.getMessage());
				return null;
			} catch (SQLException e) {
				e.printStackTrace();
			}
			return con;
		}

}