package it.pf.dbaccess;


import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;

import javax.sql.DataSource;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.sql.SQLException;
import java.util.Properties;
import java.util.Vector;

import org.apache.commons.dbcp.BasicDataSource;

/**
 * Inizializza un DataSource di tipo BasicDataSource (apache commons project)
 * @author Fabbri_P
 *
 */
public class SetupDataSource {

	private BasicDataSource _ds;
	private Properties _params;
	
	/**
	 * costruttore: i parametri del DS sono contenuti nell'OB passato come
	 * parametro
	 * @param parms
	 */
	public SetupDataSource(ObjectBean parms) {
		_params = new Properties();
		Vector attributes = (Vector)parms.get_table();
		for (int i=0; i<attributes.size(); i++) {
			ObjectBeanAttribute attr = (ObjectBeanAttribute)attributes.elementAt(i);
			if (attr.get_value() instanceof String)
				_params.setProperty(attr.get_key(),(String)attr.get_value());
		}
		setupDataSource();
	}
	
	/**
	 * costruttore di default: acquisisce i parametri a partire
	 * dal file "init.properties" caricato tramite ClassLoader
	 * (via ...class.getResource(..))
	 */
	public SetupDataSource() {
		URL curl = SetupDataSource.class.getResource("init.properties");
		if (curl != null) {
			String cfile = curl.getPath();
			if (System.getProperty("os.name").startsWith("W"))
				cfile = cfile.substring(1);
			FileInputStream ftmp;
			_params = new Properties();
			try {
				_params.load(new FileInputStream(cfile));
				setupDataSource();
			} catch (FileNotFoundException e) {
				setupDefaultDataSource();
				e.printStackTrace();
			} catch (IOException e) {
				setupDefaultDataSource();
				e.printStackTrace();
			}
		} else {
			setupDefaultDataSource();
		}
	}
	
	/**
	 * imposta il DS con parametri di defaults
	 */
    private void setupDefaultDataSource() {
        _ds = new BasicDataSource();		
        _ds.setDriverClassName("COM.ibm.db2.jdbc.app.DB2Driver");
        _ds.setUsername("db2admin");
        _ds.setPassword("db2admin");
        _ds.setUrl("jdbc:db2:DATIPF");
    }

	/**
	 * imposta il DS con i parametri passati nel costruttore
	 */
    private void setupDataSource() {
        _ds = new BasicDataSource();		
        _ds.setDriverClassName(_params.getProperty("drivername"));
        _ds.setUsername(_params.getProperty("username"));
        _ds.setPassword(_params.getProperty("password"));
        _ds.setUrl(_params.getProperty("jdbcurl"));
    }
	
	/**
	 * metodo statico per eseguire il display del numero di connessioni
	 * attive e il numero di connessioni idle
	 * @param ds
	 * @throws SQLException
	 */
    public static void printDataSourceStats(DataSource ds) throws SQLException {
        BasicDataSource bds = (BasicDataSource) ds;
        System.out.println("NumActive: " + bds.getNumActive());
        System.out.println("NumIdle: " + bds.getNumIdle());
    }

	/**
	 * distrugge il ds chiudendo tutte le connessioni
	 * @param ds
	 * @throws SQLException
	 */
    public static void shutdownDataSource(DataSource ds) throws SQLException {
        BasicDataSource bds = (BasicDataSource) ds;
        bds.close();
    }

	/**
	 * ritorna il DataSource
	 * @return DataSource
	 */
	public DataSource getDataSource() {
		return _ds;
	}

	/**
	 * @return Returns the _params.
	 */
	public Properties getProperties() {
		return _params;
	}

}
