package it.pf.dbaccess;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.sql.Connection;
import java.util.List;
import java.util.Properties;

import javax.sql.DataSource;

import it.pf.base.ConfigSingleton;
import it.pf.base.ObjectBean;
import it.pf.utils.Tools;

public class SingleStatement {

	public SingleStatement() {
		ds = ConfigSingleton.getDataSource();
		setError(null);
	}
	
	public SingleStatement(String dataSourceName) {
		ds = ConfigSingleton.getDataSource(dataSourceName);
		setError(null);
	}
	
	private ObjectBean _object;
	private String error;
	private DataSource ds;
	
	private int rowsaffected = 0;
	private int rowsfetched = 0;
	
	public String getError() {
		return error;
	}

	public void setError(String error) {
		this.error = error;
	}

	public void exec(Connection con, String queryFile, List inparms) {
		if (con == null) {
			setError("***Connessione non inizializzata!");
			System.out.println("***Connessione non inizializzata!");
			return;
		}
		SQLCommand cmd = null;
		String query = readQueryFile(queryFile);
		if (query == null) {
			setError("***Lettura file:"+queryFile+" impossibile!");
			_object = null;
			return;
		}
		cmd = new SQLCommand("test",query);
		if (inparms != null)
			cmd.set_inputParms(inparms);
		try {
			cmd.execute(con);
			_object = cmd.get_result();
			rowsaffected = cmd.get_rowsAffected();
			rowsfetched = cmd.get_rowsFetched();
		} catch (Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			setError("***Esecuzione statement:"+e.getMessage());
			_object = null;
			return;
		}
		return;
	}

	public void exec(String queryFile, List inparms) {
		if (ds == null) {
			setError("***DataSource non inizializzato!");
			System.out.println("***DataSource non inizializzato!");
			return;
		}
		SQLCommand cmd = null;
		String query = readQueryFile(queryFile);
		if (query == null) {
			setError("***Lettura file:"+queryFile+" impossibile!");
			_object = null;
			return;
		}
		cmd = new SQLCommand("test",query);
		if (inparms != null)
			cmd.set_inputParms(inparms);
		try {
			cmd.execute(ds);
			_object = cmd.get_result();
			rowsaffected = cmd.get_rowsAffected();
			rowsfetched = cmd.get_rowsFetched();
		} catch (Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			setError("***Esecuzione statement:"+e.getMessage());
			_object = null;
			return;
		}
		return;
	}

	public void execq(String queryStmt, List inparms) {
		if (ds == null) {
			setError("***DataSource non inizializzato!");
			System.out.println("***DataSource non inizializzato!");
			return;
		}
		SQLCommand cmd = null;
		String query = queryStmt;

		cmd = new SQLCommand("test",query);
		if (inparms != null)
			cmd.set_inputParms(inparms);
		try {
			cmd.execute(ds);
			_object = cmd.get_result();
			rowsaffected = cmd.get_rowsAffected();
			rowsfetched = cmd.get_rowsFetched();
		} catch (Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			setError("***Esecuzione statement:"+e.getMessage());
			_object = null;
			return;
		}
		return;
	}
	
	/**
	 * legge il file 'name' usando il classloader, pertanto il percorso relativo del file 
	 * deve essere accessibile dal classpath.
	 * @param name
	 * @return
	 */
	public String readQueryFile(String name) {
		InputStream is = SingleStatement.class.getResourceAsStream(name);
		if (is == null) {  //non accessibile dal classloader
			setError("***Lettura file:"+name+" non accessibile dal classloader.");
			return null;
		}
		byte[] content;
		try {
			int size = is.available();
			content = new byte[size];
			int byteread = 0;
			for (int i=0; i<size; i++) {
				byteread = is.read();
				if (byteread != -1)
					content[i] = (byte)byteread;
			}
			String stmt = new String(content,"ISO-8859-1");
	        return stmt;
		} catch (IOException e) {
			setError("***Lettura file:"+name+" errore:"+e.getMessage());
			e.printStackTrace();
			return null;
		}		
	}

	public ObjectBean get_object() {
		return _object;
	}

	public void set_object(ObjectBean _medici) {
		this._object = _medici;
	}
	
    /**
     * parse SQLquery and return parsed string with parameters 
     * valued from properties
     */
    public String parseQuery(String q, Properties p) {
        java.util.StringTokenizer st =
            new java.util.StringTokenizer(q, "&", true);
        java.lang.String q_prep = "";
        boolean delm = false;

        while (st.hasMoreElements()) {
            String tmp = (String) st.nextElement();
            if ("&".equals(tmp))
                delm = turn(delm);
            else {
                if (delm) {
					String value = p.getProperty(tmp.trim());
					q_prep += value;
                } else
	                q_prep += tmp;
            }
        }
        return q_prep;
    }

    /**
     * switch input on/off
     */
    private boolean turn(boolean onoff) {
        if (onoff)
            return false;
        else
            return true;
    }

	public DataSource getDs() {
		return ds;
	}

	public void setDs(DataSource ds) {
		this.ds = ds;
	}

	/**
	 * @return Returns the rowsaffected.
	 */
	public int getRowsaffected() {
		return rowsaffected;
	}

	/**
	 * @return Returns the rowsfetched.
	 */
	public int getRowsfetched() {
		return rowsfetched;
	}
    
}
