/*
 * ErrorAggregator.java
 *
 * Created on 3 agosto 2005, 11.15
 */

package it.pf.logging;

/**
 *
 * @author filippo
 */
public class ErrorAggregator  implements EntryListener, EndCycleListener {
    
    NotifyImpl iSender;
    int iiWarn;
    int iiExc;
    int iiFatal;
    boolean ibCanRecord = true;
    String isRecordedMessage;
    
    /** Creates a new instance of ErrorAggregator */
    public ErrorAggregator(NotifyImpl pSender) {
        if ( (pSender == null) )
            throw new RuntimeException("ErrorAggregator initialized with null parameters");
        iSender = pSender;
    }
    
    public void addWarn(String psMessage, boolean pInsideCycle) {
        if (pInsideCycle) {
           iiWarn++;
           recordMessage(psMessage);
        }
        else {
            String lsSubject = "WARN: ";
            iSender.sendMessage(lsSubject, psMessage, NotifyImpl.PRIORITY_NORMAL);
        }
            
    }
    
    public void addFatal(String psMessage, boolean pInsideCycle) {
        if (pInsideCycle) {
           iiFatal++;
           recordMessage(psMessage);
        }
        else {
            String lsSubject = "Fatal: ";
            iSender.sendMessage(lsSubject, psMessage, NotifyImpl.PRIORITY_NORMAL);
        }
    }
    
    public void addException(String psMessage, Exception e, boolean pInsideCycle) {
        psMessage += " " + e.getMessage();
        if (pInsideCycle) {
           iiExc++;
           recordMessage(psMessage);
        }
        else {
            String lsSubject = "Exc: ";
            iSender.sendMessage(lsSubject, psMessage, NotifyImpl.PRIORITY_NORMAL);
        }
    }
    
    private void resetCounters() {
        iiWarn = 0;
        iiFatal = 0;
        iiExc = 0;
        ibCanRecord = true;
    }
    
    private void recordMessage(String psMessage) {
        if (ibCanRecord) {
            isRecordedMessage = psMessage;
            ibCanRecord = false;
        }
    }
    
    public void onEndCycle(String psMessage) {
        int iiTotal = iiWarn + iiFatal + iiExc;
        if (iiTotal > 0 ) {
            String lsSubject = "Errors during cycle ";
            String lsMessage = "Fatal: " + iiFatal + " Warn: " + iiWarn + " Exc: " + iiExc;
            lsMessage += "\r\n " + isRecordedMessage;
            iSender.sendMessage(lsSubject, lsMessage, NotifyImpl.PRIORITY_NORMAL);
            resetCounters();
        }
    }
}
