/*
 * LogBasicImpl.java
 *
 * Created on 15 giugno 2005, 11.20
 */

package it.pf.logging;
import java.text.*;
import java.io.*;
import java.util.Calendar;
import java.util.Date;
import java.util.Properties;
/**
 *
 * @author filippo
 */
public class LogBasicImpl implements LogBasic {
    
    public static final short LOG_ERROR_MESSAGE = 0;
    public static final short LOG_STATE_MESSAGE = 1;
    private static String isFileStateLog = "log/state";
    private static String isFileErrorLog = "log/error";
    BufferedWriter logErrorFile = null, logStateFile = null;
    
    SimpleDateFormat iFormatter;
    
    /** Creates a new instance of LogBasicImpl */
    public LogBasicImpl() {
        iFormatter = new SimpleDateFormat("HH:mm:ss");
        init();
    }
    
    public synchronized void createNewLogger(Calendar lCalendar) {
        String lsErrorLog;
        String lsStateLog;
        if (lCalendar == null) {
            lsErrorLog = isFileErrorLog + ".log";
            lsStateLog = isFileStateLog + ".log";
        } else {
            SimpleDateFormat lFyyyyMMdd = new SimpleDateFormat("yyyy-MM-dd");
            String lsDate = lFyyyyMMdd.format(lCalendar.getTime());
            lsErrorLog = isFileErrorLog + "_" + lsDate + ".log";
            lsStateLog = isFileStateLog + "_" + lsDate + ".log";
        }
        try {
            if (logErrorFile != null)
                try {logErrorFile.close();} catch(Exception e1) {}
            if (logStateFile != null)
                try {logStateFile.close();} catch(Exception e1) {}
            logErrorFile = new BufferedWriter(new FileWriter(lsErrorLog, true));
            logStateFile = new BufferedWriter(new FileWriter(lsStateLog, true));
        } catch (Exception e) {
            System.out.println("Error creating log files " + e.getMessage());
        }
    }

    /**
     * init
     */
    private void init() {
		Properties properties = new Properties();
		InputStream ftmp = NotifyImpl.class.getResourceAsStream("log.properties");
		try {
			properties.load(ftmp);
			isFileErrorLog = properties.getProperty("errorlogname");
			isFileStateLog = properties.getProperty("statelogname");
		} catch (IOException e) {
			isFileStateLog = "log/state";
			isFileErrorLog = "log/error";
		}    	
    }
    
    public synchronized void logMsg(String psMessage, short piType) {
        BufferedWriter logFile;
        String lsMessage;
        Date lDate = new Date();
        psMessage = psMessage.replaceAll("\r\n", "\r\n\t\t");
        lsMessage = iFormatter.format(lDate) + "\t" + psMessage + "\r\n";
        if (piType == LOG_ERROR_MESSAGE) {
            logFile = logErrorFile;
        } else {
            logFile = logStateFile;
        }
        try {
            
            logFile.write(lsMessage);
            logFile.flush();
        } catch (Exception e) {
            System.out.println("logger doesn't work !");
        }
    }
    
    public void logLabel(String psMessage) {
        logMsg(psMessage, LOG_STATE_MESSAGE);
    }
    
    public void logWarn(String psMessage) {
        psMessage = "[WARN ] " + psMessage;
        logMsg(psMessage, LOG_ERROR_MESSAGE);
    }
    
    public void logDebug(String psMessage) {
        psMessage = "[DEBUG] " + psMessage;
        logMsg(psMessage, LOG_ERROR_MESSAGE);
    }
    
    public void logInfo(String psMessage) {
        psMessage = "[INFO ] " + psMessage;
        logMsg(psMessage, LOG_ERROR_MESSAGE);
    }
    
    public void logFatal(String psMessage) {
        psMessage = "[FATAL] " + psMessage;
        logMsg(psMessage, LOG_ERROR_MESSAGE);
    }
    
    public void logException(Exception e) {
        logException("", e);
    }
    
    public void logException(String sign, Exception e) {
        logMsg("[EXCEP] " + sign + " " + e.getMessage(), LOG_ERROR_MESSAGE);
        Throwable t = e.getCause();
        java.io.CharArrayWriter myWriter = new java.io.CharArrayWriter();
        e.printStackTrace(new java.io.PrintWriter(myWriter));
        logMsg(myWriter.toString(), LOG_ERROR_MESSAGE);
        if (t != null){
            myWriter.reset();
            t.printStackTrace(new java.io.PrintWriter(myWriter));
            logMsg("Root Cause\r\n" + myWriter.toString(), LOG_ERROR_MESSAGE);
        }
    }
    
}
