/*
 * NotifyImpl.java
 *
 * Created on 26 aprile 2005, 13.51
 */

package it.pf.logging;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.Properties;
import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

/**
 *
 * @author filippo
 */
public class NotifyImpl implements Runnable, Notify {
    
    static final int iiTimeout_ms = 25 *1000;
    static String mailhost = "insert e-mail server here";
    static String from = "insert sender address here";
    final int MAX_QUEUED_MAIL = 5;
    ArrayList iAMailData;
    Properties props;
    boolean ibThreadActive;
    LogBasic iLogBasic;
    final int MAX_SMS_PER_DAY;
    int iiPrevDay = -1;
    int iiSmsAvailable;
    /** Creates a new instance of NotifyImpl */
    public NotifyImpl(LogBasic pLogger, int piMaxSmsPerDay) {
        if (pLogger == null )
            throw new RuntimeException("NotifyImpl initialized with null parameters");
        init();
        iLogBasic = pLogger;
        iAMailData = new ArrayList();
        props = System.getProperties();
        props.put("mail.smtp.host", mailhost);
        props.put("mail.smtp.connectiontimeout", String.valueOf(iiTimeout_ms));
        props.put("mail.smtp.timeout", String.valueOf(iiTimeout_ms));
        ibThreadActive = false;
        MAX_SMS_PER_DAY = piMaxSmsPerDay - 1; // the one reserver for summary
        iiSmsAvailable = MAX_SMS_PER_DAY;
    }
    /**
     * init
     */
    private void init() {
		Properties properties = new Properties();
		InputStream ftmp = NotifyImpl.class.getResourceAsStream("log.properties");
		try {
			properties.load(ftmp);
			mailhost = properties.getProperty("mailhost");
			from = properties.getProperty("from");
		} catch (IOException e) {
			mailhost = "undefined";
			from = "undefined";
		}    	
    }
    /*
    public synchronized boolean sendMessage(String psSubject, String psMessage, int piPriority) {
        System.out.println("Messaggio " + psSubject + " " + psMessage);    
        return true;
    }
    */
    public synchronized boolean sendMessage(String psSubject, String psMessage, int piPriority) {
        try {
            if (iAMailData.size() >=  MAX_QUEUED_MAIL) {
                iLogBasic.logWarn("mail queue is full - message discarded");
                return false;
            }
            iAMailData.add(MailData.createMail(psSubject, psMessage));
            if ((piPriority == PRIORITY_HIGH) || smsStillAvailable() )
                iAMailData.add(MailData.createSms(psSubject, psMessage));
            if (! ibThreadActive) {
                ibThreadActive = true;
                new Thread(this).start();
            }
        } catch (Exception e) {
            iLogBasic.logWarn("Failure in sendMail " + e.getMessage());
            return false;
        }
        return true;
    }
    
    private synchronized MailData removeMessage() {
        if (iAMailData.size() == 0) {
            ibThreadActive = false;
            return null;
        }
        MailData lData = (MailData) iAMailData.remove(0);
        return lData;
    }
    
    public void run() {
        MailData lData;
        while ((lData = removeMessage()) != null) {
            try {
                // Get a Session object
                Session session = Session.getInstance(props, null);
                // construct the message
                Message msg = new MimeMessage(session);
                msg.setFrom(new InternetAddress(from));
                msg.setRecipients(Message.RecipientType.TO,
                        InternetAddress.parse(lData.to, false));
            /*
            if (cc != null)
                msg.setRecipients(Message.RecipientType.CC,
                                        InternetAddress.parse(cc, false));
            if (bcc != null)
                msg.setRecipients(Message.RecipientType.BCC,
                                        InternetAddress.parse(bcc, false));
             */
                msg.setSubject(lData.subject);
                msg.setText(lData.body);
                msg.setHeader("X-Mailer", "TrilLogger");
                msg.setSentDate(new Date());
                // send the thing off
                Transport.send(msg);
            } catch(Exception e) {
                iLogBasic.logWarn("Failure sending e-mail " + e.getMessage());
            }
        }
    }
    
    boolean smsStillAvailable() {
        Calendar lCalendar =  new GregorianCalendar(Locale.ITALY);
        if (lCalendar.get(Calendar.DAY_OF_MONTH) != iiPrevDay ) {
            iiPrevDay = lCalendar.get(Calendar.DAY_OF_MONTH);
            iiSmsAvailable = MAX_SMS_PER_DAY;
        }
        
        if (iiSmsAvailable > 0) {
            iiSmsAvailable--;
            return true;
        } else return false;
    }


/* Test only methods */
    
    public static void main(String[] args) {
        NotifyImpl test = new NotifyImpl(new LogBasicImpl(), 10);
        for (int i = 0 ; i < 12; i++)
            test.sendMessage("invio" + i ,"body", PRIORITY_NORMAL);
        try { Thread.sleep(5000);} catch(Exception e) {}
        for (int i = 0 ; i < 12; i++)
            test.sendMessage("invio" + i ,"body", PRIORITY_NORMAL);
        System.out.println("Terminato main");
    }
}



