package it.pf.soap;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * costruisce un Document object a partire da un file xml
 * (parsing del file xml)
 * @author Fabbri_p
 *
 */
public class DOMbuild {
	
	private Document document;
	private boolean inError;  // indica se il document  null o meno
	
	public DOMbuild(){
		inError = true;
	}
	
	public DOMbuild(String xmlfile) {
		inError = false;
		build(xmlfile);
	}

	public DOMbuild(Document doc) {
		document = doc;
	}
	
	public void build(String xmlfile) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        try {
           DocumentBuilder builder = factory.newDocumentBuilder();
           document = builder.parse( new File(xmlfile) );
           inError = false;
        } catch (SAXParseException spe) {
            inError = true;
           // Error generated by the parser
           System.out.println("\n** Parsing error"
              + ", line " + spe.getLineNumber()
              + ", uri " + spe.getSystemId());
           System.out.println("   " + spe.getMessage() );
           // Use the contained exception, if any
           Exception  x = spe;
           if (spe.getException() != null)
               x = spe.getException();
           x.printStackTrace();
        } catch (SAXException sxe) {
            inError = true;
           // Error generated during parsing)
           Exception  x = sxe;
           if (sxe.getException() != null)
               x = sxe.getException();
           x.printStackTrace();
        } catch (ParserConfigurationException pce) {
            inError = true;
            // Parser with specified options can't be built
            pce.printStackTrace();
        } catch (IOException ioe) {
            inError = true;
           // I/O error
           ioe.printStackTrace();
        }	
	}
	
	public Document getDocument() {
		return document;
	}

	public boolean isInError() {
		return inError;
	}

}
