package it.pf.soap;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Iterator;

import javax.xml.soap.MessageFactory;
import javax.xml.soap.MimeHeaders;
import javax.xml.soap.Name;
import javax.xml.soap.Node;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPBodyElement;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.soap.Text;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Creazione del messaggio SOAP, popolamento del body
 * a partire da un Document object e chiamata all'endPoint
 * @author Fabbri_p
 *
 */
public class SOAPmessage {

	private String baseMsg;
	private URL endPoint;
	
	private SOAPMessage soapMessage;

	/**
	 * default constructor = default SOAPMessage structure
	 */
	public SOAPmessage() {
		/*
		URL curl = SOAPmessage.class.getResource("SOAPbase.xml");	
		if (curl != null) {
			String cfile = curl.getPath();
			if (System.getProperty("os.name").startsWith("W"))
				cfile = cfile.substring(1);
			readBaseMsg(cfile);
		}
		*/
		setBaseMsg();
	}
	
	/**
	 * constructor carica
	 * il file SOAPbase.xml dal path indicato
	 */
	public SOAPmessage(String baseFile) {
		readBaseMsg(baseFile);
	}
	
	
	/**
	 * constructor carica
	 * il file SOAPbase.xml dal path indicato
	 * e imposta l'URL di endPoint
	 */
	public SOAPmessage(String baseFile,String url) {
		readBaseMsg(baseFile);
		try {
			URL endpoint = new URL(url);
			endPoint = endpoint;
		} catch (MalformedURLException e) {
			endPoint = null;
		}		
	}

	
	/**
	 * legge il file xml con le specifiche di base
	 * del messaggio SOAP
	 * @param baseFile
	 */
	/*
	private void readBaseMsg(String baseFile) {
		try {
			FileInputStream fis = new FileInputStream(new File(baseFile));
			byte[] ba = new byte[fis.available()];
			fis.read(ba);
			fis.close();
			baseMsg = new String(ba);
			setBaseMsg();
		} catch (Exception e) {
			baseMsg = null;
		}
	}
	*/
	/**
	 * legge il file xml con le specifiche di base
	 * del messaggio SOAP
	 * @param baseFile
	 */
	private void readBaseMsg(String baseFile) {
		try {
			InputStream is = this.getClass().getResourceAsStream(baseFile);
			byte[] ba = new byte[is.available()];
			is.read(ba);
			is.close();
			setBaseMsg(new String(ba));
		} catch (Exception e) {
			soapMessage = null;
		}
	}

	/**
	 * crea il SOAPMessage a partire dal
	 * contenuto del file xml letto
	 */
	private void setBaseMsg() {
		try {
	        MessageFactory mf = MessageFactory.newInstance();
	        //soapMessage = mf.createMessage(new MimeHeaders(), new ByteArrayInputStream(baseMsg.getBytes()));
	        soapMessage = mf.createMessage();
		} catch (SOAPException e) {
			e.printStackTrace();
		}/* catch (IOException e) {
			e.printStackTrace();
		}*/
	}

	/**
	 * crea il SOAPMessage a partire dal
	 * contenuto del file xml letto
	 */
	private void setBaseMsg(String base) {
		try {
	        MessageFactory mf = MessageFactory.newInstance();
	        soapMessage = mf.createMessage(new MimeHeaders(), new ByteArrayInputStream(base.getBytes()));
		} catch (SOAPException e) {
			e.printStackTrace();
			soapMessage = null;
		} catch (IOException ex) {
			ex.printStackTrace();
			soapMessage = null;
		}
	}

	/**
	 * imposta il body del messaggio SOAP con il contenuto passato
	 * (Document object)
	 * @param body
	 */
	public void setBodyMsg(Document body) {
		if (soapMessage != null) {
			try {
		        SOAPPart sp = soapMessage.getSOAPPart();
		        SOAPEnvelope se = (SOAPEnvelope)sp.getEnvelope();
		        SOAPBody sb = se.getBody();
		        sb.addDocument(body);
			} catch (SOAPException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * imposta l'Header con il contenuto desiderato
	 * (se necessario, es.: autenticazione)
	 * @param headerElement
	 */
	public void setHeader(SOAPElement headerElement) {
		if (soapMessage != null) {
			try {
			    SOAPPart sp = soapMessage.getSOAPPart();
			    SOAPEnvelope se = sp.getEnvelope();
			    SOAPHeader sh = se.getHeader();
			    sh.appendChild(headerElement);
			} catch (SOAPException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * esegue la call all'endPoint e ritorna il messaggio
	 * ricevuto.
	 * @return
	 */
	public SOAPMessage getResponse() {
		try {
	        SOAPConnection conn = SOAPConnectionFactory.newInstance().createConnection();
	        SOAPMessage response = conn.call(soapMessage, endPoint);
	        conn.close();
	        return response;
		} catch (SOAPException e) {
			e.printStackTrace();
			return null;
		}
	}
	

	/**
	 * esegue il display su std output
	 * dei nodi che compongono il body
	 */
	public void displayNodes() {
        try {
			SOAPBody body = soapMessage.getSOAPBody();
            Iterator iter1 = body.getChildElements();
            getContents(iter1, "");
		} catch (SOAPException e) {
			e.printStackTrace();
		}
	}

    /*
     * Retrieves the contents of the elements recursively and
     * displays them.
     *
     * @param iterator	Iterator returned by getChildElements
     * @param indent	indentation to nest element display
     */
    public void getContents(Iterator iterator, 
        String indent) {

        while (iterator.hasNext()) {
            Node node = (Node) iterator.next();
            SOAPElement element = null;
            Text text = null;
            if (node instanceof SOAPElement) {
                element = (SOAPElement)node;
                Name name = element.getElementName();
                System.out.println(indent + "Name is " + 
                    name.getQualifiedName());
                Iterator attrs = element.getAllAttributes();
                while (attrs.hasNext()){
                    Name attrName = (Name)attrs.next();
                    System.out.println(indent + 
                        " Attribute name is " + 
                        attrName.getQualifiedName());
                    System.out.println(indent + 
                        " Attribute value is " + 
                        element.getAttributeValue(attrName));
                }
                Iterator iter2 = element.getChildElements();
                getContents(iter2, indent + " ");
            } else {
                text = (Text) node;
                String content = text.getValue();
                System.out.println(indent + 
                    "Content is: " + content);
            }
        }
    }

	
    /**
     * imposta un dato nodo ad un dato valore nel body
     * @param name
     * @param value
     * @return
     */
	public boolean setNodeValue(String name, String value) {
        try {
			SOAPBody body = soapMessage.getSOAPBody();
            Iterator iter1 = body.getChildElements();
            return setContents(iter1, name, value);
		} catch (SOAPException e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
     * imposta un dato nodo ad un dato valore nel body
     * processo iterativo di ricerca del nodo e settaggio valore
	 * @param iterator
	 * @param name
	 * @param value
	 * @return
	 */
    private boolean setContents(Iterator iterator, String name, String value) {
        while (iterator.hasNext()) {
			Node node = (Node) iterator.next();
			SOAPElement element = null;
			if (node instanceof SOAPElement) {
				element = (SOAPElement) node;
				Name n = element.getElementName();
				if (n.getQualifiedName().equals(name)) {
					element.setValue(value);
					return true;
				} else {
					Iterator iter2 = element.getChildElements();
					setContents(iter2, name, value);
				}
			}
		}
		return false;
	}


    public String getBaseMsg() {
		return baseMsg;
	}

	public URL getEndPoint() {
		return endPoint;
	}

	public void setEndPoint(URL endPoint) {
		this.endPoint = endPoint;
	}

	public void setEndPoint(String endpoint) {
		try {
			this.endPoint = new URL(endpoint);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
	}

	public SOAPMessage getSoapMessage() {
		return soapMessage;
	}

	public void setSoapMessage(SOAPMessage soapMessage) {
		this.soapMessage = soapMessage;
	}

}
