package it.pf.utils;

import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import com.lowagie.text.*;
import com.lowagie.text.pdf.*;

import it.pf.utils.ScanDir;

/**
 * serve a condensare pi&ugrave; files pdf in
 * un unico file di output (.pdf) 
 */
public class CondensePdf {


	/**
	 * esecuzione: il nome del file di output viene 
	 * passato come parametro assieme al filtro da usare
	 * per selezionare solo i files desiderati dalla directory
	 * indicata (dirname).
	 */
	public void concat(String dirname, String outfile, FilenameFilter filter) {
		ScanDir rdi = new ScanDir(filter);
		rdi.setInspectSubdirs(true);
		rdi.setSortListOfFiles(true);
		try {
			Vector v = rdi.getFilesOfDir(dirname);
			Collections.sort((List)v);
			System.out.println("Nome file di output:"+outfile);
			System.out.println("-------------------------");
			this.concatFiles(v,outfile);
			System.out.println("-------------------------");
			System.out.println("FINE ELABORAZIONE:"+outfile);
		} catch (Exception e) {
			e.printStackTrace();
			System.out.println("L'argomento non e' una directory!");
			System.exit(9);
		}
	}

	/**
	 * legge vettore di file pdf e append su file di output 
	 */
	public void concatFiles(Vector files, String outFile) {
		try {
			int pageOffset = 0;
			ArrayList master = new ArrayList();
			int f = 0;
			//String outFile = args[args.length-1];
			Document document = null;
			PdfCopy  writer = null;
			for (f=0;f<files.size();f++) {
				// we create a reader for a certain document
				PdfReader reader = new PdfReader((String)files.elementAt(f));
				reader.consolidateNamedDestinations();
				// we retrieve the total number of pages
				int n = reader.getNumberOfPages();
				List bookmarks = SimpleBookmark.getBookmark(reader);
				if (bookmarks != null) {
					if (pageOffset != 0)
						SimpleBookmark.shiftPageNumbers(bookmarks, pageOffset, null);
					master.addAll(bookmarks);
				}
				pageOffset += n;
				//System.out.println("Il file " + (String)files.elementAt(f) + " ha " + n + " pagine");
				//System.out.println("There are " + n + " pages in " + (String)files.elementAt(f));
                    
				if (f == 0) {
					// step 1: creation of a document-object
					document = new Document(reader.getPageSizeWithRotation(1));
					// step 2: we create a writer that listens to the document
					writer = new PdfCopy(document, new FileOutputStream(outFile,true));
					// step 3: we open the document
					document.open();
				}
				// step 4: we add content
				PdfImportedPage page;
				for (int i = 0; i < n; ) {
					++i;
					page = writer.getImportedPage(reader, i);
					writer.addPage(page);
					//System.out.println("Processed page " + i);
				}
				PRAcroForm form = reader.getAcroForm();
				if (form != null)
					writer.copyAcroForm(reader);
			}
			if (master.size() > 0)
				writer.setOutlines(master);
			// step 5: we close the document
			if (document != null)
				document.close();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}

}
