package it.pf.utils;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Stack;

public class DirUtil {

	private ArrayList files;
	private ArrayList dirs;

	public DirUtil(String dir) {
		files = new ArrayList();
		dirs = new ArrayList();
		this.populate(dir);
	}

	private void populate(String dir_to_list) {
		if (dir_to_list != null) {
			// System.out.println("root_dir="+dir_to_list);
			File dir = new File(dir_to_list);
			if (dir.isDirectory()) {
				String abspath = dir.getAbsolutePath();
				String[] fns = sortedFileList(dir);
				abspath = abspath.replace('\\', '/');
				for (int i = 0; i < fns.length; i++) {
					String fpath = abspath + "/" + fns[i];
					File f = new File(fpath);
					if (f.isDirectory())
						dirs.add(f.getAbsolutePath());
					else
						files.add(f.getAbsolutePath());
				}
			}
		}
	}

	private String[] sortedFileList(File dir) {
		String[] f = dir.list();
		Arrays.sort(f);
		return f;
	}

	public ArrayList getFiles() {
		return files;
	}

	public ArrayList getDirs() {
		return dirs;
	}

	/**
	 * remove a directory and all its contents
	 */
	public static boolean rmDir(File dir) {
		if (!dir.exists())
			return true;
		Stack dirStack = new Stack();
		dirStack.push(dir);
		boolean deleted;

		boolean containsSubFolder;
		while (!dirStack.isEmpty()) {
			File currDir = (File) dirStack.peek();
			containsSubFolder = false;

			String[] fileArray = currDir.list();
			for (int i = 0; i < fileArray.length; i++) {
				String fileName = currDir.getAbsolutePath() + File.separator
						+ fileArray[i];
				File file = new File(fileName);
				if (file.isDirectory()) {
					dirStack.push(file);
					containsSubFolder = true;
				} else {
					deleted = file.delete(); // delete file
					if (!deleted)
						return false;
				}
			}

			if (!containsSubFolder) {
				dirStack.pop(); // remove curr dir from stack
				deleted = currDir.delete(); // delete curr dir
				if (!deleted)
					return false;
			}
		}
		return true;
	}

}
