package it.pf.utils;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

/**
 * legge il contenuto di una directory eventualmente applicando 
 * un filtro e ritorna i files della directory in un vettore
 * contenente stringhe che rappresentano il path assoluto dei
 * files.
 * 
 */
public class ScanDir {

	private boolean inspectSubdirs;
	private boolean sortListOfFiles;
	private boolean filter;
	private FilenameFilter fileFilter;
	
	public ScanDir() {
		inspectSubdirs = false;
		sortListOfFiles = false;
		fileFilter = null;
		filter = false;
	}
	
	public ScanDir(FilenameFilter accept) {
		inspectSubdirs = false;
		sortListOfFiles = false;
		fileFilter = accept;
		filter = true;		
	}

	
	/**
	 * ispeziona una directory e ritorna un vettore con 
	 * il pathname assoluto dei files contenuti
	 * l'analisi prosegue anche per le sottodirectories
	 * ricorsivamente
	 * @param dirname
	 * @return
	 * @throws Exception
	 */
	public Vector getFilesOfDir(String dirname) throws Exception {
		Vector dirlist = new Vector();
		File hd = new File(dirname);
		if (hd.isDirectory()) {
			String[] fns = null;
			if (filter)
				fns = hd.list(fileFilter);
			else
				fns = hd.list();
			for (int i=0; i<fns.length; i++) {
				File hdt = new File(dirname+"\\"+fns[i]);
				if (inspectSubdirs) {
					if (hdt.isDirectory()) {
						Vector vf = getFilesOfDir(dirname+"\\"+fns[i]);
						if (vf != null) {
							for (int j=0; j<vf.size(); j++)
								dirlist.add((String)vf.elementAt(j));
						}
					}
				}
				if (hdt.isFile())				
					dirlist.add(dirname+"\\"+fns[i]);
			}
			if (sortListOfFiles) {
				Collections.sort((List)dirlist);
			}
			return dirlist;
		}
		return null;
	}

	/**
	 * @return
	 */
	public boolean isInspectSubdirs() {
		return inspectSubdirs;
	}

	/**
	 * @return
	 */
	public boolean isSortListOfFiles() {
		return sortListOfFiles;
	}

	/**
	 * @param b
	 */
	public void setInspectSubdirs(boolean b) {
		inspectSubdirs = b;
	}

	/**
	 * @param b
	 */
	public void setSortListOfFiles(boolean b) {
		sortListOfFiles = b;
	}

}
