package it.pf.utils;

import it.pf.base.ConfigSingleton;
import it.pf.base.Constants;
import it.pf.base.ObjectBean;
import it.pf.base.ObjectBeanAttribute;
import it.pf.dbaccess.DataField;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.SQLBase;
import it.pf.dbaccess.SetupDataSource;
import it.pf.dbaccess.SingleStatement;

import java.util.*;
import java.sql.*;
import java.sql.Date;
import java.io.*;
import java.lang.reflect.*;
import java.math.BigDecimal;
import java.math.BigInteger;

import javax.sql.DataSource;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

//import COM.ibm.db2.jdbc.app.DB2ParameterMetaData;
//import COM.ibm.db2.jdbc.app.DB2PreparedStatement;



/**
 * SQL executor building java beans for results
 */
public class SqlBeanBuilder extends SQLBase  {

    private String queryFile;
    private String queryName;
    private String beanDir;
    private String SQLquery;

    private String pkg = null;

	
	
    /**
     * Sqlx constructor: receives all from Properties
     *                   used to build ResultBean classes
     */
    public SqlBeanBuilder() {

        //queryFile = (String)p.get("QUERYFILENAME");
        //queryName = extractNameFromPath(queryFile);
        //pathName = extractPathFromFile(queryFile);
        //SQLquery = readQueryFileISO(queryFile);
        //queryType = getQueryType(SQLquery);
        //pkg = (String) p.get("BEANSPACKAGE");
        //beanDir = (String) p.get("BEANSDIR");

        ObjectBean bq = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:PARAMETERS:QUERY");
        SQLquery = bq.get_content();
        queryName = (String)bq.getAttribute("name").get_value();
        bq = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:PARAMETERS:BEANSPACKAGE");
        pkg = bq.get_content();
        bq = ConfigSingleton.getResources().getFilteredAttribute("CONFIG:PARAMETERS:BEANSDIR");
        beanDir = bq.get_content();
        if (!beanDir.endsWith("/"))
        	beanDir += "/";
        
		_name = "BEANBUILDER";
		_query = SQLquery.trim();
		_valid = true;
		String[] words = _query.split(" ");
		_queryType = words[0].trim();

    }

	/**
	 * esegue lo statement acquisendo la connessione dal DataSource
	 * ricevuto nel parametro
	 * @param ds
	 * @throws Exception
	 */
	public void execute(DataSource ds) throws Exception {

		if (!is_valid())
			throw new Exception("SQLCommand not valid: not correctly initialized!");

		_result = new ObjectBean("ROWS");

		Connection con = ds.getConnection();
		if (con == null)
			throw new Exception("Connection null: DataSource not initialized?");

		try {
			executeInternal(con);
			con.close();
		} catch (Exception sqle) {
			sqle.printStackTrace();
			try {con.close();} catch (Exception e) {}
			System.out.println("ERROR SQL on query:" + _query);
			throw new Exception(sqle.getMessage());
		} finally {
			if (con != null) {
				try {con.close();} catch (Exception e) {}
			}
		}
		return;
	}

	/**
	 * metodo interno richiamato dai metodi execute:
	 * analizza lo statement e se si tratta di SELECT
	 * genera il risultato sotto forma di OB,
	 * altrimenti imposta il numero di righe influenzate
	 * dallo statement eseguito
	 * @param con
	 * @throws Exception
	 */
	private void executeInternal(Connection con) throws Exception {
		PreparedStatement statement = null;
		ResultSet rs = null;
		ResultSetMetaData rsmd = null;

		statement = con.prepareStatement(_query);
 
		String ColNames[] = new String[1000];
        String ColTypes[] = new String[1000];
        int ColSizes[] = new int[1000];
        int Nullable[] = new int[1000];
        int Scale[] = new int[1000];
 
        
        int pCount = howManyParameters(_query);
		if (pCount > 0) {
			System.out.println(_query);
			List inputParms = getInputParms(pCount);
            DataField inputParameterDataField = null;
            for (int i = 0; i < pCount; i++) {
                inputParameterDataField = (DataField) inputParms.get(i);
                statement.setObject(
	                    i + 1,
	                    inputParameterDataField.getObjectValue(),
	                    inputParameterDataField.getSqlType());
            } 
		}
		
		if (_queryType.equalsIgnoreCase("SELECT")) {
			rs = statement.executeQuery();
			rsmd = rs.getMetaData();

			int colCount = rsmd.getColumnCount();

			_rowsFetched = rs.getFetchSize();
			
            System.out.println("Attributes number:" + colCount);

            for (int i = 0; i < colCount; i++) {
                ColNames[i] = rsmd.getColumnName(i + 1);
                ColTypes[i] = rsmd.getColumnTypeName(i + 1);
                ColSizes[i] = rsmd.getColumnDisplaySize(i + 1);
                Nullable[i] = rsmd.isNullable(i + 1);
                Scale[i] = rsmd.getScale(i + 1);
            }
            try {
				writeResultBean(ColNames,ColTypes,ColSizes,Nullable,Scale,colCount);
            } catch (Exception e) {
	            con.close();
    	        return;
            }
			rs.close();	
		}
		
		return;
	}
	

	private int howManyParameters(String q) {
		int count = 0;
		int p = q.indexOf("?");
		if (p < 0)
			return 0;
		else {
			count++;
			while (p > 0) {
				p = q.indexOf("?",p+1);
				if (p > 0)
					count++;
			}
			return count;
		}
	}



    /**
     * read queryFile and store contents in string SQLquery
     */
    private String readQueryFileISO(String queryFile) {
		
		File f = null;
        FileInputStream in;
        String query = "";
        try {
        	f = new File(queryFile);
            in = new FileInputStream(f);
			//allocate a byte array to store content data
			Long l = new Long(f.length());
			int size = l.intValue();
			byte dataBytes[] = new byte[size];
			in.read(dataBytes);
			for (int i = 0; i < dataBytes.length; i++) {
				Byte b = new Byte(dataBytes[i]);
				if ( (b.intValue() == 13) | (b.intValue() == 10) )
					dataBytes[i] = 0x20;
			}
			query = new String(dataBytes,"ISO-8859-1");
            in.close();
            return query;
        } catch (Exception e) {
            System.out.println(
                "error reading queryFile:" + queryFile + "\n" + e.getMessage());
            return null;
        }
    }

    /**
     * extract name from full path
     */
    private String extractNameFromPath(String queryFile) {
        StringTokenizer st = new StringTokenizer(queryFile, "\\/");
        String last = null;
        while (st.hasMoreElements()) {
            last = (String) st.nextElement();
        }

        if (last.length() > 0) {
            st = new StringTokenizer(last, ".");
            return (String) st.nextElement();
        } else
            return null;

    }


    /**
     * extract path from full path file name
     */
    private String extractPathFromFile(String queryFile) {
    	String path = "";
    	String fragment = "";
        StringTokenizer st = new StringTokenizer(queryFile, "\\/",true);
        while (st.hasMoreElements()) {
            fragment = (String) st.nextElement();
            if (st.hasMoreElements()) {
            	path += fragment;
            }
        }

		return path;

    }

    /**
     * estabilish query type from first word of SQLquery
     * return 0 if type select
     * return 1 otherwise
     */
    private int getQueryType(String query) {
        StringTokenizer st = new StringTokenizer(query);
        String firstword = null;
        if (query.length() > 0) {
            firstword = ((String) st.nextElement()).trim();
            if (firstword.equalsIgnoreCase("SELECT"))
                return 0;
            else
                return 1;
        } else
            return -1;

    }

	/**
	 * write result bean
	 */
	private void writeResultBean(String[] colNames,
						    String[] colTypes,
						    int[] colSizes,
						    int[] Nullable,
						    int[] Scale,
						    int Cols) throws Exception {
		
		try {
			String javaBean = queryName+"Bean.java";
			String columnsfile = beanDir+javaBean;

			FileWriter cfile = new FileWriter(columnsfile);
			writeHeader(cfile,javaBean);
			for (int i = 0; i < Cols; i++) {
				String tipo = "";
				if (colTypes[i].equalsIgnoreCase("CHAR"))
					tipo = "String";
				if (colTypes[i].equalsIgnoreCase("CHARacter"))
					tipo = "String";
				if ( (colTypes[i].equalsIgnoreCase("VARCHAR")) ||
						(colTypes[i].equalsIgnoreCase("VARCHAR2")) )
					tipo = "String";
				if (colTypes[i].equalsIgnoreCase("LONG VARCHAR"))
					tipo = "String";
				if (colTypes[i].equalsIgnoreCase("INTeger"))
					tipo = "Integer";
				if (colTypes[i].equalsIgnoreCase("SMALLINT"))
					tipo = "Integer";
				if (colTypes[i].equalsIgnoreCase("DATE"))
					tipo = "java.sql.Date";
				if (colTypes[i].equalsIgnoreCase("TIME"))
					tipo = "java.sql.Time";
				if (colTypes[i].equalsIgnoreCase("TIMESTAMP"))
					tipo = "java.sql.Timestamp";
				if (colTypes[i].equalsIgnoreCase("DECIMAL"))
					tipo = "BigDecimal";
				if ( (colTypes[i].equalsIgnoreCase("NUMBER")) ||
					(colTypes[i].equalsIgnoreCase("NUMERIC")) ) {
					if (Scale[i] == 0) {
						tipo = "BigInteger";
					} else {
						tipo = "BigDecimal";
					}
				}
				String nomeparametro = colNames[i];
				writeDetail(cfile,nomeparametro,tipo,Nullable[i]);
			}
			cfile.write("\n}\n");
			cfile.close();

			javaBean = queryName+"ResultBeanLen.java";
			String lengthfile = beanDir+javaBean;

			FileWriter lfile = new FileWriter(lengthfile);
			writeHeader(lfile,javaBean);
			for (int i = 0; i < Cols; i++) {
				int displaysize = 0;
				if (colTypes[i].equalsIgnoreCase("CHAR"))
					displaysize = colSizes[i];
				if (colTypes[i].equalsIgnoreCase("CHARacter"))
					displaysize = colSizes[i];
				if ( (colTypes[i].equalsIgnoreCase("VARCHAR")) ||
						(colTypes[i].equalsIgnoreCase("VARCHAR2")) )
					displaysize = colSizes[i];
				if (colTypes[i].equalsIgnoreCase("LONG VARCHAR"))
					displaysize = colSizes[i];
				if (colTypes[i].equalsIgnoreCase("INTeger"))
					displaysize = 14;
				if (colTypes[i].equalsIgnoreCase("SMALLINT"))
					displaysize = 14;
				if (colTypes[i].equalsIgnoreCase("DATE"))
					displaysize = 10;
				if (colTypes[i].equalsIgnoreCase("TIME"))
					displaysize = 8;
				if (colTypes[i].equalsIgnoreCase("TIMESTAMP"))
					displaysize = 26;
				if (colTypes[i].equalsIgnoreCase("DECIMAL"))
					displaysize = 18;
				if ( (colTypes[i].equalsIgnoreCase("NUMBER")) ||
					(colTypes[i].equalsIgnoreCase("NUMERIC")) )
						displaysize = 18;				
				String nomeparametro = colNames[i];
				writeDetailLen(lfile,nomeparametro,displaysize);
			}
			lfile.write("\n}\n");
			lfile.close();
		} catch (Exception e) {
            System.out.println("ERROR building BEAN ");
            System.out.println(e.getMessage());
            return;
		}
		
		return;
	}

    
	/**
	 * writeHeader
	 */
	private void writeHeader(FileWriter x, String javaname) {
		try {
			x.write("package "+pkg+";\n");
			x.write("import java.math.*;\n");
			x.write("/**\n");
			x.write(" * SQL bean per "+javaname+"\n");
			x.write(" */\n");
            StringTokenizer st = new StringTokenizer(javaname, ".");
            String name = (String) st.nextElement();
			x.write("public class "+name+" {\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

    
	/**
	 * writeHeaderLen
	 */
	private void writeHeaderLen(FileWriter x, String javaname) {
		try {
			x.write("package "+pkg+";\n");
			x.write("/**\n");
			x.write(" * Costanti di lunghezza per "+javaname+"\n");
			x.write(" */\n");
            StringTokenizer st = new StringTokenizer(javaname, ".");
            String name = (String) st.nextElement();
			x.write("public final class "+name+" {\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * writeDetail
	 */
	private void writeDetail(FileWriter x, String nomeparametro, String tipoparametro, int nullable) {
		try {
			x.write("   /**\n");
			x.write("    * get and set for "+nomeparametro+"\n");
			x.write("    */\n");
			x.write("   private "+tipoparametro+" "+nomeparametro+";\n");
			if (nullable == 1)
				x.write("   private int "+nomeparametro+"null;\n");
			x.write("   public "+tipoparametro+" get"+nomeparametro+"() {\n");
			if (nullable == 1) {
				x.write("       if ("+nomeparametro+"null == -1)\n");
				x.write("       	return null;\n");
				x.write("       else\n");
				x.write("       return "+nomeparametro+";\n");
			} else {
				x.write("       return "+nomeparametro+";\n");
			}
			x.write("   }\n");
			x.write("   public void set"+nomeparametro+"("+tipoparametro+" "+nomeparametro+") {\n");
			x.write("       this."+nomeparametro+" = "+nomeparametro+";\n");
			if (nullable == 1) {
				x.write("       if ("+nomeparametro+" == null)\n");
				x.write("       	"+nomeparametro+"null = -1;\n");
			}
			x.write("   }\n\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
    
	/**
	 * writeDetailLen
	 */
	private void writeDetailLen(FileWriter x, String nomeparametro,int value) {
		try {
			x.write("   /**\n");
			x.write("    * len of "+nomeparametro+"\n");
			x.write("    */\n");
			x.write("   public static int "+nomeparametro+" = "+new Integer(value).toString()+";\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

    
	/**
	 * writeParmHeader
	 */
	private void writeParmHeader(FileWriter x, String javaname) {
		try {
			x.write("package "+pkg+";\n");
			x.write("/**\n");
			x.write(" * SQL bean per "+javaname+"\n");
			x.write(" */\n");
			StringTokenizer st = new StringTokenizer(javaname, ".");
			String name = (String) st.nextElement();
			x.write("public class "+name+" {\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * write parm bean
	 */
	private void writeParmBean(String[] colNames,
							int Cols) throws Exception {
		
		try {
			String javaBean = queryName+"ParmBean.java";
			String columnsfile = beanDir+javaBean;

			FileWriter cfile = new FileWriter(columnsfile);
			writeParmHeader(cfile,javaBean);
			for (int i = 0; i < Cols; i++) {
				String tipo = "String";
				String nomeparametro = colNames[i];
				writeParmDetail(cfile,nomeparametro,tipo);
			}
			cfile.write("\n}\n");
			cfile.close();
		} catch (Exception e) {
			System.out.println("ERROR building BEAN ");
			System.out.println(e.getMessage());
			return;
		}
		
		return;
	}

	/**
	 * writeDetail
	 */
	private void writeParmDetail(FileWriter x, String nomeparametro, String tipoparametro) {
		try {
			x.write("   /**\n");
			x.write("    * get and set for "+nomeparametro+"\n");
			x.write("    */\n");
			x.write("   private "+tipoparametro+" "+nomeparametro+";\n");
			x.write("   public "+tipoparametro+" get"+nomeparametro+"() {\n");
			x.write("       return "+nomeparametro+";\n");
			x.write("   }\n");
			x.write("   public void set"+nomeparametro+"("+tipoparametro+" "+nomeparametro+") {\n");
			x.write("       this."+nomeparametro+" = "+nomeparametro+";\n");
			x.write("   }\n\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	/**
	 * main: args[0] must be full path file name of properties
	 */
    public static void main(String[] args) {
    	String configfile = null;
    	if (args.length == 0) {
    		configfile = "config.xml";
    	} else {
    		configfile = args[0];
    	}
		ConfigSingleton.setConfigFileName(configfile);
		ObjectBean config = null;
		try {
			config = new ObjectBean(ObjectBean.fromXMLFile(configfile));
			ConfigSingleton.setResources(config);
			ConfigSingleton.getInstance();
			ObjectBeanAttribute ds_parms = config.getAttribute("DATA_SOURCE");
			SetupDataSource dsinst = new SetupDataSource((ObjectBean)ds_parms.get_value());
			DataSource ds = dsinst.getDataSource();
			ConfigSingleton.setDataSource(ds);
		} catch (Exception e) {
			System.out.println("config file problems.");
			e.printStackTrace();
			System.exit(9);
		}		
		try {
			//FileInputStream tmp = new FileInputStream("sqlparms.properties");
			//Properties pp = new Properties();
			//pp.load(tmp);
	    	SqlBeanBuilder runner = new SqlBeanBuilder();
			try {
				runner.execute(ConfigSingleton.getDataSource());
				System.out.println("Sqlbuild end run!");
			} catch (IOException e) {
				e.printStackTrace();
			}
    	} catch (Exception e) {
    		e.printStackTrace();
    	}
		System.exit(0);
    }

	private List getInputParms(int n) {
		ArrayList inputParms = new ArrayList(n);
		for (int i=0; i<n; i++) {
			Object[] sqltypes = initTypes();
			String s = (String)JOptionPane.showInputDialog(new JFrame(),"Dato "+(i+1)+": Tipo","SQL type",
                            JOptionPane.PLAIN_MESSAGE,
                            null,
                            sqltypes,
                            "VARCHAR");
			//System.out.println(s);
			String v = (String)JOptionPane.showInputDialog(new JFrame(),"Dato "+(i+1)+": Valore","Value",
                    JOptionPane.PLAIN_MESSAGE);
			//System.out.println(v);
			inputParms.add(new DataField("P"+(i+1), getSQLType(s), getSQLValue(s, v)));
		}
		return inputParms;
	}
	
	private int getSQLType(String s) {
		if (s.equals("ARRAY"))
			return Types.ARRAY;
		else if (s.equals("BIGINT"))
			return Types.BIGINT;
		else if (s.equals("BINARY"))
			return Types.BINARY;
		else if (s.equals("BIT"))
			return Types.BIT;
		else if (s.equals("BLOB"))
			return Types.BLOB;
		else if (s.equals("BOOLEAN"))
			return Types.BOOLEAN;
		else if (s.equals("CHAR"))
			return Types.CHAR;
		else if (s.equals("CLOB"))
			return Types.CLOB;
		else if (s.equals("DATE"))
			return Types.DATE;
		else if (s.equals("DECIMAL"))
			return Types.DECIMAL;
		else if (s.equals("DOUBLE"))
			return Types.DOUBLE;
		else if (s.equals("FLOAT"))
			return Types.FLOAT;
		else if (s.equals("INTEGER"))
			return Types.INTEGER;
		else if (s.equals("LONGVARBINARY"))
			return Types.LONGVARBINARY;
		else if (s.equals("LONGVARCHAR"))
			return Types.LONGVARCHAR;
		else if (s.equals("NUMERIC"))
			return Types.NUMERIC;
		else if (s.equals("REAL"))
			return Types.REAL;
		else if (s.equals("SMALLINT"))
			return Types.SMALLINT;
		else if (s.equals("TIME"))
			return Types.TIME;
		else if (s.equals("TIMESTAMP"))
			return Types.TIMESTAMP;
		else if (s.equals("VARBINARY"))
			return Types.VARBINARY;
		else if (s.equals("VARCHAR"))
			return Types.VARCHAR;
		else
			return Types.VARCHAR;
	}

	private Object getSQLValue(String s, String v) {
		if (s.equals("ARRAY"))
			return v.getBytes();
		else if (s.equals("BIGINT")) {
			BigInteger bi = new BigInteger(v);
			return bi;
		} else if (s.equals("BINARY"))
			return v.getBytes();
		else if (s.equals("BIT"))
			return v.getBytes();
		else if (s.equals("BLOB"))
			return null;
		else if (s.equals("BOOLEAN"))
			if (v.equals("true"))
				return new Boolean(true);
			else
				return new Boolean(false);
		else if (s.equals("CHAR"))
			return v;
		else if (s.equals("CLOB"))
			return v.getBytes();
		else if (s.equals("DATE")) {
			DataUtil du = new DataUtil(v);
			Date d = du.getSQLDate();
			return d;
		} else if (s.equals("DECIMAL"))
			return new BigDecimal(v);
		else if (s.equals("DOUBLE"))
			return new Double(v);
		else if (s.equals("FLOAT"))
			return new Float(v);
		else if (s.equals("INTEGER"))
			return new Integer(v);
		else if (s.equals("LONGVARBINARY"))
			return v.getBytes();
		else if (s.equals("LONGVARCHAR"))
			return v.getBytes();
		else if (s.equals("NUMERIC"))
			return new BigDecimal(v);
		else if (s.equals("REAL"))
			return new Float(v);
		else if (s.equals("SMALLINT"))
			return new Integer(v);
		else if (s.equals("TIME"))
			return new Time(1,1,1);
		else if (s.equals("TIMESTAMP"))
			return new Timestamp((long)11111);
		else if (s.equals("VARBINARY"))
			return v.getBytes();
		else if (s.equals("VARCHAR"))
			return v;
		else
			return v;
	}
	
	private Object[] initTypes() {
		Object[] t = new Object[22];
		t[0] = "ARRAY";
		t[1] = "BIGINT";
		t[2] = "BINARY";
		t[3] = "BIT";
		t[4] = "BLOB";
		t[5] = "BOOLEAN";
		t[6] = "CHAR";
		t[7] = "CLOB";
		t[8] = "DATE";
		t[9] = "DECIMAL";
		t[10] = "DOUBLE";
		t[11] = "FLOAT";
		t[12] = "INTEGER";
		t[13] = "LONGVARBINARY";
		t[14] = "LONGVARCHAR";
		t[15] = "NUMERIC";
		t[16] = "REAL";
		t[17] = "SMALLINT";
		t[18] = "TIME";
		t[19] = "TIMESTAMP";
		t[20] = "VARBINARY";
		t[21] = "VARCHAR";
		return t;
	}


}
