package it.pf.utils;

import java.util.regex.Pattern;

/**
 * Classe di utilit per la verifica della correttezza formale di un dato in forma di stringa
 * utilizza alcuni patterns predefiniti e richiamabili per indice o per nome
 * i nomi sono quelli dell'array di stringhe patternmap.
 * @author fabbri_p
 *
 */
public class StringUtil {

	/**
	 *  array di stringhe da usare a coppie nome-pattern
	 */
	public static String[] patternmap = {
			"letterenumeri","[a-zA-Z_0-9]+",			//letterenumeri
			"solonumeri","[0-9]+",				//solonumeri
			"sololettere","[a-zA-Z]+",				//sololettere
			"solospazi","\\p{Space}+",				//solospazi
			"almenounospeciale",".*\\W+.*",					//almenounospeciale
			"almenounapunteggiatura",".*\\p{Punct}+.*",			//almenounapunteggiatura
			"almenounospazio",".*\\p{Space}+.*",			//almenounospazio
			"tuttiascii","\\p{ASCII}+",			//allascii
			"tuttemaiuscole","\\p{Upper}+",			//allupper
			"tutteminuscole","\\p{Lower}+",			//alllower
			"alfanumerici","\\p{Alnum}+",			//alfanumerici
			"numeroa2decimali","\\d+[.,]\\d\\d$",		//numero con 2 decimali
			"numeroa3decimali","\\d+[.,]\\d\\d\\d$",		//numero con 2 decimali
			"numeroa4decimali","\\d+[.,]\\d\\d\\d\\d$",		//numero con 2 decimali
			"numeroa5decimali","\\d+[.,]\\d\\d\\d\\d\\d$"		//numero con 2 decimali
	};
	
	private String dato;
	
	private String matchName;
	
	/**
	 * @return Returns the matchName.
	 */
	public String getMatchName() {
		return matchName;
	}

	/**
	 * @param matchName The matchName to set.
	 */
	public void setMatchName(String matchName) {
		this.matchName = matchName;
	}

	/**
	 * costruttore di default
	 */
	public StringUtil() {
		this.setMatchName(null);
	}
	
	/**
	 * costruttore che imposta la stringa da confrontare
	 */
	public StringUtil(String d) {
		this.setMatchName(null);
		this.setDato(d);
	}

	/**
	 * esegue il match con il pattern individuato per indice
	 * all'interno di patternmap
	 * @param i
	 * @return
	 */
	public boolean match(int i) {
		if (dato != null) {
			if (i >= (patternmap.length/2))
				return false;
			int mapi = (i*2)+1;
			boolean m = Pattern.matches(patternmap[mapi], dato); 
			return m;
		} else 
			return false;
	}

	
	/**
	 * esegue il match con il pattern individuato per nome
	 * all'interno di patternmap
	 * @param i
	 * @return
	 */
	public boolean match(String nomematch) {
		if (dato != null) {
			for (int i=0; i<(patternmap.length/2); i++) {
				if (patternmap[(i*2)].equalsIgnoreCase(nomematch)) {
					this.setMatchName(nomematch);
					int mapi = (i*2)+1;
					boolean m = Pattern.matches(patternmap[mapi], dato); 
					return m;
				} 
			} 
			return false;
		} else 
			return false;
	}

	/**
	 * @return Returns the dato.
	 */
	public String getDato() {
		return dato;
	}

	/**
	 * @param dato The dato to set.
	 */
	public void setDato(String dato) {
		this.dato = dato;
	}

	/**
	 * esegue il display del contenuto di patternmap
	 */
	public void showAvailablePatterns() {
		for (int i=0; i<(patternmap.length/2); i++) {
			System.out.println("N.ro "+i+" - "+patternmap[(i*2)]+" = "+patternmap[(i*2)+1]);
		}
	}
}
