package it.pf.utils;

import it.pf.base.ConfigSingleton;
import it.pf.base.ObjectBean;
import it.pf.dbaccess.DataField;
import it.pf.dbaccess.DataRow;
import it.pf.dbaccess.DateFormatter;

import java.io.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.BitSet;
import java.util.Collections;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.util.URIUtil;

/**
 * metodi statici di utilita'
 */
public final class Tools {

	protected static byte[] CRLF = { 0x0D, 0x0A };

	/**
	 * esegue una copia del contenuto di un bean sull'istanza di un'altro bean
	 * utilizzando i metodi get per leggere da from e set per scrivere su to
	 * se il metodo set... non esiste su to ignora e prosegue
	 * @param from
	 * @param to
	 * @throws InvocationTargetException 
	 * @throws IllegalAccessException 
	 * @throws IllegalArgumentException 
	 * @throws java.lang.Exception
	 */
	public static void copyBean(Object from, Object to) 
		throws IllegalArgumentException, IllegalAccessException, InvocationTargetException  {
    	Method[] methods = (from.getClass()).getMethods();
    	for (int j=0; j<methods.length; j++){
			Method methodget = ((Method)methods[j]);
			String nome = methodget.getName();
			if (nome!=null && nome.startsWith("get")){
				if (!nome.equals("getSerializer") &&
						!nome.equals("getDeserializer") &&
						!nome.equals("getOMElement") &&
						!nome.equals("getPullParser") &&
						!nome.equals("getTypeDesc") &&
						!nome.equals("getSerializer") &&
						!nome.equals("getClass") &&
						!nome.equals("getClasse")
				) {
					Object value = methodget.invoke(from, null);
					if (value != null) {
						try {
							Method methodset = (to.getClass()).getMethod("set"+methodget.getName().substring(3), value.getClass());
							if (methodset != null) {
								methodset.invoke(to,value);
							}
						} catch (NoSuchMethodException nm) {
							//donothing
						}
					}
				} 
			}
		}
	}

	
	/*
	 * esegue un ping
	 * 
	 */
	public static String ping(String host) {
		try {
			Process ping = Runtime.getRuntime().exec("ping "+host);
			BufferedReader br = new BufferedReader(new InputStreamReader(ping.getInputStream()));
			StringBuilder sb = new StringBuilder();
			String line;
			int count = 0;
			while (((line = br.readLine()) != null) &&
					(count < 10) ) {
				sb.append(line);
				sb.append("\n");
				count++;
			}
			//System.out.print(sb.toString());
			return sb.toString();
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		
	}
	
	/**
	 * ritorna un oggetto java.sql.Date a partire da una stringa e dal formato
	 * 
	 * @param stringDateValue
	 * @param _dateFormat
	 * @return
	 */
	public static Object getObjectValueForDate(String stringDateValue,
			String _dateFormat) {
		if (stringDateValue == null) {
			return null;
		}
		Date result = null;
		try {
			DateFormatter pfDateFormatter = new DateFormatter(_dateFormat);
			String dateToFormat = null;
			if (stringDateValue.length() > _dateFormat.length())
				dateToFormat = stringDateValue.substring(0, _dateFormat
						.length());
			else
				dateToFormat = stringDateValue;
			pfDateFormatter.format(dateToFormat);
			int day = (pfDateFormatter.getDay() != -1 ? pfDateFormatter
					.getDay() : 0);
			int month = (pfDateFormatter.getMonth() != -1 ? pfDateFormatter
					.getMonth() : 0);
			int year = (pfDateFormatter.getYear() != -1 ? pfDateFormatter
					.getYear() : 0);
			result = new Date(year - 1900, month - 1, day);
		} catch (ParseException pe) {
		}
		return result;
	}

	/**
	 * ritorna un oggetto java.sql.Timestamp a partire da una stringa e dal
	 * formato
	 * 
	 * @param stringTimeStampValue
	 * @param _timestampFormat
	 * @return
	 */
	public static Object getObjectValueForTimeStamp(
			String stringTimeStampValue, String _timestampFormat) {
		ObjectBean t = ConfigSingleton.getResources().getFilteredAttribute(
				"CONFIG:DATA_BASE:PARAMETER", "NAME", "dbname");
		if (t == null)
			return getObjectValueForTimeStamp4DB2(stringTimeStampValue,
					_timestampFormat);
		else {
			String dbname = (String) t.getAttributeValue("value");
			if (dbname.startsWith("DB2"))
				return getObjectValueForTimeStamp4DB2(stringTimeStampValue,
						_timestampFormat);
			else
				return getObjectValueForTimeStamp4ORA(stringTimeStampValue,
						_timestampFormat);
		}
	}

	/**
	 * ritorna un oggetto java.sql.Timestamp a partire da una stringa e dal
	 * formato
	 * 
	 * @param stringTimeStampValue
	 * @param _timestampFormat
	 * @return
	 */
	public static Object getObjectValueForTimeStamp4DB2(
			String stringTimeStampValue, String _timestampFormat) {
		Timestamp result = null;
		try {
			DateFormatter pfDateFormatter = new DateFormatter(_timestampFormat);
			pfDateFormatter.format(stringTimeStampValue);
			int day = (pfDateFormatter.getDay() != -1 ? pfDateFormatter
					.getDay() : 0);
			int month = (pfDateFormatter.getMonth() != -1 ? pfDateFormatter
					.getMonth() : 0);
			int year = (pfDateFormatter.getYear() != -1 ? pfDateFormatter
					.getYear() : 0);
			int hour = (pfDateFormatter.getHours() != -1 ? pfDateFormatter
					.getHours() : 0);
			int minute = (pfDateFormatter.getMinutes() != -1 ? pfDateFormatter
					.getMinutes() : 0);
			int second = (pfDateFormatter.getSeconds() != -1 ? pfDateFormatter
					.getSeconds() : 0);
			int nano = (pfDateFormatter.getNanos() != 0 ? pfDateFormatter
					.getNanos() : 0);
			result = new Timestamp(year - 1900, month - 1, day, hour, minute,
					second, nano);
			if (nano != 0) {
				result.setNanos(nano);
			}
		} catch (ParseException pe) {
		}
		return result;
	}

	/**
	 * ritorna un oggetto java.sql.Timestamp a partire da una stringa e dal
	 * formato
	 * 
	 * @param stringTimeStampValue
	 * @param _timestampFormat
	 * @return
	 */
	public static Object getObjectValueForTimeStamp4ORA(
			String stringTimeStampValue, String _timestampFormat) {
		Timestamp result = null;
		try {
			DateFormatter pfDateFormatter = new DateFormatter(_timestampFormat);
			pfDateFormatter.format(stringTimeStampValue);
			int day = (pfDateFormatter.getDay() != -1 ? pfDateFormatter
					.getDay() : 0);
			int month = (pfDateFormatter.getMonth() != -1 ? pfDateFormatter
					.getMonth() : 0);
			int year = (pfDateFormatter.getYear() != -1 ? pfDateFormatter
					.getYear() : 0);
			int hour = (pfDateFormatter.getHours() != -1 ? pfDateFormatter
					.getHours() : 0);
			int minute = (pfDateFormatter.getMinutes() != -1 ? pfDateFormatter
					.getMinutes() : 0);
			int second = (pfDateFormatter.getSeconds() != -1 ? pfDateFormatter
					.getSeconds() : 0);
			// int nano = (pfDateFormatter.getNanos() != 0 ?
			// pfDateFormatter.getNanos() : 0);
			int nano = (pfDateFormatter.getNanos() != 0 ? pfDateFormatter
					.getNanos() * 1000000 : 0);
			result = new Timestamp(year - 1900, month - 1, day, hour, minute,
					second, nano);
		} catch (ParseException pe) {
		}
		return result;
	}

	public final static String getFileFromResource(String resource) {
		URL curl;
		if (resource != null)
			curl = Tools.class.getResource(resource);
		else
			return null;

		if (curl == null)
			return null;
		String cfile = curl.getPath();
		if (System.getProperty("os.name").startsWith("W"))
			cfile = cfile.substring(1);
		return cfile;
	}

	/**
	 * read queryFile and store contents in string SQLquery useful for small
	 * files only because store file content in a byte array and return a String
	 * content of file
	 */
	public final static String readQueryFileISO(String qFile) throws Exception {

		URL qurl = Tools.class.getResource(qFile);
		String queryFile = qurl.getPath();
		if (System.getProperty("os.name").startsWith("W"))
			queryFile = queryFile.substring(1);

		File f = null;
		FileInputStream in;
		f = new File(queryFile);
		in = new FileInputStream(f);
		// allocate a byte array to store content data
		Long l = new Long(f.length());
		int size = l.intValue();

		byte[] content = new byte[size];
		int byteread = 0;
		for (int i = 0; i < size; i++) {
			byteread = in.read();
			if (byteread != -1) {
				byte b = (byte) byteread;
				if ((b == 13) | (b == 10))
					content[i] = 0x20;
				else
					content[i] = (byte) byteread;
			}
		}
		String stmt = new String(content, "ISO-8859-1");
		in.close();
		return stmt;
	}

	/**
	 * vettore di bean scritto su file
	 * 
	 * @param vb
	 * @param filename
	 * @throws Exception
	 */
	public final static void vectorToFile(Vector vb, String filename)
			throws Exception {
		FileOutputStream fos = new FileOutputStream(filename);
		for (int i = 0; i < vb.size(); i++) {
			Object o = vb.elementAt(i);
			String s = beanToString(o);
			byte[] sb = s.getBytes();
			fos.write(sb);
			fos.write(CRLF);
		}
		fos.close();
	}

	/**
	 * vettore di bean letto da file
	 * 
	 * @param vb
	 * @param filename
	 * @throws Exception
	 */
	public final static Vector fileToVector(String filename, Class bean)
			throws Exception {
		BufferedReader rdr = new BufferedReader(new FileReader(filename));
		Vector vb = new Vector();
		String line;
		while ((line = rdr.readLine()) != null) {
			Object inst = bean.newInstance();
			stringToBean(line, inst);
			vb.add(inst);
		}
		rdr.close();
		return vb;
	}

	/**
	 * trasforma un bean passato come argomento in una stringa ritornata come
	 * risultato
	 * 
	 * @param o:
	 *            istanza dell'oggetto bean
	 * @return
	 * @throws Exception
	 */
	public final static String beanToString(Object o) throws Exception {
		String s = new String("");
		Method setMethod = null;
		Class cl = o.getClass();
		String methods[] = getOrderedMethods(cl);

		for (int i = 0; i < methods.length; i++) {
			if (methods[i].startsWith("get")
					& !(methods[i].equalsIgnoreCase("getClass"))) {
				try {
					Method m = cl.getMethod(methods[i], null);
					Class type = m.getReturnType();
					if (type.getName().equalsIgnoreCase("java.lang.String")) {
						String f = (String) m.invoke(o, null);
						if (f == null)
							f = new String(" ");
						s += f.trim() + " ;";
					} else {
						Object f = m.invoke(o, null);
						if (f == null)
							s += " ;";
						else
							s += f.toString().trim() + " ;";
					}
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
					throw new Exception("Argomento non previsto.");
				} catch (IllegalAccessException e) {
					e.printStackTrace();
					throw new Exception("Metodo get.. non accessibile.");
				} catch (InvocationTargetException e) {
					e.printStackTrace();
					throw new Exception("Errore di invocazione del metodo.");
				}
			}
		}
		return s;
	}

	/**
	 * dalla stringa popolo un bean e' l'inverso del metodo precedente
	 * 
	 * @param s
	 * @param o
	 * @throws Exception
	 */
	public final static void stringToBean(String s, Object o) throws Exception {
		Method setMethod = null;
		Class cl = o.getClass();
		String methods[] = getOrderedMethods(cl);
		Method[] mo = cl.getMethods();
		StringTokenizer st = new StringTokenizer(s, ";");
		for (int i = 0; i < methods.length; i++) {
			if (methods[i].startsWith("set")) {
				try {
					Method m = getMethodFromName(methods[i], mo);
					Class[] argtypes = m.getParameterTypes();
					Class argtype = argtypes[0];
					String type = argtype.getName();
					String d = st.nextToken();
					if (type.equalsIgnoreCase("int")) {
						Integer f = new Integer(d);
						Object args[] = new Object[1];
						args[0] = f;
						m.invoke(o, args);
					} else if (type.equalsIgnoreCase("long")) {
						Long f = new Long(d);
						Object args[] = new Object[1];
						args[0] = f;
						m.invoke(o, args);
					} else if (type.equalsIgnoreCase("float")) {
						Float f = new Float(d);
						Object args[] = new Object[1];
						args[0] = f;
						m.invoke(o, args);
					} else if (type.equalsIgnoreCase("double")) {
						Double f = new Double(d);
						Object args[] = new Object[1];
						args[0] = f;
						m.invoke(o, args);
					} else {
						Object args[] = new Object[1];
						Object[] dv = new Object[1];
						dv[0] = d;
						Class x = Class.forName(argtype.getName());
						Class cc = Class.forName("java.lang.String");
						Class[] ccv = new Class[1];
						ccv[0] = cc;
						Constructor c = x.getConstructor(ccv);
						String sx = (String) c.newInstance(dv);
						args[0] = sx.trim();
						// args[0] = c.newInstance(dv);
						m.invoke(o, args);
					}
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
					throw new Exception("Argomento non previsto.");
				} catch (IllegalAccessException e) {
					e.printStackTrace();
					throw new Exception("Metodo get.. non accessibile.");
				} catch (InvocationTargetException e) {
					e.printStackTrace();
					throw new Exception("Errore di invocazione del metodo.");
				}
			}
		}
		return;
	}

	/**
	 * data una classe ritorna un array di metodi ordinati
	 * 
	 * @param cl
	 * @return
	 */
	private static String[] getOrderedMethods(Class cl) {
		Method meth[] = cl.getMethods();
		String methodnames[] = new String[meth.length];
		Vector vm = new Vector(meth.length);
		for (int i = 0; i < meth.length; i++) {
			vm.add(meth[i].getName());
		}
		Collections.sort((List) vm);
		for (int i = 0; i < meth.length; i++) {
			methodnames[i] = (String) vm.elementAt(i);
		}
		return methodnames;
	}

	/**
	 * ricerca il metodo con il nome richiesto all'interno dell'array di metodi
	 * passato come secondo parametro
	 * 
	 * @param name
	 * @param mv
	 * @return
	 */
	private static Method getMethodFromName(String name, Method[] mv) {
		for (int i = 0; i < mv.length; i++) {
			if (name.equalsIgnoreCase(mv[i].getName()))
				return mv[i];
		}
		return null;

	}

	public static byte[] Alfabeto = { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06,
			0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10, 0x11,
			0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C,
			0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27,
			0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32,
			0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D,
			0x3E, 0x3F, 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
			0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53,
			0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E,
			0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
			0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74,
			0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F,
			(byte) 0x80, (byte) 0x81, (byte) 0x82, (byte) 0x83, (byte) 0x84,
			(byte) 0x85, (byte) 0x86, (byte) 0x87, (byte) 0x88, (byte) 0x89,
			(byte) 0x8A, (byte) 0x8B, (byte) 0x8C, (byte) 0x8D, (byte) 0x8E,
			(byte) 0x8F, (byte) 0x90, (byte) 0x91, (byte) 0x92, (byte) 0x93,
			(byte) 0x94, (byte) 0x95, (byte) 0x96, (byte) 0x97, (byte) 0x98,
			(byte) 0x99, (byte) 0x9A, (byte) 0x9B, (byte) 0x9C, (byte) 0x9D,
			(byte) 0x9E, (byte) 0x9F, (byte) 0xA0, (byte) 0xA1, (byte) 0xA2,
			(byte) 0xA3, (byte) 0xA4, (byte) 0xA5, (byte) 0xA6, (byte) 0xA7,
			(byte) 0xA8, (byte) 0xA9, (byte) 0xAA, (byte) 0xAB, (byte) 0xAC,
			(byte) 0xAD, (byte) 0xAE, (byte) 0xAF, (byte) 0xB0, (byte) 0xB1,
			(byte) 0xB2, (byte) 0xB3, (byte) 0xB4, (byte) 0xB5, (byte) 0xB6,
			(byte) 0xB7, (byte) 0xB8, (byte) 0xB9, (byte) 0xBA, (byte) 0xBB,
			(byte) 0xBC, (byte) 0xBD, (byte) 0xBE, (byte) 0xBF, (byte) 0xC0,
			(byte) 0xC1, (byte) 0xC2, (byte) 0xC3, (byte) 0xC4, (byte) 0xC5,
			(byte) 0xC6, (byte) 0xC7, (byte) 0xC8, (byte) 0xC9, (byte) 0xCA,
			(byte) 0xCB, (byte) 0xCC, (byte) 0xCD, (byte) 0xCE, (byte) 0xCF,
			(byte) 0xD0, (byte) 0xD1, (byte) 0xD2, (byte) 0xD3, (byte) 0xD4,
			(byte) 0xD5, (byte) 0xD6, (byte) 0xD7, (byte) 0xD8, (byte) 0xD9,
			(byte) 0xDA, (byte) 0xDB, (byte) 0xDC, (byte) 0xDD, (byte) 0xDE,
			(byte) 0xDF, (byte) 0xE0, (byte) 0xE1, (byte) 0xE2, (byte) 0xE3,
			(byte) 0xE4, (byte) 0xE5, (byte) 0xE6, (byte) 0xE7, (byte) 0xE8,
			(byte) 0xE9, (byte) 0xEA, (byte) 0xEB, (byte) 0xEC, (byte) 0xED,
			(byte) 0xEE, (byte) 0xEF, (byte) 0xF0, (byte) 0xF1, (byte) 0xF2,
			(byte) 0xF3, (byte) 0xF4, (byte) 0xF5, (byte) 0xF6, (byte) 0xF7,
			(byte) 0xF8, (byte) 0xF9, (byte) 0xFA, (byte) 0xFB, (byte) 0xFC,
			(byte) 0xFD, (byte) 0xFE, (byte) 0xFF };

	/**
	 * Decode a secret encrypted with the basic base64 scheme.
	 * 
	 * @param secret
	 *            Message to decode
	 * @return null on error, decoded String on success
	 */
	public static String DecodeBase64(byte[] secret) {
		StringBuffer output = new StringBuffer();
		boolean inalphabet[] = new boolean[256];
		char decoder[] = new char[256];

		for (int i = 0; i < 256; i++) {
			inalphabet[i] = false;
			decoder[i] = 0;
		}

		for (int i = Alfabeto.length - 1; i >= 0; i--) {
			inalphabet[(int) Alfabeto[i]] = true;
			decoder[(int) Alfabeto[i]] = (char) i;
		}

		int inidx = 0;
		int c = 0;
		int char_count = 0;
		int bits = 0;
		boolean error = false;

		while (inidx < secret.length) {
			c = secret[inidx++];

			if (c == '=')
				break;

			if (c > 255 || !inalphabet[(int) c])
				continue;

			bits += decoder[c];
			char_count++;
			if (char_count == 4) {
				output.append((char) (bits >> 16));
				output.append((char) ((bits >> 8) & 0xff));
				output.append((char) (bits & 0xff));
				bits = 0;
				char_count = 0;
			} else
				bits <<= 6;
		}

		if (c == '\0') {
			if (char_count != 0) {
				System.out.println("base64 encoding incomplete: at least "
						+ ((4 - char_count) * 6) + " bits truncated");
				error = true;
			}
		} else {
			// c == '='
			switch (char_count) {
			case 1:
				System.out.println("output so far: " + output.toString());
				System.out
						.println("base64 encoding incomplete: at least 2 bits missing");
				error = true;
				break;
			case 2:
				output.append((char) (bits >> 10));
				break;
			case 3:
				output.append((char) (bits >> 16));
				output.append((char) ((bits >> 8) & 0xff));
				break;
			}
		}

		if (!error)
			return output.toString();
		else
			return null;
	}

	/**
	 * Encode a message with the basic base64 scheme.
	 * 
	 * @param secret
	 *            Message to encode
	 * @return null on error, an encoded String on success
	 */
	public static String EncodeBase64(byte[] secret) {
		StringBuffer output = new StringBuffer();

		// Index of the input string
		int inidx = 0;

		// character value
		int c;

		int char_count = 0;
		int bits = 0;
		boolean error = false;

		while (inidx < secret.length) {
			c = secret[inidx++];

			// This will mess up internationalization. I wonder if it is really
			// necessary for HTTP?
			if (c > 255) {
				System.out.println("encountered char > 255 (decimal %d): " + c);
				error = true;
				break;
			}

			bits += c;
			char_count++;

			if (char_count == 3) {
				output.append(Alfabeto[(bits >> 18)]);
				output.append(Alfabeto[(bits >> 12) & 0x3f]);
				output.append(Alfabeto[(bits >> 6) & 0x3f]);
				output.append(Alfabeto[(bits & 0x3f)]);

				bits = 0;
				char_count = 0;
			} else
				bits <<= 8;
		}

		if (!error) {
			if (char_count != 0) {
				bits <<= 16 - (8 * char_count);
				output.append(Alfabeto[(bits >> 18)]);
				output.append(Alfabeto[(bits >> 12) & 0x3f]);

				if (char_count == 1) {
					output.append("==");
				} else {
					output.append(Alfabeto[(bits >> 6) & 0x3f]);
					output.append('=');
				}
			}

			return output.toString();
		}

		// Returns null on error
		return null;
	}

	/**
	 * Alphabet used in encoding and decoding basic base64 authentication. See
	 * the HTTP 1.1 RFC for details.
	 */
	public static String Alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

	/**
	 * Decode a secret encrypted with the basic base64 HTTP 1.1 authentication
	 * scheme.
	 * 
	 * @param secret
	 *            Message to decode
	 * @return null on error, decoded String on success
	 */
	public static String HTTPDecodeBase64(String secret) {
		StringBuffer output = new StringBuffer();
		boolean inalphabet[] = new boolean[256];
		char decoder[] = new char[256];

		for (int i = 0; i < 256; i++) {
			inalphabet[i] = false;
			decoder[i] = 0;
		}

		for (int i = Alphabet.length() - 1; i >= 0; i--) {
			inalphabet[(int) Alphabet.charAt(i)] = true;
			decoder[(int) Alphabet.charAt(i)] = (char) i;
		}

		int inidx = 0;
		int c = 0;
		int char_count = 0;
		int bits = 0;
		boolean error = false;

		while (inidx < secret.length()) {
			c = secret.charAt(inidx++);

			if (c == '=')
				break;

			if (c > 255 || !inalphabet[(int) c])
				continue;

			bits += decoder[c];
			char_count++;
			if (char_count == 4) {
				output.append((char) (bits >> 16));
				output.append((char) ((bits >> 8) & 0xff));
				output.append((char) (bits & 0xff));
				bits = 0;
				char_count = 0;
			} else
				bits <<= 6;
		}

		if (c == '\0') {
			if (char_count != 0) {
				System.out.println("base64 encoding incomplete: at least "
						+ ((4 - char_count) * 6) + " bits truncated");
				error = true;
			}
		} else {
			// c == '='
			switch (char_count) {
			case 1:
				System.out.println("output so far: " + output.toString());
				System.out
						.println("base64 encoding incomplete: at least 2 bits missing");
				error = true;
				break;
			case 2:
				output.append((char) (bits >> 10));
				break;
			case 3:
				output.append((char) (bits >> 16));
				output.append((char) ((bits >> 8) & 0xff));
				break;
			}
		}

		if (!error)
			return output.toString();
		else
			return null;
	}

	/**
	 * Encode a message with the basic base64 HTTP 1.1 authentication scheme.
	 * Adapted from James' JAWS HTTP_Helpers code.
	 * 
	 * @param secret
	 *            Message to encode
	 * @return null on error, an encoded String on success
	 */
	public static String HTTPEncodeBase64(String secret) {
		StringBuffer output = new StringBuffer();

		// Index of the input string
		int inidx = 0;

		// character value
		int c;

		int char_count = 0;
		int bits = 0;
		boolean error = false;

		while (inidx < secret.length()) {
			c = secret.charAt(inidx++);

			// This will mess up internationalization. I wonder if it is really
			// necessary for HTTP?
			if (c > 255) {
				System.out.println("encountered char > 255 (decimal %d): " + c);
				error = true;
				break;
			}

			bits += c;
			char_count++;

			if (char_count == 3) {
				output.append(Alphabet.charAt(bits >> 18));
				output.append(Alphabet.charAt((bits >> 12) & 0x3f));
				output.append(Alphabet.charAt((bits >> 6) & 0x3f));
				output.append(Alphabet.charAt(bits & 0x3f));

				bits = 0;
				char_count = 0;
			} else
				bits <<= 8;
		}

		if (!error) {
			if (char_count != 0) {
				bits <<= 16 - (8 * char_count);
				output.append(Alphabet.charAt(bits >> 18));
				output.append(Alphabet.charAt((bits >> 12) & 0x3f));

				if (char_count == 1) {
					output.append("==");
				} else {
					output.append(Alphabet.charAt((bits >> 6) & 0x3f));
					output.append('=');
				}
			}

			return output.toString();
		}

		// Returns null on error
		return null;
	}

	/**
	 * ritorna vero se il numero � pari falso altrimenti
	 * 
	 * @return boolean
	 * @param i
	 *            int
	 */
	public static final boolean isEven(int i) {
		float f = i;
		float meta = java.lang.Math.round(f / 2);
		float diff = (f / 2) - meta;
		if (diff != 0)
			return false;
		else
			return true;
	}

	/**
	 * ritorna vero se il numero � dispari falso altrimenti � sufficiente negare
	 * il risultato di isEven.
	 * 
	 * @return boolean
	 * @param i
	 *            int
	 */
	public static final boolean isOdd(int i) {
		boolean result = isEven(i);
		return !(result);
	}

	/**
	 * genera una stringa come l'argomento, per la quale i caratteri ' @ " !
	 * sono resi nella corrispondente sequenza http: %27 %40 %22 %21 (http
	 * encoded).
	 */
	public static final String encodeHttp(String toEncode) {
		URIUtil u = new URIUtil();
		BitSet bs = new BitSet(256);
		bs.clear(); // all false
		bs.flip(0, 255); // all true: allow all characters not to be escaped
		bs.flip(39); // ' false: this char must be escaped
		bs.flip(64); // @ false: this char must be escaped
		bs.flip(34); // " false: this char must be escaped
		bs.flip(33); // ! false: this char must be escaped
		String encoded = null;
		try {
			encoded = u.encode(toEncode, bs);
			encoded = encoded.replace('+', ' ');
		} catch (URIException e) {
			e.printStackTrace();
			return "";
		}
		return encoded;
	}

	/**
	 * rigenera la stringa originale trasformando gli encode http nei caratteri
	 * originali
	 */
	public static final String decodeHttp(String toDecode) {
		URIUtil u = new URIUtil();
		try {
			return u.decode(toDecode);
		} catch (URIException e) {
			e.printStackTrace();
			return "";
		}
	}

	/**
	 * Verifica la ricorrenza di contained all'interno di container
	 * 
	 * @param container
	 * @param contained
	 * @return
	 */
	public static final boolean stringContains(String container,
			String contained) {
		String t = container;
		while (t.length() >= contained.length()) {
			if (t.startsWith(contained)) {
				return true;
			} else {
				t = t.substring(1);
			}
		}
		return false;
	}


	/**
	 * mappa un DataRow su un bean adeguato
	 * @param dr
	 * @param bean
	 * @throws Exception
	 */
	public static void mapDRtoBean(DataRow dr, Object bean) throws Exception {
		Class beanClass = null;
		try {
			beanClass = (Class) Class.forName(bean.getClass().getName());
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
		Method setMethod = null;
		Field[] fields = beanClass.getDeclaredFields();
		Method methods[] = beanClass.getMethods();

		for (int i = 0; i < fields.length; i++) {
			String fieldname = getCapitalized(fields[i].getName());
			//String fieldname = fields[i].getName().toUpperCase();
			DataField df = dr.getColumn(fieldname);
			if (df != null) {
				Object fieldvalue = dr.getColumn(fieldname).getObjectValue();
				try {
					Method method = getMethod(methods, "set" + fieldname.trim());
					if (method != null) {
						if (setInvoke(bean, method, fieldvalue) < 0)
							throw new Exception(
									"Errore in popolamento bean");
					}
				} catch (Exception e) {
					System.out.println("ERRORE in storeResult:");
					System.out.println(e.getMessage());
					throw new Exception(
							"I/O error in lettura input:" + e.getMessage());
				}
			}
		}

	}

	protected static String getCapitalized(String s) {
		if (s.length() == 0)
			return s;
		if (s.length() == 1)
			return s.toUpperCase();
		String capitalized = s.substring(1);
		String fchar = s.substring(0, 1).toUpperCase();
		return fchar+capitalized;
	}

	/*
	 * iterate in the method array to match the one with name equal 2-nd param
	 * return method or null if check fails
	 */
	protected static Method getMethod(Method[] m, String match) {
		for (int i = 0; i < m.length; i++) {
			String methodName = m[i].getName();
			if (methodName.equals(match))
				return m[i];
		}
		return null;
	}

	/**
	 * invoke set method on bean
	 */
	protected static int setInvoke(Object beanInst, java.lang.reflect.Method setter,
			Object value) {

		Object args[] = new Object[1];
		args[0] = value;

		try {
			setter.invoke(beanInst, args);
		} catch (java.lang.reflect.InvocationTargetException e) {
			System.out.println("java.lang.reflect.InvocationTargetException"
					+ e.getMessage());
			e.printStackTrace();
			return -1;
		} catch (IllegalAccessException es) {
			System.out.println("IllegalAccessException" + es.getMessage());
			es.printStackTrace();
			return -1;
		} catch (IllegalArgumentException es) {
			System.out.println("IllegalArgumentException" + es.getMessage());
			es.printStackTrace();
			return -1;
		}
		return 0;
	}
	
	public final static boolean classHasMethod(Class clazz, String method, Object[] args) {
		Method methods[] = clazz.getMethods();
		for (int i = 0; i < methods.length; i++) {
			String methodName = methods[i].getName();
			if (methodName.equals(method)) {
				
			}
		}
		return false;
	}

}
