package it.pf.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

public class ZipUtil {

	public ZipUtil() {
	}

	/**
	 * build a zip file from source directory
	 * @param source
	 * @param destination
	 * @return
	 */
	public boolean implodeZip(String source, String destination) {
		try {
			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(
				destination));
			// create a new File object based on the directory we
			// have to zip File
			File zipdir = new File(source);
			// get a listing of the directory content
			String[] dirList = zipdir.list();
			byte[] readBuffer = new byte[2156];
			int bytesIn = 0;
			// loop through dirList, and zip the files
			for (int i = 0; i < dirList.length; i++) {
				File f = new File(zipdir, dirList[i]);
				if (f.isDirectory()) {
					// if the File object is a directory, call this
					// function again to add its content recursively
					String filePath = f.getPath();
					implodeZip(filePath, destination);
					// loop again
					continue;
				}
				// if we reached here, the File object f was not a directory
				// create a FileInputStream on top of f
				FileInputStream fis = new FileInputStream(f);
				// create a new zip entry
				ZipEntry anEntry = new ZipEntry(f.getPath());
				// place the zip entry in the ZipOutputStream object
				zos.putNextEntry(anEntry);
				// now write the content of the file to the ZipOutputStream
				while ((bytesIn = fis.read(readBuffer)) != -1) {
					zos.write(readBuffer, 0, bytesIn);
				}
				// close the Stream
				fis.close();
			}
			zos.close();
			return true;
		} catch (Exception e) {
			// handle exception
			return false;
		}
	}

	/**
     * Unzip file 
     * 1-st parameter = full path name of the zip file
     * 2-nd parameter = destination directory for unzipped files and directories
     */
	public boolean exlpodeZip(String source, String destination) {
		try {
			// Open the ZIP file
			ZipInputStream in = new ZipInputStream(new FileInputStream(source));

			// Get the first entry
			ZipEntry entry = in.getNextEntry();
			while (entry != null) {
				// Open the output file
				if (entry.isDirectory()) {
					new File(destination + "/" + entry.getName()).mkdir();
				} else {
					//OutputStream out = new FileOutputStream(destination + "/" + entry.getName());

					OutputStream out = new FileOutputStream(entry.getName());
					// Transfer bytes from the ZIP file to the output file
					byte[] buf = new byte[1024];
					int len;
					while ((len = in.read(buf)) > 0) {
						out.write(buf, 0, len);
					}

					// Close the streams
					out.close();
				}
				entry = in.getNextEntry();
			}
			in.close();
			return true;
		} catch (IOException e) {
			return false;
		}
	}

	/**
     * Unzip file 
     * 1-st parameter = full path name of the zip file
     * 2-nd parameter = destination directory for unzipped files and directories
    private boolean processZipFile(String filename, String p) {
		byte[] cbuf = new byte[8192];
		int count = 0, max = 8192;
		InputStream in = null;
		OutputStream out = null;
		try {
			ZipFile zf = new ZipFile(filename);
			Enumeration l = zf.entries();
			while (l.hasMoreElements()) {
				ZipEntry ze = new ZipEntry(l.nextElement().toString());
				if (!ze.isDirectory()) {
					String tmp = ze.getName();
					String fn = null;
					int pb = tmp.lastIndexOf("/");
					if (pb < 0)
						pb = tmp.lastIndexOf("\\");
					if (pb >= 0) {
						pb++;
						fn = tmp.substring(pb);
					} else {
						fn = tmp;
					}
					String path = p+"/"+fn;
					in = zf.getInputStream(ze);
					out = new FileOutputStream(path);
					while ((count = in.read(cbuf,0,max)) != -1)
						out.write(cbuf, 0, count);
						out.write(cbuf,0,count);
					out.close();
					in.close();
				}
			}
			zf.close();
			return true;
		} catch(Exception ze) {
			return false;
		}
    }
    */

    /**
     * Return contents of the zip file in Enumeration
     * 1-st parameter = full path name of the zip file
     */
    private Enumeration listZipFile(String filename) throws IOException {
		try {
			ZipFile zf = new ZipFile(filename);
			Enumeration l = zf.entries();
			zf.close();
			return l;
		} catch(java.util.zip.ZipException ze) {
			return null;
		}
    }

}
